;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: MIDIimgdata.pro,v 1.2 2003/09/09 09:56:17 koehler Exp $
;
; MIDIimgdata.pro
; Created:     Sat Aug  2 17:04:20 2003 by Koehler@rommie
; Last change: Fri Aug 29 12:03:18 2003
;
; PURPOSE:
;	transparent reading of multi-file midi-datasets
;
; CALLING SEQUENCE:
;	i = obj_new('MIDIimgdata',file,IERR=ierr)
;
; INPUT:
;	file	space-separated string of filenames
;	ierr	used to return error status
;
; OUTPUT:
;	an object
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO MIDIimgdata__Define
d = { MIDIimgdata, $
      filelist: ptr_new(), $
      nframes:  ptr_new(), $
      open_file: obj_new('imagedata'), $
      open_no:   -1 $
}
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function MIDIimgdata::init, file, IERR=ierr

  self.filelist= ptr_new( oirFileList(file[0]))
  nFiles = N_elements(*self.filelist)
  self.nframes = ptr_new( intarr(nFiles))
  self.open_no = -1

  for iFile= 0, nFiles-1 do begin
      imgObj = obj_new('imagedata', (*self.filelist)[iFile], ierr=ierr)
      (*self.nframes)[iFile] = (imgObj->naxis())[1]
      print,"File ",(*self.filelist)[iFile]," has",(*self.nframes)[iFile]," rows"
      obj_destroy,imgObj
  endfor
  return,1
end

PRO MIDIimgdata::cleanup
  print,"MIDIimgdata cleaning up..."
  if (self.open_no ge 0) then obj_destroy, self.open_file
  ptr_free, self.filelist
  ptr_free, self.nframes
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function MIDIimgdata::readRowsf, wantRow, IERR=ierr
  iFile= 0
  iRow = wantRow
  nFiles= N_elements(*self.filelist)
  while (iRow gt (*self.nframes)[iFile] and iFile lt nFiles) do begin
      ;print,Format='(I5,"...",$)',iRow
      ; iRow is not in file iFile, but later
      iRow = iRow - (*self.nframes)[iFile]
      iFile= iFile + 1
  endwhile
  ;print,iRow
  ; is iRow in one of our files?
  if (iFile ge nFiles) then return, ptr_new()

  ; do we have to open the file first?
  if (iFile ne self.open_no) then begin
      if (self.open_no ge 0) then obj_destroy, self.open_file
      ;print, "User wants row",wantRow," => file",iFile," row",iRow
      print, "MIDIimgdata opening file no",iFile
      print, (*self.filelist)[iFile]
      self.open_file= obj_new('imagedata', (*self.filelist)[iFile], ierr=ierr)
      self.open_no  = iFile
  endif

  return, self.open_file->readRowsf(iRow, IERR=ierr)
end
