;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: calibrators.pro,v 1.2 2003/09/09 09:56:17 koehler Exp $
;
; calibrators.pro
; Created:     Fri Aug  8 11:47:55 2003 by Koehler@sun47
; Last change: Fri Aug 29 12:03:56 2003
;
; PURPOSE:
;	provide a lookup table with diameters of calibrators
;	based on e-mail from Ch.Leinert on Fri May 23 11:13:25 2003
;
; CALLING SEQUENCE:
;	c = calibrator, "HD4711"
;
; INPUT:
;	name of calibrator, preferentially as HD-number
;
; OUTPUT:
;	struct containing name, pos, diam, and errdiam
;
; STUPIDITIES:
;	You can't have strings in IDL that start with a digit
;
Function calibrator, name

  caldb = [ $
	    { name:"DELTA_SGR_CAL",pos:" 18 20 59.64  -29 49 41.2",diam: 5.78, errdiam: 0.15 },$ ;HD168454
	    { name:"DELTA_SGR", pos:" 18 20 59.64  -29 49 41.2",diam: 5.78, errdiam: 0.15 },$ ;HD168454
	    { name:"DELSGR",	pos:" 18 20 59.64  -29 49 41.2",diam: 5.78, errdiam: 0.15 },$ ;HD168454
	    { name:"HR6257_CAL",pos:" 16 53 42.42  -43 03  3.5",diam: 4.83, errdiam: 0.5  },$ ;HD152161
	    { name:"ALFAQL",	pos:" 19 50 47.00  +08 52  6.0",diam: 3.22, errdiam: 0.01 },$ ;HD187642
            { name:"HD402"   , pos:" 00 08 33.50  -17 34 41.4", diam: 2.16, errdiam: 0.07 },$
	    { name:"HD496"   , pos:" 00 09 24.64  -45 44 50.7", diam: 2.17, errdiam: 0.06 },$
	    { name:"HD598"   , pos:" 00 10 26.76  +28 39 10.4", diam: 2.56, errdiam: 0.11 },$
	    { name:"HD1014"  , pos:" 00 14 27.63  -07 46 49.9", diam: 4.49, errdiam: 0.07 },$
	    { name:"HD1038"  , pos:" 00 14 38.42  -18 55 58.3", diam: 4.65, errdiam: 0.07 },$
	    { name:"HD1522"  , pos:" 00 19 25.67  -08 49 26.1", diam: 3.21, errdiam: 0.07 },$
	    { name:"HD1632"  , pos:" 00 20 45.52  +32 54 40.3", diam: 2.36, errdiam: 0.16 },$
	    { name:"HD2024"  , pos:" 00 24 32.90  -04 38 52.0", diam: 2.21, errdiam: 0.07 },$
	    { name:"HD2151"  , pos:" 00 25 45.07  -77 15 15.3", diam: 2.14, errdiam: 0.01 },$
	    { name:"HD3627"  , pos:" 00 39 19.68  +30 51 39.7", diam: 4.  , errdiam: 0.04 },$
	    { name:"HD4128"  , pos:" 00 43 35.37  -17 59 11.8", diam: 5.04, errdiam: 0.07 },$
	    { name:"HD4502"  , pos:" 00 47 20.33  +24 16  1.8", diam: 2.31, errdiam: 0.01 },$
	    { name:"HD5112"  , pos:" 00 53 00.49  -01 08 39.3", diam: 3.16, errdiam: 0.07 },$
	    { name:"HD5437"  , pos:" 00 56 01.49  -11 15 59.5", diam: 2.38, errdiam: 0.07 },$
	    { name:"HD6311"  , pos:" 01 02 42.91  -65 27 22.0", diam: 2.78, errdiam: 0.09 },$
	    { name:"HD6595"  , pos:" 01 06 05.05  -46 43  6.3", diam: 2.41, errdiam: 0.06 },$
	    { name:"HD6805"  , pos:" 01 08 35.39  -10 10 56.2", diam: 3.2 , errdiam: 0.07 },$
	    { name:"HD8498"  , pos:" 01 23 30.96  -30 56 44.2", diam: 2.39, errdiam: 0.07 },$
	    { name:"HD9053"  , pos:" 01 28 21.93  -43 19  5.6", diam: 5.91, errdiam: 0.07 },$
	    { name:"HD9138"  , pos:" 01 30 11.12  +06 08 37.8", diam: 2.43, errdiam: 0.06 },$
	    { name:"HD9362"  , pos:" 01 31 15.10  -49 04 21.7", diam: 2.1 , errdiam: 0.05 },$
	    { name:"HD10052" , pos:" 01 36 44.80  -58 16 15.5", diam: 2.91, errdiam: 0.16 },$
	    { name:"HD10144" , pos:" 01 37 42.85  -57 14 12.3", diam: 1.62, errdiam: 0.08 },$
	    { name:"HD10380" , pos:" 01 41 25.89  +05 29 15.4", diam: 2.85, errdiam: 0.06 },$
	    { name:"HD10550" , pos:" 01 42 43.51  -03 41 24.7", diam: 2.04, errdiam: 0.07 },$
	    { name:"HD10700" , pos:" 01 44 04.08  -15 56 14.9", diam: 1.94, errdiam: 0.02 },$
	    { name:"HD10824" , pos:" 01 45 59.26  -05 43 59.9", diam: 2.35, errdiam: 0.07 },$
	    { name:"HD11353" , pos:" 01 51 27.63  -10 20  6.1", diam: 2.7 , errdiam: 0.07 },$
	    { name:"HD11937" , pos:" 01 55 57.47  -51 36 32.0", diam: 2.01, errdiam: 0.02 },$
	    { name:"HD11961" , pos:" 01 58 03.77  +31 08  3.8", diam: 5.41, errdiam: 0.11 },$
	    { name:"HD12076" , pos:" 01 58 27.68  -07 04 42.1", diam: 2.18, errdiam: 0.07 },$
	    { name:"HD12524" , pos:" 02 01 42.38  -44 42 48.6", diam: 2.53, errdiam: 0.07 },$
	    { name:"HD13325" , pos:" 02 10 37.60  +19 30  1.2", diam: 3.69, errdiam: 0.26 },$
	    { name:"HD13596" , pos:" 02 13 03.31  +15 16 47.5", diam: 2.17, errdiam: 0.16 },$
	    { name:"HD14512" , pos:" 02 21 14.49  +23 25 35.8", diam: 2.69, errdiam: 0.28 },$
	    { name:"HD16815" , pos:" 02 40 40.03  -39 51 19.4", diam: 2.21, errdiam: 0.04 },$
	    { name:"HD17709" , pos:" 02 51 30.84  +35 03 35.1", diam: 3.81, errdiam: 0.09 },$
	    { name:"HD18322" , pos:" 02 56 25.65  -08 53 53.3", diam: 2.4 , errdiam: 0.04 },$
	    { name:"HD18700" , pos:" 03 00 44.14  +10 52 13.4", diam: 2.18, errdiam: 0.05 },$
	    { name:"HD18884" , pos:" 03 02 16.77  +04 05 23.0", diam:11.02, errdiam: 0.04 },$
	    { name:"HD20720" , pos:" 03 19 31.00  -21 45 28.3", diam:10.03, errdiam: 0.1  },$
	    { name:"HD20644" , pos:" 03 20 20.36  +29 02 54.5", diam: 3.62, errdiam: 0.15 },$
	    { name:"HD21120" , pos:" 03 24 48.79  +09 01 43.9", diam: 2.16, errdiam: 0.01 },$
	    { name:"HD21754" , pos:" 03 30 52.38  +12 56 12.0", diam: 2.18, errdiam: 0.03 },$
	    { name:"HD23319" , pos:" 03 42 50.06  -37 18 48.7", diam: 1.9 , errdiam: 0.08 },$
	    { name:"HD23249" , pos:" 03 43 14.90  -09 45 48.2", diam: 2.33, errdiam: 0.01 },$
	    { name:"HD23817" , pos:" 03 44 11.98  -64 48 24.9", diam: 2.58, errdiam: 0.04 },$
	    { name:"HD25470" , pos:" 04 00 15.53  -51 33 53.9", diam: 2.24, errdiam: 0.14 },$
	    { name:"HD25605" , pos:" 04 04 18.76  +12 30 26.8", diam: 2.4 , errdiam: 0.62 },$
	    { name:"HD25604" , pos:" 04 04 41.72  +22 04 54.9", diam: 1.88, errdiam: 0.05 },$
	    { name:"HD26967" , pos:" 04 13 00.11  -42 17 39.7", diam: 2.48, errdiam: 0.06 },$
	    { name:"HD27256" , pos:" 04 14 25.48  -62 28 25.9", diam: 2.44, errdiam: 0.06 },$
	    { name:"HD27697" , pos:" 04 22 56.09  +17 32 33.0", diam: 2.13, errdiam: 0.04 },$
	    { name:"HD28307" , pos:" 04 28 34.50  +15 57 43.9", diam: 1.98, errdiam: 0.06 },$
	    { name:"HD28749" , pos:" 04 31 52.67  -00 02 38.4", diam: 1.96, errdiam: 0.2  },$
	    { name:"HD29065" , pos:" 04 34 11.77  -08 58 12.9", diam: 2.29, errdiam: 0.17 },$
	    { name:"HD29051" , pos:" 04 35 02.37  +17 12  4.4", diam: 2.48, errdiam: 0.37 },$
	    { name:"HD29139" , pos:" 04 35 55.24  +16 30 33.5", diam: 19.9, errdiam: 0.02 },$
	    { name:"HD29503" , pos:" 04 38 10.82  -14 18 14.5", diam: 2.47, errdiam: 0.07 },$
	    { name:"HD30593" , pos:" 04 47 18.92  -36 12 33.6", diam: 1.89, errdiam: 0.14 },$
	    { name:"HD30354" , pos:" 04 47 47.93  +22 08 21.9", diam: 2.15, errdiam: 0.07 },$
	    { name:"HD31009" , pos:" 04 49 04.75  -56 39 59.7", diam: 2.24, errdiam: 0.13 },$
	    { name:"HD31421" , pos:" 04 56 22.27  +13 30 52.1", diam: 2.49, errdiam: 0.06 },$
	    { name:"HD31398" , pos:" 04 56 59.62  +33 09 57.9", diam: 7.15, errdiam: 0.67 },$
	    { name:"HD31767" , pos:" 04 58 32.90  +01 42 50.5", diam: 2.65, errdiam: 0.36 },$
	    { name:"HD32831" , pos:" 05 04 24.40  -35 28 58.7", diam: 2.14, errdiam: 0.07 },$
	    { name:"HD33042" , pos:" 05 04 58.01  -49 34 40.2", diam: 2.65, errdiam: 0.13 },$
	    { name:"HD32683" , pos:" 05 06 27.52  +34 51 18.1", diam: 2.78, errdiam: 0.07 },$
	    { name:"HD33684" , pos:" 05 07 34.03  -63 23 58.8", diam: 5.09, errdiam: 0.13 },$
	    { name:"HD33856" , pos:" 05 13 17.48  +02 51 40.6", diam: 2.05, errdiam: 0.12 },$
	    { name:"HD34334" , pos:" 05 18 10.57  +33 22 17.8", diam: 2.42, errdiam: 0.09 },$
	    { name:"HD35369" , pos:" 05 23 56.83  -07 48 29.0", diam: 1.89, errdiam: 0.04 },$
	    { name:"HD35536" , pos:" 05 25 01.74  -10 19 44.0", diam: 2.05, errdiam: 0.12 },$
	    { name:"HD35620" , pos:" 05 27 38.89  +34 28 33.2", diam: 2.04, errdiam: 0.15 },$
	    { name:"HD36079" , pos:" 05 28 14.72  -20 45 34.0", diam: 2.9 , errdiam: 0.04 },$
	    { name:"HD36597" , pos:" 05 31 12.76  -35 28 13.9", diam: 2.6 , errdiam: 0.09 },$
	    { name:"HD36673" , pos:" 05 32 43.82  -17 49 20.2", diam: 1.54, errdiam: 0.1  },$
	    { name:"HD37350" , pos:" 05 33 37.52  -62 29 23.4", diam: 1.78, errdiam: 0.13 },$
	    { name:"HD37160" , pos:" 05 36 54.39  +09 17 26.4", diam: 2.02, errdiam: 0.02 },$
	    { name:"HD39425" , pos:" 05 50 57.59  -35 46  5.9", diam: 3.68, errdiam: 0.02 },$
	    { name:"HD39045" , pos:" 05 51 25.75  +32 07 28.9", diam: 3.24, errdiam: 0.21 },$
	    { name:"HD39400" , pos:" 05 52 26.44  +01 51 18.5", diam: 2.22, errdiam: 0.22 },$
	    { name:"HD40808" , pos:" 05 59 08.81  -42 48 54.5", diam: 2.44, errdiam: 0.09 },$
	    { name:"HD40657" , pos:" 06 00 03.35  -03 04 27.3", diam: 2.3 , errdiam: 0.08 },$
	    { name:"HD42042" , pos:" 06 07 41.64  -19 09 57.1", diam: 3.33, errdiam: 0.1  },$
	    { name:"HD42474" , pos:" 06 11 56.25  +23 12 25.4", diam: 2.09, errdiam: 0.07 },$
	    { name:"HD42995" , pos:" 06 14 52.66  +22 30 24.5", diam:10.85, errdiam: 0.13 },$
	    { name:"HD43039" , pos:" 06 15 22.69  +29 29 53.1", diam: 1.91, errdiam: 0.04 },$
	    { name:"HD44033" , pos:" 06 20 04.22  +14 39  4.1", diam: 2.4 , errdiam: 0.12 },$
	    { name:"HD45002" , pos:" 06 21 40.74  -56 21 44.3", diam: 2.17, errdiam: 0.23 },$
	    { name:"HD45669" , pos:" 06 22 38.27  -69 59  2.6", diam: 2.11, errdiam: 0.15 },$
	    { name:"HD44478" , pos:" 06 22 57.63  +22 30 48.9", diam: 13.79, errdiam: 0.09 },$
	    { name:"HD45348" , pos:" 06 23 57.11  -52 41 44.4", diam: 6.64, errdiam: 0.11 },$
	    { name:"HD45724" , pos:" 06 29 14.81  +02 38 46.3", diam: 2.04, errdiam: 0.21 },$
	    { name:"HD46612" , pos:" 06 34 35.63  +04 58  4.5", diam: 2.81, errdiam: 0.19 },$
	    { name:"HD47205" , pos:" 06 36 41.04  -19 15 21.2", diam: 2.25, errdiam: 0.01 },$
	    { name:"HD47886" , pos:" 06 41 17.22  +11 00 11.5", diam: 2.31, errdiam: 1.6  },$
	    { name:"HD48217" , pos:" 06 41 56.39  -09 10  3.1", diam: 2.45, errdiam: 0.07 },$
	    { name:"HD48329" , pos:" 06 43 55.93  +25 07 52.0", diam: 0.24, errdiam: 0.16 },$
	    { name:"HD48433" , pos:" 06 43 59.29  +13 13 40.9", diam: 2.02, errdiam: 0.07 },$
	    { name:"HD48915" , pos:" 06 45 08.92  -16 42 58.0", diam: 5.85, errdiam: 0.   },$
	    { name:"HD49161" , pos:" 06 47 19.83  +08 02 14.1", diam: 2.7 , errdiam: 0.12 },$
	    { name:"HD49293" , pos:" 06 47 51.65  +02 24 43.8", diam: 1.88, errdiam: 0.05 },$
	    { name:"HD50778" , pos:" 06 54 11.40  -12 02 19.1", diam: 3.76, errdiam: 0.04 },$
	    { name:"HD51799" , pos:" 06 56 15.99  -48 43 16.1", diam: 3.98, errdiam: 0.12 },$
	    { name:"HD51725" , pos:" 06 58 07.12  -09 01 38.2", diam: 3.16, errdiam: 0.14 },$
	    { name:"HD53047" , pos:" 07 00 51.48  -51 24  9.3", diam: 3.54, errdiam: 0.19 },$
	    { name:"HD52666" , pos:" 07 01 56.38  -05 43 19.4", diam: 2.53, errdiam: 0.21 },$
	    { name:"HD54605" , pos:" 07 08 23.48  -26 23 35.5", diam: 3.67, errdiam: 0.11 },$
	    { name:"HD54300" , pos:" 07 08 42.61  +10 01 26.5", diam: 0.29, errdiam: 0.58 },$
	    { name:"HD55865" , pos:" 07 08 44.87  -70 29 56.1", diam: 2.39, errdiam: 0.04 },$
	    { name:"HD54719" , pos:" 07 11 08.37  +30 14 42.6", diam: 2.57, errdiam: 0.01 },$
	    { name:"HD57890" , pos:" 07 22 23.09  -20 30 24.0", diam: 3.82, errdiam: 0.11 },$
	    { name:"HD58207" , pos:" 07 25 43.60  +27 47 53.1", diam: 2.43, errdiam: 0.07 },$
	    { name:"HD58972" , pos:" 07 28 09.79  +08 55 31.9", diam: 3.31, errdiam: 0.07 },$
	    { name:"HD59381" , pos:" 07 29 22.11  -10 19 36.0", diam: 2.13, errdiam: 0.14 },$
	    { name:"HD60522" , pos:" 07 35 55.35  +26 53 44.7", diam: 3.75, errdiam: 0.07 },$
	    { name:"HD61421" , pos:" 07 39 18.12  +05 13 30.0", diam: 5.17, errdiam: 0.   },$
	    { name:"HD61772" , pos:" 07 40 23.21  -15 15 50.1", diam: 2.9 , errdiam: 0.24 },$
	    { name:"HD61935" , pos:" 07 41 14.83  -09 33  4.1", diam: 2.21, errdiam: 0.01 },$
	    { name:"HD62082" , pos:" 07 41 23.53  -22 20 12.2", diam: 2.59, errdiam: 0.11 },$
	    { name:"HD63295" , pos:" 07 41 49.26  -72 36 21.9", diam: 2.14, errdiam: 0.09 },$
	    { name:"HD62044" , pos:" 07 43 18.73  +28 53  0.6", diam: 2.11, errdiam: 0.14 },$
	    { name:"HD62576" , pos:" 07 43 32.39  -28 24 39.2", diam: 3.89, errdiam: 0.12 },$
	    { name:"HD62285" , pos:" 07 44 06.91  +25 47  3.0", diam: 2.52, errdiam: 0.28 },$
	    { name:"HD62345" , pos:" 07 44 26.85  +24 23 52.8", diam: 2.47, errdiam: 0.17 },$
	    { name:"HD62509" , pos:" 07 45 18.95  +28 01 34.3", diam: 6.11, errdiam: 0.07 },$
	    { name:"HD62721" , pos:" 07 46 07.45  +18 30 36.2", diam: 2.75, errdiam: 0.15 },$
	    { name:"HD62898" , pos:" 07 47 30.32  +33 24 56.5", diam: 2.45, errdiam: 0.13 },$
	    { name:"HD63640" , pos:" 07 48 08.55  -40 39  8.6", diam: 2.81, errdiam: 0.11 },$
	    { name:"HD63700" , pos:" 07 49 17.66  -24 51 35.2", diam: 3.24, errdiam: 0.15 },$
	    { name:"HD63852" , pos:" 07 49 35.40  -33 17 20.2", diam: 2.38, errdiam: 0.22 },$
	    { name:"HD64440" , pos:" 07 52 13.03  -40 34 32.8", diam: 2.45, errdiam: 0.12 },$
	    { name:"HD65183" , pos:" 07 56 22.72  -30 17  4.5", diam: 1.43, errdiam: 0.21 },$
	    { name:"HD65953" , pos:" 08 01 13.33  -01 23 33.4", diam: 3.  , errdiam: 0.06 },$
	    { name:"HD66435" , pos:" 08 02 06.13  -37 03  0.1", diam: 1.37, errdiam: 0.1  },$
	    { name:"HD66141" , pos:" 08 02 15.94  +02 20  4.5", diam: 2.53, errdiam: 0.07 },$
	    { name:"HD67582" , pos:" 08 06 40.34  -45 15 57.7", diam: 2.47, errdiam: 0.33 },$
	    { name:"HD67523" , pos:" 08 07 32.65  -24 18 15.6", diam: 1.41, errdiam: 0.02 },$
	    { name:"HD68553" , pos:" 08 11 21.49  -39 37  6.8", diam: 4.01, errdiam: 0.55 },$
	    { name:"HD69194" , pos:" 08 13 34.25  -50 11 45.9", diam: 2.76, errdiam: 0.13 },$
	    { name:"HD69142" , pos:" 08 14 02.92  -40 20 52.4", diam: 2.07, errdiam: 0.04 },$
	    { name:"HD71701" , pos:" 08 20 38.54  -77 29  4.1", diam: 2.3 , errdiam: 0.04 },$
	    { name:"HD70555" , pos:" 08 21 23.03  -33 03 15.7", diam: 2.54, errdiam: 0.14 },$
	    { name:"HD70276" , pos:" 08 21 42.86  +17 17  6.8", diam: 0.14, errdiam: 0.07 },$
	    { name:"HD70946" , pos:" 08 23 16.93  -38 17  9.9", diam: 1.45, errdiam: 0.23 },$
	    { name:"HD71250" , pos:" 08 26 43.94  +12 39 16.6", diam: 4.39, errdiam: 0.06 },$
	    { name:"HD72094" , pos:" 08 31 35.73  +18 05 39.9", diam: 2.52, errdiam: 0.01 },$
	    { name:"HD72342" , pos:" 08 32 35.01  +08 29 20.0", diam: 0.41, errdiam: 0.12 },$
	    { name:"HD73840" , pos:" 08 40 01.47  -12 28 31.3", diam: 2.46, errdiam: 0.12 },$
	    { name:"HD74006" , pos:" 08 40 06.14  -35 18 30.1", diam: 1.98, errdiam: 0.05 },$
	    { name:"HD74167" , pos:" 08 40 35.26  -45 11 28.0", diam: 2.62, errdiam: 0.12 },$
	    { name:"HD74180" , pos:" 08 40 37.57  -46 38 55.5", diam: 2.38, errdiam: 0.39 },$
	    { name:"HD74772" , pos:" 08 44 23.95  -42 38 57.4", diam: 1.74, errdiam: 0.03 },$
	    { name:"HD74442" , pos:" 08 44 41.10  +18 09 15.5", diam: 2.34, errdiam: 0.08 },$
	    { name:"HD74956" , pos:" 08 44 42.23  -54 42 31.8", diam: 1.45, errdiam: 0.01 },$
	    { name:"HD74739" , pos:" 08 46 41.82  +28 45 35.6", diam: 2.11, errdiam: 0.07 },$
	    { name:"HD74874" , pos:" 08 46 46.51  +06 25  7.7", diam: 1.98, errdiam: 0.06 },$
	    { name:"HD75156" , pos:" 08 48 39.54  +12 32 49.0", diam: 1.2 , errdiam: 0.21 },$
	    { name:"HD76294" , pos:" 08 55 23.63  +05 56 44.0", diam: 3.09, errdiam: 0.02 },$
	    { name:"HD78004" , pos:" 09 04 09.28  -47 05 51.9", diam: 2.76, errdiam: 0.04 },$
	    { name:"HD80007" , pos:" 09 13 11.98  -69 43  1.9", diam: 1.63, errdiam: 0.03 },$
	    { name:"HD80108" , pos:" 09 16 23.04  -44 15 56.6", diam: 2.96, errdiam: 0.51 },$
	    { name:"HD80404" , pos:" 09 17 05.41  -59 16 30.8", diam: 1.78, errdiam: 0.12 },$
	    { name:"HD80493" , pos:" 09 21 03.30  +34 23 33.2", diam: 6.63, errdiam: 0.07 },$
	    { name:"HD81034" , pos:" 09 21 50.99  -42 11 41.5", diam: 3.37, errdiam: 0.1  },$
	    { name:"HD81146" , pos:" 09 24 39.26  +26 10 56.4", diam: 2.3 , errdiam: 0.07 },$
	    { name:"HD81575" , pos:" 09 25 07.86  -43 58 35.8", diam: 3.49, errdiam: 0.1  },$
	    { name:"HD81420" , pos:" 09 25 24.04  -05 07  2.6", diam: 3.41, errdiam: 0.02 },$
	    { name:"HD81797" , pos:" 09 27 35.24  -08 39 31.0", diam: 9.05, errdiam: 0.05 },$
	    { name:"HD82150" , pos:" 09 29 14.72  -35 57  4.8", diam: 2.81, errdiam: 0.12 },$
	    { name:"HD82308" , pos:" 09 31 43.23  +22 58  4.7", diam: 3.84, errdiam: 0.07 },$
	    { name:"HD82381" , pos:" 09 31 57.58  +09 42 56.8", diam: 2.05, errdiam: 0.06 },$
	    { name:"HD83425" , pos:" 09 38 27.29  +04 38 57.5", diam: 2.44, errdiam: 0.02 },$
	    { name:"HD83618" , pos:" 09 39 51.36  -01 08 34.1", diam: 3.23, errdiam: 0.02 },$
	    { name:"HD84810" , pos:" 09 45 14.81  -62 30 28.4", diam: 2.82, errdiam: 0.3  },$
	    { name:"HD84441" , pos:" 09 45 51.07  +23 46 27.3", diam: 2.59, errdiam: 0.02 },$
	    { name:"HD85436" , pos:" 09 50 19.68  -50 26  7.4", diam: 2.34, errdiam: 0.07 },$
	    { name:"HD85622" , pos:" 09 51 40.67  -46 32 51.4", diam: 1.87, errdiam: 0.15 },$
	    { name:"HD85503" , pos:" 09 52 45.82  +26 00 25.0", diam: 3.3 , errdiam: 0.01 },$
	    { name:"HD85709" , pos:" 09 53 42.93  +05 57 30.9", diam: 1.68, errdiam: 0.06 },$
	    { name:"HD85951" , pos:" 09 54 52.21  -19 00 33.7", diam: 3.35, errdiam: 0.15 },$
	    { name:"HD87870" , pos:" 10 07 32.17  -22 29 22.6", diam: 3.1 , errdiam: 0.22 },$
	    { name:"HD87837" , pos:" 10 07 54.27  +09 59 51.0", diam: 3.45, errdiam: 0.03 },$
	    { name:"HD87901" , pos:" 10 08 22.31  +11 58  1.9", diam: 1.36, errdiam: 0.   },$
	    { name:"HD88284" , pos:" 10 10 35.28  -12 21 14.7", diam: 2.39, errdiam: 0.03 },$
	    { name:"HD88647" , pos:" 10 11 35.46  -58 49 39.9", diam: 3.57, errdiam: 0.1  },$
	    { name:"HD89056" , pos:" 10 16 40.74  +13 43 42.0", diam: 2.19, errdiam: 0.2  },$
	    { name:"HD89388" , pos:" 10 17 04.98  -61 19 56.3", diam: 5.17, errdiam: 0.1  },$
	    { name:"HD89353" , pos:" 10 18 07.59  -28 59 31.2", diam: 0.57, errdiam: 0.97 },$
	    { name:"HD89682" , pos:" 10 19 36.75  -55 01 45.5", diam: 3.11, errdiam: 0.2  },$
	    { name:"HD89484" , pos:" 10 19 58.43  +19 50 28.5", diam: 6.64, errdiam: 0.01 },$
	    { name:"HD89945" , pos:" 10 22 41.78  -09 23 42.5", diam: 2.26, errdiam: 0.11 },$
	    { name:"HD90362" , pos:" 10 25 44.27  -07 03 35.4", diam: 2.21, errdiam: 0.08 },$
	    { name:"HD90432" , pos:" 10 26 05.43  -16 50 10.7", diam: 4.45, errdiam: 0.07 },$
	    { name:"HD90610" , pos:" 10 27 09.10  -31 04  4.0", diam: 3.37, errdiam: 0.11 },$
	    { name:"HD91550" , pos:" 10 34 00.88  -23 44 42.6", diam: 2.86, errdiam: 0.18 },$
	    { name:"HD91942" , pos:" 10 35 35.30  -57 33 27.5", diam: 3.49, errdiam: 0.19 },$
	    { name:"HD92036" , pos:" 10 37 13.72  -27 24 45.5", diam: 3.67, errdiam: 0.14 },$
	    { name:"HD92314" , pos:" 10 38 07.12  -60 32 56.3", diam: 2.2 , errdiam: 0.69 },$
	    { name:"HD92397" , pos:" 10 38 45.00  -59 10 58.8", diam: 3.32, errdiam: 0.25 },$
	    { name:"HD92436" , pos:" 10 38 59.62  -58 49  0.7", diam: 2.99, errdiam: 0.09 },$
	    { name:"HD93070" , pos:" 10 43 32.29  -60 33 59.8", diam: 4.15, errdiam: 0.14 },$
	    { name:"HD93655" , pos:" 10 48 40.56  -01 57 32.0", diam: 2.25, errdiam: 0.01 },$
	    { name:"HD93813" , pos:" 10 49 37.49  -16 11 37.1", diam: 4.23, errdiam: 0.04 },$
	    { name:"HD94452" , pos:" 10 52 51.20  -62 00 32.8", diam: 2.32, errdiam: 0.07 },$
	    { name:"HD94264" , pos:" 10 53 18.71  +34 12 53.5", diam: 2.46, errdiam: 0.02 },$
	    { name:"HD94510" , pos:" 10 53 29.66  -58 51 11.4", diam: 2.09, errdiam: 0.01 },$
	    { name:"HD94683" , pos:" 10 54 29.60  -61 49 35.8", diam: 2.44, errdiam: 0.23 },$
	    { name:"HD95272" , pos:" 10 59 46.46  -18 17 55.6", diam: 2.19, errdiam: 0.05 },$
	    { name:"HD95578" , pos:" 11 01 49.67  -02 29  4.5", diam: 2.84, errdiam: 0.04 },$
	    { name:"HD96918" , pos:" 11 08 35.39  -58 58 30.1", diam: 2.56, errdiam: 0.75 },$
	    { name:"HD96760" , pos:" 11 08 56.44  +20 15 28.0", diam: 0.4 , errdiam: 0.14 },$
	    { name:"HD97068" , pos:" 11 10 36.77  +11 18  5.9", diam: 2.22, errdiam: 0.05 },$
	    { name:"HD97451" , pos:" 11 12 03.54  -58 25 24.1", diam: 4.55, errdiam: 0.16 },$
	    { name:"HD97576" , pos:" 11 13 14.69  -44 22 20.0", diam: 4.1 , errdiam: 0.26 },$
	    { name:"HD97778" , pos:" 11 15 12.23  +23 05 43.8", diam: 2.89, errdiam: 0.1  },$
	    { name:"HD98292" , pos:" 11 17 19.04  -67 49 24.6", diam: 3.03, errdiam: 1.75 },$
	    { name:"HD98262" , pos:" 11 18 28.74  +33 05 39.5", diam: 4.4 , errdiam: 0.04 },$
	    { name:"HD98430" , pos:" 11 19 20.45  -14 46 42.8", diam: 2.99, errdiam: 0.06 },$
	    { name:"HD99167" , pos:" 11 24 36.59  -10 51 33.6", diam: 3.45, errdiam: 0.01 },$
	    { name:"HD99363" , pos:" 11 25 54.00  -13 45  4.7", diam: 2.24, errdiam: 0.11 },$
	    { name:"HD224889", pos:" 00 01 35.70  -77 03 56.6", diam: 2.11, errdiam: 0.09 },$
	    { name:"HD224935", pos:" 00 01 57.62  -06 00 50.7", diam: 7.1 , errdiam: 0.07 },$
	    { name:"HD281813", pos:" 04 13 24.45  +29 07 22.5", diam: 2.88, errdiam: 1.4  },$
	    { name:"HD100407", pos:" 11 33 00.12  -31 51 27.4", diam: 2.35, errdiam: 0.04 },$
	    { name:"HD100713", pos:" 11 34 51.08  -60 47 10.1", diam: 2.18, errdiam: 0.07 },$
	    { name:"HD100920", pos:" 11 36 56.93  -00 49 25.5", diam: 1.79, errdiam: 0.01 },$
	    { name:"HD101104", pos:" 11 37 34.06  -60 54 11.6", diam: 5.99, errdiam: 0.13 },$
	    { name:"HD101666", pos:" 11 41 43.95  -32 29 57.9", diam: 2.31, errdiam: 0.12 },$
	    { name:"HD102461", pos:" 11 47 19.14  -57 41 47.4", diam: 2.96, errdiam: 0.18 },$
	    { name:"HD102839", pos:" 11 49 56.61  -70 13 32.9", diam: 2.01, errdiam: 0.56 },$
	    { name:"HD102964", pos:" 11 51 08.69  -45 10 24.5", diam: 2.44, errdiam: 0.13 },$
	    { name:"HD103661", pos:" 11 56 12.32  +15 43 44.3", diam: 0.51, errdiam: 0.05 },$
	    { name:"HD104207", pos:" 12 00 04.70  +19 25  9.7", diam: 0.87, errdiam: 0.05 },$
	    { name:"HD104933", pos:" 12 04 57.22  -60 58  5.7", diam: 2.81, errdiam: 0.14 },$
	    { name:"HD107218", pos:" 12 19 42.59  -19 11 56.0", diam: 2.13, errdiam: 0.07 },$
	    { name:"HD107446", pos:" 12 21 21.61  -60 24  4.1", diam: 4.23, errdiam: 0.05 },$
	    { name:"HD108381", pos:" 12 26 56.27  +28 16  6.3", diam: 2.18, errdiam: 0.07 },$
	    { name:"HD108759", pos:" 12 29 57.89  -41 44  9.2", diam: 3.35, errdiam: 0.2  },$
	    { name:"HD108821", pos:" 12 30 17.45  -23 41 47.1", diam: 2.59, errdiam: 0.17 },$
	    { name:"HD109379", pos:" 12 34 23.23  -23 23 48.3", diam: 3.29, errdiam: 0.07 },$
	    { name:"HD110380", pos:" 12 41 39.64  -01 26 57.8", diam: 1.59, errdiam: 0.07 },$
	    { name:"HD112142", pos:" 12 54 21.16  -09 32 20.4", diam: 5.36, errdiam: 0.12 },$
	    { name:"HD112213", pos:" 12 55 19.43  -42 54 56.6", diam: 2.95, errdiam: 0.18 },$
	    { name:"HD112985", pos:" 13 02 16.26  -71 32 55.9", diam: 3.19, errdiam: 0.06 },$
	    { name:"HD113996", pos:" 13 07 10.73  +27 37 29.1", diam: 2.9 , errdiam: 0.07 },$
	    { name:"HD115211", pos:" 13 17 13.01  -66 47  0.4", diam: 2.52, errdiam: 0.36 },$
	    { name:"HD115521", pos:" 13 17 36.28  +05 28 11.5", diam: 4.05, errdiam: 0.06 },$
	    { name:"HD115659", pos:" 13 18 55.30  -23 10 17.4", diam: 2.88, errdiam: 0.07 },$
	    { name:"HD116658", pos:" 13 25 11.58  -11 09 40.8", diam: 0.85, errdiam: 0.12 },$
	    { name:"HD116492", pos:" 13 25 30.38  -63 25 14.8", diam: 2.27, errdiam: 0.07 },$
	    { name:"HD116870", pos:" 13 26 43.17  -12 42 27.6", diam: 2.44, errdiam: 0.12 },$
	    { name:"HD117041", pos:" 13 30 25.41  -73 58 31.5", diam: 2.04, errdiam: 0.07 },$
	    { name:"HD117440", pos:" 13 31 02.66  -39 24 26.3", diam: 2.69, errdiam: 0.21 },$
	    { name:"HD118508", pos:" 13 36 59.08  +24 36 47.9", diam: 2.57, errdiam: 0.07 },$
	    { name:"HD120323", pos:" 13 49 26.72  -34 27  2.8", diam: 9.16, errdiam: 0.07 },$
	    { name:"HD120477", pos:" 13 49 28.64  +15 47 52.5", diam: 4.46, errdiam: 0.02 },$
	    { name:"HD120539", pos:" 13 49 42.83  +21 15 50.8", diam: 2.42, errdiam: 0.05 },$
	    { name:"HD120404", pos:" 13 51 47.43  -69 24  4.5", diam: 2.94, errdiam: 0.44 },$
	    { name:"HD120933", pos:" 13 51 47.48  +34 26 39.3", diam: 5.47, errdiam: 0.07 },$
	    { name:"HD121370", pos:" 13 54 41.08  +18 23 51.8", diam: 2.19, errdiam: 0.01 },$
	    { name:"HD122451", pos:" 14 03 49.40  -60 22 22.9", diam: 0.93, errdiam: 0.21 },$
	    { name:"HD123123", pos:" 14 06 22.30  -26 40 56.5", diam: 3.43, errdiam: 0.06 },$
	    { name:"HD123139", pos:" 14 06 40.95  -36 22 11.8", diam: 5.17, errdiam: 0.02 },$
	    { name:"HD124294", pos:" 14 12 53.75  -10 16 25.3", diam: 3.12, errdiam: 0.06 },$
	    { name:"HD124897", pos:" 14 15 39.67  +19 10 56.7", diam: 19.2, errdiam: 0.01 },$
	    { name:"HD125180", pos:" 14 17 28.45  +15 15 48.2", diam: 2.58, errdiam: 0.07 },$
	    { name:"HD125687", pos:" 14 22 52.24  -55 57 44.4", diam: 3.92, errdiam: 0.88 },$
	    { name:"HD124882", pos:" 14 26 55.23  -83 40  4.4", diam: 2.79, errdiam: 0.1  },$
	    { name:"HD126927", pos:" 14 28 41.72  -06 54  1.9", diam: 2.14, errdiam: 0.12 },$
	    { name:"HD127093", pos:" 14 28 46.03  +25 51 14.0", diam: 2.66, errdiam: 0.07 },$
	    { name:"HD127665", pos:" 14 31 49.79  +30 22 17.2", diam: 3.74, errdiam: 0.04 },$
	    { name:"HD128193", pos:" 14 38 10.15  -62 43  3.8", diam: 2.18, errdiam: 0.41 },$
	    { name:"HD129456", pos:" 14 43 39.44  -35 10 25.2", diam: 3.27, errdiam: 0.1  },$
	    { name:"HD129989", pos:" 14 44 59.21  +27 04 27.4", diam: 4.2 , errdiam: 0.06 },$
	    { name:"HD129902", pos:" 14 45 11.71  -01 25  3.1", diam: 2.27, errdiam: 0.12 },$
	    { name:"HD130694", pos:" 14 50 17.31  -27 57 37.3", diam: 3.1 , errdiam: 0.14 },$
	    { name:"HD131217", pos:" 14 54 56.94  -61 04 33.0", diam: 2.25, errdiam: 0.29 },$
	    { name:"HD131109", pos:" 14 57 52.98  -76 39 45.6", diam: 2.03, errdiam: 0.15 },$
	    { name:"HD133124", pos:" 15 02 06.51  +25 00 29.3", diam: 2.87, errdiam: 0.07 },$
	    { name:"HD133165", pos:" 15 02 54.04  +02 05 28.7", diam: 1.8 , errdiam: 0.03 },$
	    { name:"HD133216", pos:" 15 04 04.22  -25 16 55.1", diam:11.18, errdiam: 0.14 },$
	    { name:"HD133582", pos:" 15 04 26.74  +26 56 51.5", diam: 2.35, errdiam: 0.06 },$
	    { name:"HD133774", pos:" 15 06 37.60  -16 15 24.5", diam: 2.77, errdiam: 0.4  },$
	    { name:"HD134505", pos:" 15 12 17.09  -52 05 57.3", diam: 2.48, errdiam: 0.03 },$
	    { name:"HD134468", pos:" 15 13 01.02  -61 44 37.6", diam: 2.5 , errdiam: 1.03 },$
	    { name:"HD135291", pos:" 15 17 38.89  -63 36 37.7", diam: 1.98, errdiam: 0.15 },$
	    { name:"HD135758", pos:" 15 17 49.84  -30 08 55.2", diam: 1.95, errdiam: 0.14 },$
	    { name:"HD135902", pos:" 15 20 13.00  -57 23 52.9", diam: 2.44, errdiam: 0.38 },$
	    { name:"HD137066", pos:" 15 27 33.08  -64 31 53.4", diam: 2.61, errdiam: 0.25 },$
	    { name:"HD138140", pos:" 15 32 16.52  -50 52 16.3", diam: 2.77, errdiam: 0.07 },$
	    { name:"HD138492", pos:" 15 33 46.94  -37 38 23.8", diam: 2.36, errdiam: 0.07 },$
	    { name:"HD138905", pos:" 15 35 31.58  -14 47 22.3", diam: 2.21, errdiam: 0.07 },$
	    { name:"HD138816", pos:" 15 36 12.10  -44 23 48.6", diam: 2.1 , errdiam: 0.27 },$
	    { name:"HD138538", pos:" 15 36 43.22  -66 19  1.3", diam: 2.43, errdiam: 0.09 },$
	    { name:"HD139127", pos:" 15 38 03.21  -42 34  2.5", diam: 3.07, errdiam: 0.11 },$
	    { name:"HD139997", pos:" 15 41 56.80  -19 40 43.8", diam: 3.46, errdiam: 0.38 },$
	    { name:"HD140573", pos:" 15 44 16.07  +06 25 32.3", diam: 4.61, errdiam: 0.04 },$
	    { name:"HD140224", pos:" 15 45 02.82  -54 52  2.2", diam: 2.09, errdiam: 0.07 },$
	    { name:"HD140708", pos:" 15 48 17.25  -60 28 32.7", diam: 2.21, errdiam: 0.07 },$
	    { name:"HD141054", pos:" 15 49 13.68  -52 12 57.0", diam: 2.12, errdiam: 0.42 },$
	    { name:"HD141992", pos:" 15 51 15.91  +20 58 40.5", diam: 3.16, errdiam: 0.04 },$
	    { name:"HD142198", pos:" 15 53 49.54  -16 43 45.5", diam: 1.95, errdiam: 0.17 },$
	    { name:"HD142804", pos:" 15 57 05.21  -16 02  3.9", diam: 2.77, errdiam: 0.61 },$
	    { name:"HD144386", pos:" 16 08 10.84  -53 04 44.2", diam: 2.38, errdiam: 0.07 },$
	    { name:"HD145897", pos:" 16 13 50.90  -11 50 15.9", diam: 1.99, errdiam: 0.46 },$
	    { name:"HD145544", pos:" 16 15 26.27  -63 41  8.4", diam: 2.45, errdiam: 0.2  },$
	    { name:"HD146003", pos:" 16 16 43.28  -53 48 40.0", diam: 4.26, errdiam: 0.29 },$
	    { name:"HD146791", pos:" 16 18 19.29  -04 41 33.0", diam: 2.87, errdiam: 0.06 },$
	    { name:"HD146686", pos:" 16 19 50.42  -50 09 19.8", diam: 2.27, errdiam: 0.05 },$
	    { name:"HD147084", pos:" 16 20 38.18  -24 10  9.6", diam: 2.2 , errdiam: 1.83 },$
	    { name:"HD147749", pos:" 16 22 21.43  +33 47 56.6", diam: 3.39, errdiam: 0.07 },$
	    { name:"HD147767", pos:" 16 22 29.22  +33 42 12.5", diam: 2.36, errdiam: 0.07 },$
	    { name:"HD147929", pos:" 16 24 52.06  -02 28 16.6", diam: 2.  , errdiam: 0.07 },$
	    { name:"HD148390", pos:" 16 27 32.27  +02 52 14.3", diam: 2.06, errdiam: 0.22 },$
	    { name:"HD148513", pos:" 16 28 33.98  +00 39 54.0", diam: 2.02, errdiam: 0.19 },$
	    { name:"HD148478", pos:" 16 29 24.46  -26 25 55.2", diam:25.92, errdiam: 1.45 },$
	    { name:"HD148856", pos:" 16 30 13.20  +21 29 22.6", diam: 3.31, errdiam: 0.04 },$
	    { name:"HD149401", pos:" 16 37 08.50  -52 22 54.1", diam: 2.41, errdiam: 0.24 },$
	    { name:"HD150052", pos:" 16 38 47.73  -08 37  7.0", diam: 2.36, errdiam: 0.65 },$
	    { name:"HD150680", pos:" 16 41 17.16  +31 36  9.8", diam: 2.21, errdiam: 0.01 },$
	    { name:"HD150214", pos:" 16 41 24.78  -42 03 37.8", diam: 2.64, errdiam: 0.61 },$
	    { name:"HD321898", pos:" 16 42 58.20  -38 42 40.5", diam: 2.56, errdiam: 0.07 },$
	    { name:"HD149324", pos:" 16 43 04.66  -77 31  2.8", diam: 2.04, errdiam: 0.06 },$
	    { name:"HD151203", pos:" 16 45 22.52  +15 44 43.0", diam: 3.99, errdiam: 0.05 },$
	    { name:"HD151011", pos:" 16 45 30.18  -19 08 12.9", diam: 2.77, errdiam: 0.62 },$
	    { name:"HD151217", pos:" 16 45 49.89  +08 34 57.4", diam: 2.73, errdiam: 0.05 },$
	    { name:"HD151051", pos:" 16 46 50.31  -41 28 51.9", diam: 2.26, errdiam: 0.07 },$
	    { name:"HD150798", pos:" 16 48 39.89  -69 01 39.8", diam: 8.76, errdiam: 0.12 },$
	    { name:"HD151249", pos:" 16 49 47.16  -59 02 29.0", diam: 5.34, errdiam: 0.11 },$
	    { name:"HD152173", pos:" 16 50 38.96  +29 48 23.5", diam: 2.1 , errdiam: 0.07 },$
	    { name:"HD152161", pos:" 16 53 42.42  -43 03  3.5", diam: 4.83, errdiam: 0.5  },$
	    { name:"HD152334", pos:" 16 54 35.01  -42 21 40.7", diam: 3.99, errdiam: 0.07 },$
	    { name:"HD152820", pos:" 16 57 11.18  -33 15 34.2", diam: 2.57, errdiam: 0.34 },$
	    { name:"HD153210", pos:" 16 57 40.10  +09 22 30.1", diam: 3.48, errdiam: 0.08 },$
	    { name:"HD152885", pos:" 16 57 50.22  -39 06 56.6", diam: 2.87, errdiam: 0.5  },$
	    { name:"HD152786", pos:" 16 58 37.21  -55 59 24.5", diam: 7.21, errdiam: 0.21 },$
	    { name:"HD152980", pos:" 16 59 35.05  -53 09 37.6", diam: 3.64, errdiam: 0.12 },$
	    { name:"HD153162", pos:" 17 00 55.70  -56 28 48.0", diam: 2.09, errdiam: 0.44 },$
	    { name:"HD326926", pos:" 17 09 42.26  -40 22 52.4", diam: 2.91, errdiam: 0.07 },$
	    { name:"HD155066", pos:" 17 12 14.63  -53 22 34.0", diam: 2.48, errdiam: 0.32 },$
	    { name:"HD155644", pos:" 17 12 27.81  +10 35  6.6", diam: 2.73, errdiam: 1.31 },$
	    { name:"HD156681", pos:" 17 18 36.99  +10 51 52.1", diam: 2.71, errdiam: 0.44 },$
	    { name:"HD156705", pos:" 17 21 02.23  -41 52 50.7", diam: 2.56, errdiam: 0.07 },$
	    { name:"HD156277", pos:" 17 21 59.48  -67 46 14.4", diam: 1.92, errdiam: 0.09 },$
	    { name:"HD157236", pos:" 17 23 21.59  -28 08 34.2", diam: 2.49, errdiam: 0.25 },$
	    { name:"HD158228", pos:" 17 27 44.08  +08 26 31.3", diam: 3.29, errdiam: 0.47 },$
	    { name:"HD158899", pos:" 17 30 44.31  +26 06 38.3", diam: 2.95, errdiam: 0.09 },$
	    { name:"HD159187", pos:" 17 33 16.98  +00 06 11.8", diam: 2.39, errdiam: 0.48 },$
	    { name:"HD159561", pos:" 17 34 56.07  +12 33 36.1", diam: 1.59, errdiam: 0.04 },$
	    { name:"HD159968", pos:" 17 36 21.42  +27 33 59.9", diam: 2.92, errdiam: 0.18 },$
	    { name:"HD159433", pos:" 17 36 32.83  -38 38  7.1", diam: 2.07, errdiam: 0.06 },$
	    { name:"HD159532", pos:" 17 37 19.13  -42 59 52.2", diam: 2.77, errdiam: 0.08 },$
	    { name:"HD159881", pos:" 17 38 11.99  -28 02 48.3", diam: 2.56, errdiam: 0.6  },$
	    { name:"HD160677", pos:" 17 39 57.52  +31 12  8.9", diam: 2.11, errdiam: 0.12 },$
	    { name:"HD160668", pos:" 17 42 51.10  -36 56 44.0", diam: 2.22, errdiam: 0.23 },$
	    { name:"HD160748", pos:" 17 43 06.86  -33 03  4.6", diam: 2.45, errdiam: 1.94 },$
	    { name:"HD160810", pos:" 17 43 26.00  -35 17 51.6", diam: 2.72, errdiam: 0.63 },$
	    { name:"HD161096", pos:" 17 43 28.35  +04 34  2.3", diam: 4.38, errdiam: 0.08 },$
	    { name:"HD161204", pos:" 17 44 34.41  -07 51 21.8", diam: 2.54, errdiam: 0.07 },$
	    { name:"HD160635", pos:" 17 45 43.99  -64 43 25.9", diam: 2.94, errdiam: 0.11 },$
	    { name:"HD161797", pos:" 17 46 27.53  +27 43 14.4", diam: 1.92, errdiam: 0.01 },$
	    { name:"HD161849", pos:" 17 48 41.67  -19 46 46.6", diam: 2.16, errdiam: 0.07 },$
	    { name:"HD161892", pos:" 17 49 51.48  -37 02 35.9", diam: 3.69, errdiam: 0.06 },$
	    { name:"HD316665", pos:" 17 55 13.24  -28 00 21.8", diam: 2.28, errdiam: 0.07 },$
	    { name:"HD163993", pos:" 17 57 45.89  +29 14 52.4", diam: 2.16, errdiam: 0.03 },$
	    { name:"HD163376", pos:" 17 57 47.81  -41 42 58.7", diam: 3.79, errdiam: 0.12 },$
	    { name:"HD163917", pos:" 17 59 01.59  -09 46 25.1", diam: 2.75, errdiam: 0.11 },$
	    { name:"HD163755", pos:" 17 59 05.57  -30 15 12.7", diam: 3.18, errdiam: 0.96 },$
	    { name:"HD164064", pos:" 17 59 36.76  -04 49 16.1", diam: 2.07, errdiam: 0.52 },$
	    { name:"HD164349", pos:" 18 00 03.42  +16 45  3.3", diam: 1.99, errdiam: 0.14 },$
	    { name:"HD164358", pos:" 18 01 23.12  -17 09 24.7", diam: 2.2 , errdiam: 1.02 },$
	    { name:"HD164579", pos:" 18 02 22.24  -12 19  6.1", diam: 2.67, errdiam: 1.44 },$
	    { name:"HD164857", pos:" 18 03 56.48  -16 56  6.9", diam: 2.45, errdiam: 0.07 },$
	    { name:"HD165135", pos:" 18 05 48.49  -30 25 26.7", diam: 3.33, errdiam: 0.05 },$
	    { name:"HD167036", pos:" 18 14 15.90  -21 42 47.4", diam: 2.26, errdiam: 0.27 },$
	    { name:"HD167121", pos:" 18 15 05.11  -29 49 22.8", diam: 2.31, errdiam: 0.32 },$
	    { name:"HD167618", pos:" 18 17 37.64  -36 45 42.1", diam: 11.33, errdiam: 0.04 },$
	    { name:"HD168532", pos:" 18 19 10.68  +24 26 45.8", diam: 2.27, errdiam: 0.21 },$
	    { name:"HD168415", pos:" 18 20 08.79  -15 49 54.1", diam: 2.2 , errdiam: 0.3  },$
	    { name:"HD168454", pos:" 18 20 59.64  -29 49 41.2", diam: 5.78, errdiam: 0.15 },$
	    { name:"HD168723", pos:" 18 21 18.60  -02 53 55.8", diam: 2.87, errdiam: 0.04 },$
	    { name:"HD168592", pos:" 18 22 18.57  -38 39 24.8", diam: 3.65, errdiam: 0.11 },$
	    { name:"HD168339", pos:" 18 23 13.62  -61 29 38.0", diam: 3.32, errdiam: 0.12 },$
	    { name:"HD168988", pos:" 18 23 19.93  -20 39 13.1", diam: 2.13, errdiam: 0.07 },$
	    { name:"HD169414", pos:" 18 23 41.89  +21 46 11.1", diam: 2.87, errdiam: 0.03 },$
	    { name:"HD169022", pos:" 18 24 10.32  -34 23  4.6", diam: 1.42, errdiam: 0.04 },$
	    { name:"HD169274", pos:" 18 24 38.34  -19 43 58.3", diam: 2.2 , errdiam: 1.34 },$
	    { name:"HD169334", pos:" 18 24 53.02  -16 16 35.4", diam: 2.18, errdiam: 0.07 },$
	    { name:"HD169420", pos:" 18 25 21.04  -20 32 30.0", diam: 2.47, errdiam: 0.38 },$
	    { name:"HD169916", pos:" 18 27 58.24  -25 25 18.1", diam: 3.75, errdiam: 0.04 },$
	    { name:"HD170016", pos:" 18 28 10.14  -21 14 19.3", diam: 2.66, errdiam: 0.74 },$
	    { name:"HD169767", pos:" 18 28 49.86  -49 04 14.1", diam: 2.12, errdiam: 0.04 },$
	    { name:"HD170975", pos:" 18 32 43.32  -14 51 56.4", diam: 3.62, errdiam: 0.75 },$
	    { name:"HD171115", pos:" 18 33 53.49  -24 01 56.2", diam: 3.01, errdiam: 0.5  },$
	    { name:"HD171443", pos:" 18 35 12.43  -08 14 38.7", diam: 3.44, errdiam: 0.16 },$
	    { name:"HD172137", pos:" 18 39 02.52  -15 02 21.4", diam: 2.51, errdiam: 0.59 },$
	    { name:"HD171967", pos:" 18 39 35.16  -43 11  9.2", diam: 3.82, errdiam: 0.17 },$
	    { name:"HD172768", pos:" 18 42 17.90  -07 20 14.4", diam: 2.72, errdiam: 1.52 },$
	    { name:"HD171759", pos:" 18 43 02.14  -71 25 41.2", diam: 2.57, errdiam: 0.07 },$
	    { name:"HD173074", pos:" 18 43 37.37  -01 33 32.5", diam: 2.45, errdiam: 0.47 },$
	    { name:"HD173460", pos:" 18 46 20.61  -22 23 31.8", diam: 2.66, errdiam: 0.59 },$
	    { name:"HD173484", pos:" 18 46 47.24  -29 37 53.9", diam: 3.35, errdiam: 0.31 },$
	    { name:"HD173764", pos:" 18 47 10.47  -04 44 52.3", diam: 2.1 , errdiam: 0.58 },$
	    { name:"HD174116", pos:" 18 49 40.11  -20 19 28.8", diam: 2.12, errdiam: 0.42 },$
	    { name:"HD174796", pos:" 18 52 25.70  -03 43 32.3", diam: 2.1 , errdiam: 1.77 },$
	    { name:"HD175100", pos:" 18 53 29.90  +09 39 30.8", diam: 2.31, errdiam: 2.81 },$
	    { name:"HD174974", pos:" 18 54 10.18  -22 44 41.4", diam: 2.18, errdiam: 0.66 },$
	    { name:"HD175309", pos:" 18 54 28.96  +10 37 57.0", diam: 3.38, errdiam: 1.14 },$
	    { name:"HD175775", pos:" 18 57 43.80  -21 06 24.0", diam: 3.26, errdiam: 0.23 },$
	    { name:"HD175329", pos:" 18 58 36.45  -60 12  2.0", diam: 2.08, errdiam: 0.14 },$
	    { name:"HD176124", pos:" 18 59 23.82  -19 16 45.7", diam: 3.61, errdiam: 0.66 },$
	    { name:"HD176411", pos:" 18 59 37.20  +15 04  5.0", diam: 2.14, errdiam: 0.07 },$
	    { name:"HD176541", pos:" 18 59 58.11  +22 48 52.2", diam: 3.5 , errdiam: 0.22 },$
	    { name:"HD176670", pos:" 19 00 00.83  +32 08 43.9", diam: 2.82, errdiam: 0.19 },$
	    { name:"HD176678", pos:" 19 01 40.83  -05 44 20.8", diam: 2.35, errdiam: 0.13 },$
	    { name:"HD177241", pos:" 19 04 40.98  -21 44 29.4", diam: 2.35, errdiam: 0.09 },$
	    { name:"HD177494", pos:" 19 04 46.92  +02 59  2.8", diam: 2.23, errdiam: 0.07 },$
	    { name:"HD177716", pos:" 19 06 56.41  -27 40 13.5", diam: 3.72, errdiam: 0.07 },$
	    { name:"HD178131", pos:" 19 07 26.39  -01 07 32.3", diam: 2.41, errdiam: 0.38 },$
	    { name:"HD178690", pos:" 19 08 37.70  +24 10 45.5", diam: 2.22, errdiam: 1.74 },$
	    { name:"HD178524", pos:" 19 09 45.83  -21 01 25.0", diam: 1.58, errdiam: 0.27 },$
	    { name:"HD178345", pos:" 19 10 01.76  -39 20 26.9", diam: 2.46, errdiam: 0.13 },$
	    { name:"HD179820", pos:" 19 13 54.43  +02 37 30.8", diam: 3.17, errdiam: 0.07 },$
	    { name:"HD179987", pos:" 19 14 18.27  +11 42 46.0", diam: 1.43, errdiam: 1.83 },$
	    { name:"HD180377", pos:" 19 15 33.36  +18 30 58.9", diam: 2.43, errdiam: 0.31 },$
	    { name:"HD180810", pos:" 19 16 43.50  +31 08 22.2", diam: 2.4 , errdiam: 0.07 },$
	    { name:"HD181070", pos:" 19 18 30.89  +15 32 32.0", diam: 2.09, errdiam: 0.07 },$
	    { name:"HD181410", pos:" 19 19 31.89  +22 34 14.7", diam: 4.72, errdiam: 0.25 },$
	    { name:"HD181925", pos:" 19 24 21.48  -43 43 19.4", diam: 2.11, errdiam: 0.14 },$
	    { name:"HD182429", pos:" 19 25 08.10  -13 26 20.6", diam: 2.48, errdiam: 0.07 },$
	    { name:"HD183439", pos:" 19 28 42.33  +24 39 53.7", diam: 3.76, errdiam: 0.09 },$
	    { name:"HD183912", pos:" 19 30 43.28  +27 57 34.9", diam: 4.04, errdiam: 0.14 },$
	    { name:"HD183799", pos:" 19 32 28.51  -30 52 10.1", diam: 3.65, errdiam: 0.28 },$
	    { name:"HD184406", pos:" 19 34 05.35  +07 22 44.2", diam: 2.16, errdiam: 0.04 },$
	    { name:"HD185622", pos:" 19 39 25.34  +16 34 16.0", diam: 2.84, errdiam: 1.73 },$
	    { name:"HD186860", pos:" 19 45 42.91  +30 15 25.6", diam: 2.32, errdiam: 0.63 },$
	    { name:"HD187076", pos:" 19 47 23.26  +18 32  3.4", diam: 8.05, errdiam: 0.13 },$
	    { name:"HD187086", pos:" 19 50 14.06  -47 33 26.6", diam: 2.49, errdiam: 0.19 },$
	    { name:"HD187642", pos:" 19 50 47.00  +08 52  6.0", diam: 3.22, errdiam: 0.01 },$
	    { name:"HD187929", pos:" 19 52 28.37  +01 00 20.4", diam: 1.78, errdiam: 0.44 },$
	    { name:"HD188512", pos:" 19 55 18.79  +06 24 24.4", diam: 1.98, errdiam: 0.02 },$
	    { name:"HD188947", pos:" 19 56 18.37  +35 05 00.3", diam: 2.16, errdiam: 0.07 },$
	    { name:"HD188603", pos:" 19 56 56.83  -27 10 11.6", diam: 2.86, errdiam: 0.29 },$
	    { name:"HD189319", pos:" 19 58 45.43  +19 29 31.7", diam: 5.55, errdiam: 0.04 },$
	    { name:"HD189577", pos:" 20 00 03.31  +17 30 59.4", diam: 5.05, errdiam: 0.11 },$
	    { name:"HD189140", pos:" 20 00 26.49  -43 02 35.9", diam: 2.06, errdiam: 0.11 },$
	    { name:"HD190248", pos:" 20 08 43.61  -66 10 55.4", diam: 1.8 , errdiam: 0.01 },$
	    { name:"HD192876", pos:" 20 17 38.87  -12 30 29.6", diam: 0.89, errdiam: 0.91 },$
	    { name:"HD193218", pos:" 20 17 54.50  +34 05 25.2", diam: 3.98, errdiam: 0.39 },$
	    { name:"HD192947", pos:" 20 18 03.26  -12 32 41.5", diam: 2.27, errdiam: 0.08 },$
	    { name:"HD193495", pos:" 20 21 00.68  -14 46 52.9", diam: 2.72, errdiam: 0.22 },$
	    { name:"HD194317", pos:" 20 23 51.62  +32 11 24.6", diam: 2.62, errdiam: 0.12 },$
	    { name:"HD196036", pos:" 20 34 16.35  +19 31 50.1", diam: 2.28, errdiam: 0.32 },$
	    { name:"HD196321", pos:" 20 36 43.63  -02 32 59.9", diam: 3.14, errdiam: 0.24 },$
	    { name:"HD196171", pos:" 20 37 34.03  -47 17 29.4", diam: 3.09, errdiam: 0.05 },$
	    { name:"HD196777", pos:" 20 40 02.94  -18 08 19.2", diam: 3.65, errdiam: 0.14 },$
	    { name:"HD196917", pos:" 20 41 23.66  -31 35 53.8", diam: 2.15, errdiam: 0.12 },$
	    { name:"HD197989", pos:" 20 46 12.68  +33 58 12.9", diam: 4.29, errdiam: 0.02 },$
	    { name:"HD197964", pos:" 20 46 39.50  +16 07 27.5", diam: 2.17, errdiam: 0.01 },$
	    { name:"HD198048", pos:" 20 49 28.96  -46 13 36.6", diam: 2.86, errdiam: 0.11 },$
	    { name:"HD198700", pos:" 20 54 48.60  -58 27 15.0", diam: 3.1 , errdiam: 0.11 },$
	    { name:"HD200043", pos:" 20 59 58.80  +32 29 44.3", diam: 2.2 , errdiam: 0.22 },$
	    { name:"HD202369", pos:" 21 15 44.84  -15 10 17.4", diam: 4.46, errdiam: 0.1  },$
	    { name:"HD203504", pos:" 21 22 05.20  +19 48 16.2", diam: 0.16, errdiam: 0.06 },$
	    { name:"HD204075", pos:" 21 26 40.03  -22 24 40.8", diam: 1.97, errdiam: 0.12 },$
	    { name:"HD204724", pos:" 21 29 56.90  +23 38 19.8", diam: 4.11, errdiam: 0.06 },$
	    { name:"HD204867", pos:" 21 31 33.53  -05 34 16.2", diam: 2.6 , errdiam: 0.1  },$
	    { name:"HD205478", pos:" 21 41 28.65  -77 23 24.2", diam: 2.28, errdiam: 0.03 },$
	    { name:"HD206778", pos:" 21 44 11.16  +09 52 30.0", diam: 8.38, errdiam: 0.09 },$
	    { name:"HD206859", pos:" 21 44 30.70  +17 21  0.1", diam: 2.04, errdiam: 0.12 },$
	    { name:"HD207932", pos:" 21 52 18.18  +21 16 23.4", diam: 4.27, errdiam: 0.17 },$
	    { name:"HD208631", pos:" 21 57 18.71  +17 45 44.5", diam: 1.45, errdiam: 0.2  },$
	    { name:"HD209709", pos:" 22 05 03.46  +14 48 57.3", diam: 2.94, errdiam: 0.08 },$
	    { name:"HD209750", pos:" 22 05 47.04  -00 19 11.5", diam: 2.94, errdiam: 0.1  },$
	    { name:"HD210090", pos:" 22 07 29.97  +18 00  2.5", diam: 2.78, errdiam: 0.13 },$
	    { name:"HD210050", pos:" 22 08 19.38  -34 33 21.6", diam: 2.23, errdiam: 0.14 },$
	    { name:"HD210502", pos:" 22 10 37.43  +11 37 28.3", diam: 2.36, errdiam: 0.09 },$
	    { name:"HD211416", pos:" 22 18 30.09  -60 15 34.5", diam: 5.6 , errdiam: 0.07 },$
	    { name:"HD213119", pos:" 22 29 07.98  +09 07 44.5", diam: 2.26, errdiam: 0.08 },$
	    { name:"HD213009", pos:" 22 29 16.17  -43 29 44.0", diam: 2.07, errdiam: 0.09 },$
	    { name:"HD215182", pos:" 22 43 00.14  +30 13 16.5", diam: 2.91, errdiam: 0.07 },$
	    { name:"HD215369", pos:" 22 45 37.88  -53 30  0.4", diam: 1.91, errdiam: 0.1  },$
	    { name:"HD215665", pos:" 22 46 31.88  +23 33 56.4", diam: 2.22, errdiam: 0.06 },$
	    { name:"HD216131", pos:" 22 49 00.19  +24 36  5.7", diam: 2.37, errdiam: 0.04 },$
	    { name:"HD216032", pos:" 22 49 35.50  -13 35 33.5", diam: 4.89, errdiam: 0.11 },$
	    { name:"HD215985", pos:" 22 50 00.96  -55 14  2.8", diam: 2.15, errdiam: 0.11 },$
	    { name:"HD216386", pos:" 22 52 36.88  -07 34 46.6", diam: 6.71, errdiam: 0.05 },$
	    { name:"HD217005", pos:" 22 57 48.50  -26 10  0.0", diam: 2.13, errdiam: 0.07 },$
	    { name:"HD217364", pos:" 23 00 52.81  -52 45 14.9", diam: 2.02, errdiam: 0.04 },$
	    { name:"HD217902", pos:" 23 04 39.63  -53 57 53.7", diam: 2.18, errdiam: 0.1  },$
	    { name:"HD218594", pos:" 23 09 26.80  -21 10 20.7", diam: 3.1 , errdiam: 0.07 },$
	    { name:"HD218670", pos:" 23 10 21.54  -45 14 48.2", diam: 2.06, errdiam: 0.07 },$
	    { name:"HD219449", pos:" 23 15 53.49  -09 05 15.8", diam: 2.02, errdiam: 0.07 },$
	    { name:"HD219615", pos:" 23 17 09.94  +03 16 56.2", diam: 2.24, errdiam: 0.04 },$
	    { name:"HD219784", pos:" 23 18 49.44  -32 31 55.3", diam: 2.04, errdiam: 0.07 },$
	    { name:"HD220009", pos:" 23 20 20.58  +05 22 52.7", diam: 1.81, errdiam: 0.05 },$
	    { name:"HD220088", pos:" 23 20 49.56  +30 24 53.7", diam: 2.26, errdiam: 0.14 },$
	    { name:"HD220321", pos:" 23 22 58.23  -20 06  2.1", diam: 2.38, errdiam: 0.07 },$
	    { name:"HD220704", pos:" 23 26 02.79  -20 38 31.2", diam: 3.44, errdiam: 0.07 },$
	    { name:"HD223075", pos:" 23 46 23.52  +03 29 12.5", diam: 2.1 , errdiam: 0.14 },$
	    { name:"HD223800", pos:" 23 52 47.56  -12 01  0.9", diam: 2.85, errdiam: 0.07 },$
	    { name:"iras_15583-5030", pos:" 16 02 05.60  -50 39  2.0", diam: 1.28, errdiam: 0.07 }$
          ]

  for i=0, N_elements(caldb)-1 do begin
      if (caldb[i].name eq STRUPCASE(name)) then return, caldb[i]
  endfor
  print,"Object not found in list of calibrators: ",name
  return, {name: name, pos:"unknown", diam:-1., errdiam:99.}
end
