;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: chop_nod_phot.pro,v 1.9 2003/11/08 06:35:36 koehler Exp $
;
; Last Change: Sun Oct 31 19:28:53 2004
;
; Main routine is "chop_nod_phot"
;
; INPUTS:
;	fname		MIDI chopping or nodding data
;	fnamesky	MIDI nodding data of sky
;	threshold	convergence factor to determine best aperture radius
;	/SILENT 	tells routine to be non interactive
;	/REVERSE  flags for object and sky are reversed (Feb.2003 run)
;
; OUTPUTS:
;	an ascii file in subdirectory PHOTOMETRY
;
; TODO:
;	is Beam A always in data2 and Beam B in data1?
;	is the orientation correct?
;
;-----------------------------------------------------------------------
;-----------------------------------------------------------------------

;draw a circle (by Kevin Ivory)
function circle, xcenter, ycenter, radius
points = (2*!PI/99.0) * FINDGEN(100)
x = xcenter + radius *cos(points)
y = ycenter + radius *sin(points)
return,transpose([[x],[y]])
end

;-----------------------------------------------------------------------

function trunc,nb
if nb-fix(nb) gt 0.5 then result=fix(nb)+1 else result=fix(nb)
return, result
end

;-----------------------------------------------------------------------

function make_maskname, fname
  first_char      = STRPOS(fname,'/MIDI.20')+6
  last_char       = STRPOS(fname,'.fits')
  short_file_name = STRMID(fname,first_char,last_char-first_char)
  return, 'PHOTOMETRY/Mask_'+short_file_name+'.fits'
end

;-----------------------------------------------------------------------
;-----------------------------------------------------------------------
;---------------	MAIN PROG	--------------------------------
;-----------------------------------------------------------------------

pro chop_nod_phot, fname, skyname, THRESHOLD=threshold, SILENT=silent, $
                   REVERSE=reverse

  ; make once and for all sure that fname is a scalar
  fname = fname[0]
  hdrname= (oirFileList(fname))[0]

  if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'

  if N_Params() gt 1  then begin
      if not keyword_set(silent) then $
        print, 'skyname given, this must be nodding data'

      skyname = skyname[0]

      ;im_on= oirgetmeanrms(fname)
      spawn, 'rm PHOTOMETRY/nod_ftemp.fits PHOTOMETRY/nod_skytemp.fits'
      spawn, getenv("MIAroot") + '/bin/oirMeanRMS "' + fname + '" PHOTOMETRY/nod_ftemp.fits'
      im_on= oirgetdata('PHOTOMETRY/nod_ftemp.fits')

      ;im_sky = oirgetmeanrms(skyname)
      spawn, getenv("MIAroot") + '/bin/oirMeanRMS "' + skyname + '" PHOTOMETRY/nod_skytemp.fits'
      im_sky= oirgetdata('PHOTOMETRY/nod_skytemp.fits')

      win1 = im_on[0].data1-im_sky[0].data1
      win2 = im_on[0].data2-im_sky[0].data2
      win1_stdev= im_on[1].data1-im_sky[1].data1
      win2_stdev= im_on[1].data2-im_sky[1].data2

  endif else begin
      if not keyword_set(silent) then $
        print, 'no skyname given, this must be chopping data'

      ;-------------- Walter routine to extract chooped images --------------
      ;------- before and after keywors are valid only for Feb. Chopping-----
      ;------- It was a bad tuning --------------------------------

      chop_image= MIDIChopImageC(fname, before=3, after=3)
      if keyword_set(reverse) then begin
          on_source_win1 = chop_image[1].data1
          on_source_win2 = chop_image[1].data2
          off_source_win1= chop_image[0].data1
          off_source_win2= chop_image[0].data2
      endif else begin
          on_source_win1 = chop_image[0].data1
          on_source_win2 = chop_image[0].data2
          off_source_win1= chop_image[1].data1
          off_source_win2= chop_image[1].data2
      endelse
      win1 = on_source_win1-off_source_win1
      win2 = on_source_win2-off_source_win2
  endelse

  n=size(win1)
  x_size=n[1]
  y_size=n[2]
  NDIT  =n[3]

  factor = 5


;---------------- PHOTOMETRY ROUTINE --------------

;---- DEFINITION OF MIDI FOV (OPEN FIELD MODE) ----------
;FOV_x1 = 15
;FOV_x2 = 45
;FOV_y1 = 15
;FOV_y2 = 45

FOV_x1 = 0
FOV_x2 = x_size-1
FOV_y1 = 0
FOV_y2 = y_size-1

print,''
print,'-------------  FIRST FIND THE STAR ---------------'
print,''
fit2D_A= gauss2dfit(win2(FOV_x1:FOV_x2,FOV_y1:FOV_y2),a)
print, 'Star gauss. A:', a[4]+FOV_x1, a[5]+FOV_y1
print, 'FWHM x, y',  a[2], a[3]
print, 'Max image', max(win2, status), ' Pos X = ', status mod x_size, ' Pos y = ', trunc(status/x_size)
xm1 = status mod x_size
ym1 = trunc(status/x_size)
print,''
fit2D_B= gauss2dfit(win1(FOV_x1:FOV_x2,FOV_y1:FOV_y2),b)
print, 'Star gauss. B:', b[4]+FOV_x1, b[5]+FOV_y1
print, 'FWHM x, y',  b[2], b[3]
print, 'Max image', max(win1, status), ' Pos X = ', status mod x_size, ' Pos y = ', trunc(status/x_size)
xm2 = status mod x_size
ym2 = trunc(status/x_size)
print,''

;draw the acquisition images, mark the peak of the fitted gaussians & maximae

window,0,xsize=2*n[1]*factor,ysize=n[2]*factor
win1(where(win1 lt 1))=1
win2(where(win2 lt 1))=1

tvscl, rebin(sqrt(win1),n[1]*factor,n[2]*factor),1
tvscl, rebin(sqrt(win2),n[1]*factor,n[2]*factor),0

XYOUTS,5*factor,60*factor,'Beam A',/DEVICE
XYOUTS,(5+n[1])*factor,60*factor,'Beam B',/DEVICE

;orientation of the images

midifieldangle,fname[0],para,phin,phie,rot
print,''
print,'North counted clockwise with respect to >>up<< (+V), East = North - 90 deg:'
print,''
print,'Phi North: ',phin
print,'Phi East: ',phie
phin = -phin
phie = -phie
print,''

sn=sin(2*!PI*phin/360)
se=sin(2*!PI*phie/360)
cn=cos(2*!PI*phin/360)
ce=cos(2*!PI*phie/360)

PLOTS,[62*factor,(62-5*sn)*factor],[15*factor,(15+5*cn)*factor],/DEVICE
XYOUTS,(62-10*sn)*factor,(15+10*cn)*factor,'N',/DEVICE
PLOTS,[62*factor,(62-5*se)*factor],[15*factor,(15+5*ce)*factor],/DEVICE
XYOUTS,(62-10*se)*factor,(15+10*ce)*factor,'E',/DEVICE

PLOTS,[xm1*factor-3,xm1*factor+3],[ym1*factor+3,ym1*factor-3],/DEVICE,Color=1
PLOTS,[xm1*factor+3,xm1*factor-3],[ym1*factor+3,ym1*factor-3],/DEVICE,Color=1
PLOTS,[(xm2+n[1])*factor-3,(xm2+n[1])*factor+3],[ym2*factor+3,ym2*factor-3],/DEVICE,Color=1
PLOTS,[(xm2+n[1])*factor+3,(xm2+n[1])*factor-3],[ym2*factor+3,ym2*factor-3],/DEVICE,Color=1

plot:
PLOTS,[a[4]*factor-5,a[4]*factor+5],[a[5]*factor,a[5]*factor],/DEVICE,Color=1
PLOTS,[a[4]*factor,a[4]*factor],[a[5]*factor-5,a[5]*factor+5],/DEVICE,Color=1
PLOTS,[(b[4]+n[1])*factor-5,(b[4]+n[1])*factor+5],[b[5]*factor,b[5]*factor],/DEVICE,Color=1
PLOTS,[(b[4]+n[1])*factor,(b[4]+n[1])*factor],[b[5]*factor-5,b[5]*factor+5],/DEVICE,Color=1

if not keyword_set(silent) then begin
    !p.multi=[0, 0, 4, 0, 0]
    window, 1, xsize=600, ysize=600
    plot, win2(FOV_x1:FOV_x2,trunc(a[5])+FOV_y1), xtitle='x', title="Beam A, fit of x-position"
    oplot, fit2D_A(*,trunc(a[5])), linestyle=1
    plot, win2(trunc(a[4])+FOV_x1,FOV_y1:FOV_y2), xtitle='y', title="Beam A, fit of y-position"
    oplot, fit2D_A(trunc(a[4]),*), linestyle=1
    plot, win1(FOV_x1:FOV_x2,trunc(b[5])+FOV_y1), xtitle='x', title="Beam B, fit of x-position"
    oplot, fit2D_B(*,trunc(b[5])), linestyle=1
    plot, win1(trunc(b[4])+FOV_x1,FOV_y1:FOV_y2), xtitle='y', title="Beam B, fit of y-position"
    oplot, fit2D_B(trunc(b[4]),*), linestyle=1
    !p.multi=[0, 0, 1, 0, 0]

    testOk1= "string"
    read, '<?> Are the gaussian fits (+) okay? (y/n) ? ', testOK1
    if (strupcase(testOK1) eq "N") then begin
        testOk2= "string"
        read, '<?> Enter coordinates with mouse? (n/y) ? ', testOK2
        if (strupcase(testOK2) eq "Y") then begin
        wset, 0

        a[4] = 100
        a[5] = 100
        b[4] = 100
        b[5] = 100

        while (a[4] lt 0 or a[4] gt 62 or a[5] lt 0 or a[5] gt 69) do begin
        print,'<!>Click in the left image to select your source'
        cursor, ax, ay, /DEVICE,/DOWN
        a[4] = ax / factor
        a[5] = ay / factor
        print,a[4],a[5]
        endwhile

        while (b[4] lt 0 or b[4] gt 62 or b[5] lt 0 or b[5] gt 69) do begin
        print,'<!>Click in the right image to select your source'
        cursor, bx, by, /DEVICE,/DOWN
        b[4] = bx / factor - n[1]
        b[5] = by / factor
        print,b[4],b[5]
        endwhile

        goto, plot
        endif else begin
        print, '<!> OK, try another file'
        goto, ende
        endelse
    endif
endif

photo_test = 10
photo_A = dblarr(photo_test)
photo_B = dblarr(photo_test)
conv_photo_A =dblarr(photo_test-2)
conv_photo_B =dblarr(photo_test-2)

best_radius_found_A = 0
best_radius_found_B = 0
if not keyword_set(threshold) then threshold = 0.1

FOR i=1, photo_test DO BEGIN
    midiaper, win2, a[4]+FOV_x1,a[5]+FOV_y1,magsA,errapA,skyA,skyerrA,145.,[i],[i+6,i+8],[-99999,99999],/FLUX, /SILENT
    midiaper, win1, b[4]+FOV_x1,b[5]+FOV_y1,magsB,errapB,skyB,skyerrB,145.,[i],[i+6,i+8],[-99999,99999],/FLUX, /SILENT
    photo_A(i-1)=magsA(0)
    photo_B(i-1)=magsB(0)
    if not keyword_set(silent) then print,"aperture radius",i,magsA[0],magsB[0]

    IF ((i gt 1) and (i lt photo_test)) THEN BEGIN
        conv_photo_A(i-2) =(photo_A(i-1)-photo_A(i-2))/photo_A(i-1)
        conv_photo_B(i-2) =(photo_B(i-1)-photo_B(i-2))/photo_B(i-1)

        if ((conv_photo_A(i-2) le threshold) and (best_radius_found_A eq 0)) then begin
            best_radius_A=i
            best_radius_found_A = 1
        endif

        if ((conv_photo_B(i-2) le threshold) and (best_radius_found_B eq 0)) then begin
            best_radius_B=i
            best_radius_found_B = 1
        endif
    ENDIF
ENDFOR

    if ((best_radius_found_A eq 0) or (best_radius_found_B eq 0)) then begin
        print, '<!> NO PHOTOMETRIC RADIUS FOUND'
        goto, ende
    endif

;rA = best_radius_A
;rB = best_radius_B

rA = best_radius_A < best_radius_B
rB = rA

if not keyword_set(silent) then begin
    absc = dindgen(photo_test-2)+2
    !p.multi=[0, 0, 2, 0, 0]
    ;plot,photo_A
    ;plot,photo_B
    plot, absc, conv_photo_A, xtitle='pixels', yrange=[0,0.8], ytitle='convergence', title='Beam A'
    oplot,[rA,rA],[0, threshold]
    plot, absc, conv_photo_B, xtitle='pixels', yrange=[0,0.8], ytitle='convergence', title='Beam B'
    oplot,[rB,rB],[0, threshold]
endif

midiaper, win2, a[4]+FOV_x1,a[5]+FOV_y1, magsA,errapA,skyA,skyerrA,145.,[rA,rA+1],[rA+6,rA+8],[-99999,99999], maskA, /FLUX, /SILENT
midiaper, win1, b[4]+FOV_x1,b[5]+FOV_y1, magsB,errapB,skyB,skyerrB,145.,[rB,rB+1],[rB+6,rB+8],[-99999,99999], maskB, /FLUX, /SILENT

  im=obj_new('fitsfile',hdrname)
  he = im->prihead()
  OBJECT    = STRCOMPRESS(he->getpar('OBJECT'),/remove_all)
  DIT       = he->getpar('DET DIT')
  obj_destroy,im

print,''
print,'------------- Aperture Photometry ---------------'
print,''
print, '           	Flux (photons)	     error     		Sky	    error'
print,''
print,'rA=', rA,' : rB= ', rB
print, 'MIDI Beam A ', magsA[0]*145/DIT,errapA[0]*145/DIT,skyA[0]*145/DIT,skyerrA[0]*145/DIT
print, 'MIDI Beam B ', magsB[0]*145/DIT,errapB[0]*145/DIT,skyB[0]*145/DIT,skyerrB[0]*145/DIT
print,''
print,'rA=', rA+1,' : rB= ', rB+1
print, 'MIDI Beam A ', magsA[1]*145/DIT,errapA[1]*145/DIT,skyA[0]*145/DIT,skyerrA[0]*145/DIT
print, 'MIDI Beam B ', magsB[1]*145/DIT,errapB[1]*145/DIT,skyB[0]*145/DIT,skyerrB[0]*145/DIT
print,''

     first_char    = STRPOS(fname,'/MIDI.20')+6
     last_char     = STRPOS(fname,'.fits')
     short_file_name=STRMID(fname,first_char,last_char-first_char)


     OPENW,lun,'PHOTOMETRY/Phot_'+short_file_name+'.txt',/get_lun

     printf,lun,OBJECT
     printf,lun,short_file_name
     if N_Params() gt 1  then begin
         first_char    = STRPOS(skyname,'/MIDI.20')+6
         last_char     = STRPOS(skyname,'.fits')
         printf,lun, STRMID(skyname,first_char,last_char-first_char), $
           '	name of sky'
     endif

     printf,lun, DIT,'	Time exposure in s'

     printf,lun, a[4]+FOV_x1,a[5]+FOV_y1, '	Star position in Beam A'
     printf,lun, b[4]+FOV_x1,b[5]+FOV_y1, '	Star position in Beam B'
     printf,lun,rA,'	Aperture for Beam A in pixels'
     printf,lun,rB,'	Aperture for Beam B in pixels'

     printf,lun,magsA[0]  *145/DIT,'	Flux A in Photons/s'
     printf,lun,errapA[0] *145/DIT,'	error Flux A'
     printf,lun,skyA[0]   *145/DIT,'	sky A'
     printf,lun,skyerrA[0]*145/DIT,'	error sky A'

     printf,lun,magsB[0] *145/DIT,'	Flux B in Photons/s'
     printf,lun,errapB[0]*145/DIT,'	error Flux B'
     printf,lun,skyB[0] *145/DIT,'	sky B'
     printf,lun,skyerrB[0]*145/DIT,'	error sky B'

     printf,lun,rA+1,'	Aperture for Beam A in pixels'
     printf,lun,rB+1,'	Aperture for Beam B in pixels'

     printf,lun,magsA[1]*145/DIT,'	Flux A in Photons/s'
     printf,lun,errapA[1]*145/DIT,'	error Flux A'

     printf,lun,magsB[1]*145/DIT,'	Flux B in Photons/s'
     printf,lun,errapB[1]*145/DIT,'	error Flux B'

     CLOSE,lun & free_lun,lun

;help,maskA,maskB
oirNewData, hdrname, make_maskname(fname), $
  { data1: maskB[*,*,0], data2: maskA[*,*,0] }
; the data for the mask is a struct with entries data1 and data2
; channel B is from win1, which is from data1


;draw the acquisition images and the apertures

wset,0

PLOTS,CIRCLE(a[4]*factor,a[5]*factor,rA*factor),/DEVICE
PLOTS,CIRCLE(a[4]*factor,a[5]*factor,(rA+6)*factor),/DEVICE
PLOTS,CIRCLE(a[4]*factor,a[5]*factor,(rA+8)*factor),/DEVICE
PLOTS,CIRCLE((b[4]+n[1])*factor,b[5]*factor,rB*factor),/DEVICE
PLOTS,CIRCLE((b[4]+n[1])*factor,b[5]*factor,(rB+6)*factor),/DEVICE
PLOTS,CIRCLE((b[4]+n[1])*factor,b[5]*factor,(rB+8)*factor),/DEVICE

ende:

end
