;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
; fringe_phot.pro
; Created:     Thu Apr 10 15:01:39 2003 by Koehler@sun47
; Last change: Sun Oct 31 19:29:06 2004
;
; PURPOSE:
;	save the world
;
; CALLING SEQUENCE:
;
;
; INPUT:
;	fringefile	MIDI file of fringe search or tracking
;	photfile	MIDI file with chopping data
;
; OUTPUT:
;
;
PRO fringe_phot, fringefile, photfile, MOVIE=movie

    fringefile= fringefile[0]
    photfile  = photfile[0]

    first_char    = STRPOS(fringefile,'/MIDI.20')+6
    last_char     = STRPOS(fringefile,'.fits')
    short_fringefile= STRMID(fringefile,first_char,last_char-first_char)

    maskfile = make_maskname(photfile)
    print,"Maskfile is ",maskfile

    short_maskfile= STRMID(maskfile, 11) ;rm "PHOTOMETRY/"

    ;fringe = midiRawFringe(fringefile,maskfile,time,opd)

    ;comprfile = 'PHOTOMETRY/CompressedFringe.fits'
    comprfile = 'PHOTOMETRY/Compr_'+short_fringefile+'_'+short_maskfile

    if file_test(comprfile) then begin
        print, comprfile, " exists"
    endif else begin
        cmd = getenv("MIAroot") + '/bin/oirCompress "'+ $
          fringefile +'" '+ maskfile +' '+ comprfile
        print, ''
        print, cmd
        spawn, cmd
    endelse
    print, ''
    print, 'getting data...'
    data = oirgetdata(comprfile)

    ; the following was (partly) stolen from midiRawFringe
    dtime = 86400. * (data[1].time[0] - data[0].time[0])
    tZero = 1.
    nZero = (tZero/dtime) > 10
    ;print,"dtime ",dtime,", nzero ",nzero

    ; we have to use the original filenames because oircompress
    ; removes all the interesting things from the header
    filename = oirFileList(fringefile)

    ;   figure out scan size from header
    im = OBJ_NEW('imagedata', filename[0])
    scanLength = (im->head())->getpar('maxstep')

    ;   figure out channel location from OPT1
    beam = (im->prihead())->getpar('INS OPT1')
    OBJ_DESTROY,im

    ;print, "nZero",nzero,", scanlength",scanlength,", beam is ",beam
    ;    calculate scan size data
    nData = N_ELEMENTS(data)
    nScan = nData/scanLength
    if (beam EQ 'HIGH_SENS') then fringe = data.data1-data.data2 $
    else if (beam EQ 'SCI_PHOT') then fringe = data.data2 - data.data3

    opd = data.localOpd[1]-data.localOpd[0] + data.opd[1]-data.opd[0]

    ; 1st two points are garbage
    fringe[0:1] = MEDIAN(fringe[2:200])
    ; last two opd points are garbage
    opd[nData-2:nData-1] = median(opd[nData-scanLength:nData-2])

    ; remove low freq trend
    fringe = fringe - SMOOTH(fringe, nZero,/EDGE_TRUNCATE)

    ; reform into scans, droping last fractional scan if any
    nFringe= LONG(scanLength)*LONG(nScan)
    fringe = REFORM(fringe(0:nFringe-1), scanLength, nScan,/overwrite)
    opd    = REFORM(opd(0:nFringe-1), scanLength, nScan,/overwrite)

    ;help,opd,fringe

    ampl = fltarr(nScan,/nozero)

    xrange = [ min(opd), max(opd) ]
    yrange = [ min(fringe), max(fringe) ]
    for f=0,nScan-1 do begin
        ampl[f] = max(abs(fringe[*,f]))
                                ;print,ampl[f]
        if Keyword_set(movie) then begin
            plot, opd[*,f], fringe[*,f], xrange=xrange, yrange=yrange
            wait, .02
        endif
    endfor
    wait, .3
    multi = !P.multi
    !p.multi=[0, 0, 2, 0, 0]
    plot, ampl,    xtitle = "Scan no.", title = "One-sided Fringe Amplitude"
    plot, opd[scanLength/2,*],xtitle="Scan no.", title="mid-opd of scan"
    !P.multi = multi

    print,"Maximum amplitude: ", max(ampl)
end
