;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;; From chesneau Fri Mar  7 09:58:23 2003
;; Date: Fri, 7 Mar 2003 09:58:22 +0100 (MET)
;; From: Olivier Chesneau <chesneau>
;;
;; USAGE: make_midi_log, '/disk-c/mididata/2003-02-23'
;;
;; writes important header information of all MIDI.*[0-9].fits files
;; in the given directory to the file "logbook.txt" in the current
;; directory.
;;
;; UPDATE: name of the logbook can be given via "LOGBOOK" parameter
;;
;; BUGS: Crashes if one of the files is broken (i.e. not a valid FITS-file)
;;	 Just forget giving it a directory whose name starts with a number!
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO make_midi_log, select_dir, LOGBOOK = logbook

  select_files = select_dir + "/MIDI.*[0-9].fits"
  ;print,"select_files ",select_files

  files_found=FINDFILE(select_files, COUNT=nr_files)
  files_found=STRCOMPRESS(files_found,/remove_all)
  ;print,"files_found ",files_found

  ;nr_files=N_ELEMENTS(files_found)
  ;print,"nr_files ",nr_files

  if nr_files le 0 then begin
      print, "No MIDI-files found in ", select_dir
  endif else begin
      IF (N_ELEMENTS(logbook) EQ 0) THEN logbook= 'logbook.txt'

    OPENW,lun,logbook,/get_lun

    printf,lun,'--------------------------------------------------------------------------------------------'
    printf,lun,'                               ', select_dir
    printf,lun,'--------------------------------------------------------------------------------------------'
    printf,lun,'short_file_name		       OBJECT OPT1 GRIS  FILT   SHUT     NRTS_MODE       DIT   NDIT'
    ;		123456789.123456789.123456|1234567890|1234|1234|123456|123456|123456789.12345|12345678|1234
    ;		2003-12-34T01:23:45.000_01   HD123456
    printf,lun,'--------------------------------------------------------------------------------------------'

    FOR index_file=0,nr_files-1 DO BEGIN

      ; check whether the file is a legal file
      index_hdr=STRPOS(files_found[index_file],'99')
      IF (index_hdr EQ -1) THEN BEGIN
        im = obj_new('fitsfile',files_found[index_file])
        he = im->prihead()
        TPL_ID        = STRCOMPRESS(he->getpar('TPL ID'),/remove_all)
        ;OBJECT       = STRCOMPRESS(he->getpar('OBJECT'),/remove_all)
        OBJECT        = STRCOMPRESS(he->getpar('OBS TARG NAME'),/remove_all)
        RA            = STRCOMPRESS(he->getpar('RA'),/remove_all)
        DEC           = STRCOMPRESS(he->getpar('DEC'),/remove_all)
  ;
        nexp          = STRCOMPRESS(he->getpar('TPL NEXP'),/remove_all)
        expno         = STRCOMPRESS(he->getpar('TPL EXPNO'),/remove_all)
        DET_DIT       = STRCOMPRESS(he->getpar('DET DIT'),/remove_all)
        DET_NDIT      = STRCOMPRESS(he->getpar('DET NDIT'),/remove_all)
        DPR_TYPE      = STRCOMPRESS(he->getpar('DPR TYPE'),/remove_all)
        DPR_TECH      = STRCOMPRESS(he->getpar('DPR TECH'),/remove_all)
        DPR_CATG      = STRCOMPRESS(he->getpar('DPR CATG'),/remove_all)
        DET_NRTS_MODE = STRCOMPRESS(he->getpar('DET NRTS MODE'),/remove_all)
        INS_GRIS_NAME = STRCOMPRESS(he->getpar('INS GRIS NAME'),/remove_all)
        INS_FILT_NAME = STRCOMPRESS(he->getpar('INS FILT NAME'),/remove_all)
        INS_OPT1_NAME = STRCOMPRESS(he->getpar('INS OPT1 NAME'),/remove_all)
        INS_SHUT_NAME = strcompress(he->getpar('INS SHUT NAME'),/remove_all)
  ;
        first_char    = STRPOS(files_found[index_file],'/MIDI.20')+6
        last_char     = STRPOS(files_found[index_file],'.fits')
        short_file_name=STRMID(files_found[index_file],first_char,last_char-first_char)
        printf,lun,short_file_name, OBJECT,$
          INS_OPT1_NAME, INS_GRIS_NAME, INS_FILT_NAME, INS_SHUT_NAME, DET_NRTS_MODE,$
  ;  index_file    I4,X
        DET_DIT,DET_NDIT, $
              FORMAT='(A26,X,A10,X,A4,X,A4,X,A6,X,A6,X, A15,X,D8.3,X, I4 )'
  ;             INS_FILT_NAME,INS_OPT1_NAME,DET_DIT,DET_NDIT, $
        ;PRINT,index_file,'	',short_file_name --- too confusing ;-)
        PRINT,'	',short_file_name
        obj_destroy,im

      ENDIF
    ENDFOR
    CLOSE,lun & free_lun, lun
  ENDELSE
END
