;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: xmidimovie.pro,v 1.3 2003/09/09 09:56:17 koehler Exp $
;
; xmidimovie.pro
; Created:     Thu Jun  5 10:46:20 2003 by Koehler@sun47
; Last change: Mon Jun  7 11:02:40 2004
;
; based on draw_widget_example from idl 5.6
;
; PURPOSE:
;	play fringe movies
;
; CALLING SEQUENCE:
;	xmidimovie, x, y
;
; INPUT:
;	x	1- or 2D array of x-coordinates
;	y	2D-array of y-coordinates
;		xmidimovie will plot each slice of y in turn
;
; OUTPUT:
;	pretty pictures on the screen
;
PRO xmidimovie_event, ev

  COMPILE_OPT HIDDEN

  COMMON xmidimovie, xarray, yarray

  sz = size(yarray)
  last = sz[2]-1

  ; Retrieve the anonymous structure contained in the user value of
  ; the top-level base widget.

  WIDGET_CONTROL, ev.TOP, GET_UVALUE=st

  ; Set the draw widget as the current IDL drawable.

  WSET, st.drawID

  ; if it's one of our buttons or sliders, react

  case (ev.ID) OF
      st.play: BEGIN
          if (st.playing eq 0) then begin
              st.playing = 1
              WIDGET_CONTROL, st.draw, TIMER=0.01
              WIDGET_CONTROL, st.play, SET_VALUE='Pause'
          endif else begin
              st.playing = 0
              WIDGET_CONTROL, st.play, SET_VALUE='Play'
          endelse
      end
      st.gotobeg: st.current= 0
      st.backw:   st.current= st.current-1
      st.forw:    st.current= st.current+1
      st.gotoend: st.current= last

      st.frameno: st.current= ev.value
      st.speed:   st.delay = ev.value
      st.about:   m=dialog_message(DIALOG_PARENT=ev.top,/INFORMATION,$
                                    ["Written by Rainer Koehler",$
                                     "Moral support by Friedrich vom Stein"])
      else: ;relax
  endcase

  if st.current lt 0 then  st.current= last
  if st.current gt last then  st.current= 0

  ; plot the current frame before incrementing the no.

  if (size(xarray,/N_Dimensions) gt 1) then begin
      x= xarray[*,st.current]
  endif else x = xarray

  plot, x, yarray[*,st.current], $
        xrange= st.xrange, yrange= st.yrange, xtitle=st.xtitle, ytitle=st.ytitle

  ; If it is a timer event, go to the next frame and restart the timer

  IF (TAG_NAMES(ev, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') $
    and (st.playing ne 0) THEN BEGIN
      st.current = st.current+1
      WIDGET_CONTROL, ev.ID, TIMER= st.delay/1000.
  ENDIF

  ; Reset the user value of the top-level base widget to the modified
  ; st structure.  And set the slider to the new frame no.

  WIDGET_CONTROL, ev.TOP, SET_UVALUE=st
  WIDGET_CONTROL, st.frameno,SET_VALUE=st.current

  ; If the event is from the quit button, commit suicide

  IF (ev.ID EQ st.quitID) THEN  WIDGET_CONTROL, ev.TOP, /DESTROY
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidimovie, xarray_in, yarray_in, XTITLE=xtitle, YTITLE=ytitle

  COMMON xmidimovie, xarray, yarray

  xarray = xarray_in
  yarray = yarray_in

  if not keyword_set(xtitle) then xtitle = ""
  if not keyword_set(ytitle) then ytitle = ""

  !P.multi = 0	; avoid multiple plots per page

  ; Create a base widget containing a draw widget and a sub-base
  base  = Widget_Base(Title="Mozzarella", /COLUMN)
  draw  = Widget_Draw(base, XSIZE=720, YSIZE=405)
  baseh = Widget_Base(base, Space=0,/ROW,/ALIGN_RIGHT,/BASE_ALIGN_BOTTOM)
  basev = Widget_Base(baseh,Space=0,/COLUMN)
  base2 = Widget_Base(basev, /ROW)
  ;label = Widget_Label(base2, XSIZE=200)
  speed  = Widget_Slider(base2, TITLE="Delay", MAXIMUM=1000, VALUE=100, xsize=80)
  frameno= Widget_Slider(base2,/DRAG, TITLE="Frame No.", xsize=560, $
                        MAXIMUM=N_elements(yarray_in[0,*])-1, VALUE=0)

  base3  = Widget_Base(basev, /ROW, /ALIGN_CENTER)
  gotobeg= Widget_Button(base3, VALUE='|<<',  xsize=80, ysize=42)
  backw  = Widget_Button(base3, VALUE=' < ',  xsize=80, ysize=42)
  play   = Widget_Button(base3, VALUE='Pause',xsize=80, ysize=42)
  forw   = Widget_Button(base3, VALUE=' > ',  xsize=80, ysize=42)
  gotoend= Widget_Button(base3, VALUE='>>|',  xsize=80, ysize=42)
  quit   = Widget_Button(base3, VALUE='Quit', xsize=80, ysize=42)

  expert = 0
  expertname= getenv('vltidl')
  if (expertname ne "") then $
    expertname= expertname + "/ews/midi/heidelberg/"
  expertname= expertname + "expert.bmp"
  ;print,"expert ",expertname
  if file_test(expertname) then begin
      ;print,"expert found"
      ;read_ppm, expertname, expert
      ;help,expert
      ;expert= reverse(transpose(expert,[1,2,0]),2)
      ;help,expert
      expert= Widget_Button(baseh, VALUE=expertname,/Bitmap)
  endif else begin
      print,"expert not found, running in dumb mode"
  endelse

  ; Realize the widget hierarchy, then retrieve the widget ID of the
  ; draw widget.
  WIDGET_CONTROL, base, /REALIZE
  WIDGET_CONTROL, draw, GET_VALUE=drawID

  ; Set the timer value of the draw widget.
  WIDGET_CONTROL, draw, TIMER=0.0

  ; Store the widget ID of several widgets and some other important
  ; stuff, and set the user value of the top-level base widget to
  ; this structure.
  st = { gotobeg:gotobeg, backw:backw, play:play, forw:forw, gotoend:gotoend,$
         draw:draw, drawID:drawID, quitID:quit, speed:speed, frameno:frameno,$
         about:expert, playing:1, current:0, delay:10, $
         xrange: [ min(xarray_in), max(xarray_in) ], xtitle:xtitle, $
         yrange: [ min(yarray_in), max(yarray_in) ], ytitle:ytitle }
  WIDGET_CONTROL, base, SET_UVALUE=st

  ; Register the widget with the XMANAGER.
  XMANAGER, 'xmidimovie', base, /NO_BLOCK

  ; Set some display device parameters.
  ;DEVICE, RETAIN=2, DECOMPOSED=0
END
