;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: xmidispvisi.pro,v 1.8 2003/11/08 06:52:55 koehler Exp $
;
; xmidivisi.pro
; Created:     Thu Jun 12 01:13:57 2003 by Koehler@rommie
; Last change: Mon Nov  1 17:07:22 2004
;
; PURPOSE:
;	save the world (or a small part of it)
;
; CALLING SEQUENCE:
;	x->obj_new('xmidivisi', fringefile, photfileA, photfileB)
;	*** (there are more options, use the source!) ***
;
; SHORTCUT:
;	x = xmdv(files)
;	same options as in the oo case
;
; INPUT:
;	fringefile:	MIDI file with fringe search or track data
;	photfileA/B:	MIDI files with chopping data, shutter A/B
;			open, respectively.
;
;	/RUNNING_HIPASS apply hi-pass filter by subtracting
;			boxcar-smoothed fringes.  Default is to
;			subtract mean of each scan from itself.
;
; OUTPUT:
;	an object that can do wonderful things
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Helper-routine to provide dimension-keyword with old idl-versions
;
Function FFTdim,input,Dimension=dim

 if !Version.release ge 5.5 then return, FFT(input,Dimension=dim)

 ;print,"Using FFT-workaround for ancient IDL-versions"
 sz = size(input)
 output= complex(input)

 if sz[0] eq 2 then begin
     if dim eq 1 then begin
         for i=0,sz[2]-1 do  output[*,i]= FFT(input[*,i])
     endif else if dim eq 2 then begin
         for i=0,sz[1]-1 do  output[i,*]= FFT(input[i,*])
     endif else print,"No such dimension:",dim
 endif else if sz[0] eq 3 then begin
     if dim eq 1 then begin
         print,"dim 1 of 3"
         for i=0,sz[2]-1 do $
           for j=0,sz[3]-1 do  output[*,i,j]= FFT(input[*,i,j])
     endif else if dim eq 2 then begin
         print,"dim 2 of 3"
         for i=0,sz[1]-1 do $
           for j=0,sz[3]-1 do  output[i,*,j]= FFT(input[i,*,j])
     endif else if dim eq 3 then begin
         print,"dim 3 of 3"
         for i=0,sz[1]-1 do $
           for j=0,sz[2]-1 do  output[i,j,*]= FFT(input[i,j,*])
     endif else print,"No such dimension:",dim
 endif
 return,output
end
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO xmidispvisi__Define
dings = { xmidispvisi, $
          fringefile: '', photfiles: ['',''], maskfile: '', $
          objectname: '', $
          PhFilt: '', FrFilt: '', Beam: '', DIT: 0., $
          scanLength: 0, totopd: 0., $
          longwl: 0, shortwl: 0, lambda: fltarr(320), $
          trace1: replicate({midisptrace},2), trace2: replicate({midisptrace},2),$
          traceFr1: {midisptrace}, traceFr2: {midisptrace},$
          specA1: fltarr(320), specA2: fltarr(320), $
          specB1: fltarr(320), specB2: fltarr(320), specAB: fltarr(320),$
          opd: ptr_new(), flux1: ptr_new(), flux2: ptr_new(),$
          fringe: ptr_new(), foufri: ptr_new(), amplfou: ptr_new(), $
          goodthresh: -1., oldgthres: -1., goodfringes: ptr_new(), Ngood: -1, $
          badthresh:  -1., oldbthres: -1.,  badfringes: ptr_new(), Nbad: -1, $
          noiseopd: 1e-3, $
          la_start: 0, la_end: 128, la_step: 4, la_width: 4, $
          $
          menu: -1, $
          topdraw: -1, topdrawID: -1, $
          lftdraw: -1, lftdrawID: -1, $
          rgtdraw: -1, rgtdrawID: -1, $
          goodthl: -1, badthl: -1, $
          sigamp: -1, noiamp: -1, corramp: -1, corrflx: -1, visibi: -1, $
          print:  -1, frimov: -1, foumov: -1, wrtlog: -1, quit: -1, $
          color: lonarr(6), multi: intarr(5), button: 0 $
}
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::get_objectname
  return, self.objectname
end

Function xmidispvisi::get_filename, PHOT=phot, MASK=mask

  if keyword_set(phot) then return, self.photfiles
  if keyword_set(mask) then return, self.maskfile
  return, self.fringefile
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::fringes
  sz = size(*self.opd)
  result = fltarr(2,sz[1],sz[2])
  result[0,*,*] = *self.opd
  result[1,*,*] = total(*self.fringe,1)
  return, result
end

Function xmidispvisi::whitefringeampl
  return, *self.amplfou
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::get_thresholds
  return, [ self.goodthresh, self.badthresh, self.noiseopd ]
end

Pro xmidispvisi::set_thresholds, good, bad, noiseopd
  self.goodthresh = good
  self.badthresh  = bad
  if N_params() gt 2 then self.noiseopd = noiseopd
  if Widget_info( long(self.topdraw),/Valid_id) then begin
      self->update_gui
  endif else self->update_GoodBad
end

Function xmidispvisi::get_percent_goodbad
  nScans = N_elements(*self.amplfou)
  return, [ 100.*self.Ngood/nScans, 100.*self.Nbad/nScans ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::plot_trace, iFile, x_size, mtrace, NOCOLOR=nocolor

	;    Fringes PhotA PhotB
  symbols= [ 2,	     1,    7	]

  rng = mtrace.range[0:mtrace.nrange]
  tr = make_trace(x_size,mtrace.trace_coeff,mtrace.sigma_coeff)

  if keyword_set(nocolor) then begin
      oplot, rng, mtrace.trace[rng], psym=symbols[iFile]
      oplot, tr[0,*], linestyle=iFile
      oplot, tr[1,*], linestyle=iFile
      oplot, tr[2,*], linestyle=iFile
  endif else begin
      oplot, rng, mtrace.trace[rng], psym=symbols[iFile], color= self.color[iFile]
      oplot, tr[0,*], color= self.color[iFile]
      oplot, tr[1,*], color= self.color[iFile]
      oplot, tr[2,*], color= self.color[iFile]
  endelse
end

PRO xmidispvisi::plot_traces, mtraces, TITLE=title, NOCOLOR=nocolor

  nTraces= N_elements(mtraces)
  x_size = mtraces[0].xsize
  mint = 999
  maxt = 0
  for i=0,nTraces-1 do begin
      if mtraces[i].trace_coeff[0] ne 0 then begin
          tr= make_trace(x_size, mtraces[i].trace_coeff, mtraces[i].sigma_coeff)
          mint = min([ mint, reform(tr[1,*]) ])
          maxt = max([ maxt, reform(tr[2,*]) ])
      endif
  endfor

  plot, [0,x_size], [mint,maxt], TITLE=title, /nodata
  for i=0,nTraces-1 do $
    if mtraces[i].trace_coeff[0] ne 0 then begin
      print,"trace",i,", color=",self.color[i]
      self->plot_trace, i, x_size, mtraces[i],NOCOLOR=nocolor
    endif
end


Function xmidispvisi::get_trace, iFile, WIN1=win1, WIN2=win2
  if not keyword_set(win2) then begin
      if (iFile<0) then return, self.traceFr1
      return, self.trace1[iFile]
  endif else begin
      if (iFile<0) then return, self.traceFr2
      return, self.trace2[iFile]
  endelse
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; this is here for historic reasons.  The C-program ist faster...
;;
Function xmidispvisi::FTscans
  t0 = systime(1)
  nScan  = N_elements(*self.amplfou)
  print,nScan," scans"
  ;
  ; find out how large the frames are
  ; Note that MIDIimgdata has only readRowsf, not readRows
  ;
  imgObj = obj_new('MIDIimgdata', self.fringefile, ierr=ierr)
  firstframe = imgObj->readRowsf(1)

  sz1 = size(firstframe.data1)
  sz2 = size(firstframe.data2)
  scandata1 = fltarr(self.scanlength,sz1[1],sz1[2])
  scandata2 = fltarr(self.scanlength,sz2[1],sz2[2])

  result = replicate( { data1: fltarr(sz1[1],sz1[2]),$
                        data2: fltarr(sz2[1],sz2[2]) }, nScan)
  iRow = 1	;rows in fbts are 1-relative!

  ; where we expect the fringes in fourier space as function of x
  freq  = self.totopd / self.lambda
  wfreq1= ceil(freq) - freq
  wfreq2= freq - floor(freq)
  freq  = fix(freq)
  print,"totopd:",self.totopd,", frequenzen:"
  print,freq,format='(I3)'

  for iScan= 0, nScan-1 do begin
      print,Format='("Scan",I4," - ",$)',iScan
      if (where( *self.goodfringes eq iScan) lt 0) then begin
          print,"boring"
          iRow = iRow + self.scanlength
      endif else begin
          print, "good,", iRow,". row"
          for i= 0, self.scanlength-1 do begin
              data = imgObj->readRowsf(iRow, ierr=ierr)
              iRow = iRow + 1
              scandata1[i,*,*] = data.data1
              scandata2[i,*,*] = data.data2
          endfor
          ;
          ; FFT along frame-no
          scandata1 = abs(FFTdim( scandata1, Dimension=1))
          scandata2 = abs(FFTdim( scandata2, Dimension=1))
          ; and pick the right freq
          for x= self.la_start, self.la_end+self.la_width-1 do begin
              result[iScan].data1[x,*]= 2 * $
                (scandata1[freq[x],x,*]*wfreq1[x] + scandata1[freq[x]+1,x,*]*wfreq2[x])
              result[iScan].data2[x,*]= 2 * $
                (scandata2[freq[x],x,*]*wfreq1[x] + scandata2[freq[x]+1,x,*]*wfreq2[x])
          endfor
      endelse
  endfor
  Obj_destroy,imgObj
  print, systime(1)-t0, " seconds"
  return, result
end

Function xmidispvisi::FTframe, MASKWD=maskwd, SHOW=show, NOCOLOR=nocolor

  if not keyword_set(maskwd) then maskwd=1.0

  FTscanname= getenv("MIDIFTSCANDIR") + $
    "/FTscan_" + MIDIname_datime(self.fringefile) + ".fits"

  if not file_test(FTscanname) then begin
      print,"Computing Powerspectra of each pixel..."
      cmd = getenv("MIAroot") + '/bin/midiFTscans'
      if file_test(cmd,/EXECUTABLE) then begin
          spawn, cmd + ' "' + self.fringefile + '" ' + FTscanname
          result = oirgetdata(FTscanname)
      endif else begin
          print, cmd + " not found, falling back on IDL code"
          print, "This has not been tested!!!"
          result = self->FTscans()
      endelse
  endif else result = oirgetdata(FTscanname)

  sz1 = size(result[0].data1)
  sz2 = size(result[0].data2)

  ; integrate over good scans (if set) or all scans
  if (self.Ngood gt 0) then begin
      print,"Adding FT data of good scans"
      sum = { data1: total( result[*self.goodfringes].data1,3) / self.Ngood,$
              data2: total( result[*self.goodfringes].data2,3) / self.Ngood }
  endif else begin
      print,"Adding FT data of all scans"
      nData = N_elements(result)
      sum = { data1: total( result.data1,3) / nData,$
              data2: total( result.data2,3) / nData }
  endelse

  mtrace= {midisptrace}

  for iFile= 0, 1 < (N_elements(self.photfiles)-1) do begin
      print, "Reading photometry",iFile
      print, self.photfiles[iFile]
      chop_image= MIDIChopImageC(self.photfiles[iFile], before=1, after=1)
      win1 = chop_image[0].data1 - chop_image[1].data1
      win2 = chop_image[0].data2 - chop_image[1].data2

      ; create masks that are 3 times wider than std chop_nod_disp
      chop_nod_disp_one_window, win1,3.,2,1,mtrace,fx,fxc,photmask1,/silent
      self.trace1[iFile] = mtrace
      chop_nod_disp_one_window, win2,3.,2,1,mtrace,fx,fxc,photmask2,/silent
      self.trace2[iFile] = mtrace
  endfor
  result = replicate( { data1: fltarr(sz1[1],sz1[2]),$
                        data2: fltarr(sz2[1],sz2[2]) }, 3)
  result[0] = sum

  result[1].data1 = sum.data1
  result[1].data2 = sum.data2

  for i= 0,sz1[1]-1 do begin
      ;; we can't use the dimension-keyword
      ;; because idl5.4 doesn't support it
      result[1].data1[i,13] = median(result[1].data1[i,14:16])
      result[1].data2[i,13] = median(result[1].data2[i,14:16])
      result[1].data1[i,35] = median(result[1].data1[i,32:34])
      result[1].data2[i,35] = median(result[1].data2[i,32:34])
  endfor
  for i= 0,12 do begin
      result[1].data1[*,i] = result[1].data1[*,13]
      result[1].data2[*,i] = result[1].data2[*,13]
  endfor
  for i=36,sz1[2]-1 do begin
      result[1].data1[*,i] = result[1].data1[*,35]
      result[1].data2[*,i] = result[1].data2[*,35]
  endfor
  for i= 0,sz1[1]-1 do begin
      result[1].data1[i,*]= result[1].data1[i,*] - min(result[1].data1[i,*])
      result[1].data2[i,*]= result[1].data2[i,*] - min(result[1].data2[i,*])
  endfor

  if keyword_set(debug) then begin
      window,9
      !P.multi=[2,1,2]
      plot, [0,320],[0,max(result[1].data1[0:119,15:19])],/nodata
      for i=15,18 do oplot,result[1].data1[0:119,i],linestyle=i-15
      plot, [0,320],[0,max(result[1].data1[0:119,29:34])],/nodata
      for i=28,34 do oplot,result[1].data1[0:119,i],linestyle=i-28
  endif

  if keyword_set(show) then begin
      window,0,xsize=1210,ysize=500
      for i=0,1 do begin
          tvscl,rebin(result[i].data1, 604,164,/Sample),2*i
          tvscl,rebin(result[i].data2, 604,164,/Sample),2*i+1
      endfor
  endif
  ;noise = fltarr(sz2[1])
  ;for iCol=0,sz2[1]-1 do begin
  ;    m = mask2[iCol,*]
  ;    ;print,"mslice: ",m
  ;    ;print,"where : ",where(m lt 0.1)
  ;    noise[iCol] = mean( sum.data2[iCol,where(m lt 0.1)])
  ;    result[1].data2[iCol,*] = (sum.data2[iCol,*] - noise[iCol]) * m
  ;endfor
  ;print,"Noise: ",noise[11:15]
  ;print,"Result0:",total(result[0].data2[11:15,*],2)/sz2[2]
  ;print,"Result1:",total(result[1].data2[*,*],2)/sz2[2]

  ; now create a standard mask from the masked rms fringedata
  ; but use for tracing only the good part of the spectrum
  print,"Making fringe masks..."
  iFile = N_elements(self.photfiles)

  bgnoise = mean( [reform(result[1].data1[0:119,14:20],120*7),$
                   reform(result[1].data1[0:119,28:34],120*7) ])
  print, "background level in window 1: ",bgnoise

  chop_nod_disp_one_window, $
    result[1].data1, maskwd, 2,1, mtrace,fx,fxc, mask1,/silent, $
    XMIN=self.la_start, XMAX=self.la_end+self.la_width-1, BGNOISE=bgnoise
  self.traceFr1 = mtrace        ;ok="" & read,"Hit return",ok

  bgnoise = mean( [reform(result[1].data2[0:119,14:20],120*7),$
                   reform(result[1].data2[0:119,28:34],120*7) ])
  print, "background level in window 2: ",bgnoise

  chop_nod_disp_one_window, $
    result[1].data2, maskwd, 2,1, mtrace,fx,fxc, mask2,/silent, $
    XMIN=self.la_start, XMAX=self.la_end+self.la_width-1, BGNOISE=bgnoise
  self.traceFr2 = mtrace        ;ok="" & read,"Hit return",ok

  result[2].data1[0:sz1[1]-1,*] = mask1
  result[2].data2[0:sz2[1]-1,*] = mask2

  if keyword_set(show) then begin
      wset,0
      tvscl,rebin(result[2].data1, 604,164,/Sample),4
      tvscl,rebin(result[2].data2, 604,164,/Sample),5

      !P.multi=1
      window,3,title="Traces Window 1"
      self->plot_traces, [self.traceFr1, self.trace1[0], self.trace1[1]],$
        title="Window 1", NOCOLOR=nocolor
      rng = self.traceFr1.range[0:self.traceFr1.nrange]
      oplot, rng, self.traceFr1.trace[rng] + self.traceFr1.FWHM[rng], psym=3
      oplot, rng, self.traceFr1.trace[rng] - self.traceFr1.FWHM[rng], psym=3

      window,1,title="Traces Window 2"
      self->plot_traces, [self.traceFr2, self.trace2[0], self.trace2[1]],$
        title="Window 2", NOCOLOR=nocolor
      rng = self.traceFr2.range[0:self.traceFr2.nrange]
      oplot, rng, self.traceFr2.trace[rng] + self.traceFr2.FWHM[rng], psym=3
      oplot, rng, self.traceFr2.trace[rng] - self.traceFr2.FWHM[rng], psym=3
  endif
  return, result
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::meanmask, WIDTH=width, SHOW=show

  if not keyword_set(width) then width=1.

  print,"Maskwidth is",width

  mtrace= {midisptrace}
  mean_tcoeff1= fltarr(3) & mean_tcoeff2= fltarr(3)
  mean_scoeff1= fltarr(2) & mean_scoeff2= fltarr(2)

  for iFile= 0, 1 < (N_elements(self.photfiles)-1) do begin
      print, "Reading photometry",iFile
      print, self.photfiles[iFile]

      if iFile eq 0 then begin
          self.maskfile= self.maskfile+"_"+MIDIname_datime(self.photfiles[0])
      endif else begin
          self.maskfile= self.maskfile+"+"+MIDIname_time(self.photfiles[iFile])
      endelse
      chop_image= MIDISkySubImage(self.photfiles[iFile], before=1, after=1)

      chop_nod_disp_one_window, chop_image.int1,1.,2,1,mtrace,fx,fxc,photmask1,/silent
      self.trace1[iFile] = mtrace
      mean_tcoeff1 = mean_tcoeff1 + mtrace.trace_coeff
      mean_scoeff1 = mean_scoeff1 + mtrace.sigma_coeff

      chop_nod_disp_one_window, chop_image.int2,1.,2,1,mtrace,fx,fxc,photmask2,/silent
      self.trace2[iFile] = mtrace
      mean_tcoeff2 = mean_tcoeff2 + mtrace.trace_coeff
      mean_scoeff2 = mean_scoeff2 + mtrace.sigma_coeff
  endfor
  iFile = N_elements(self.photfiles)
  self.traceFr1.trace_coeff = mean_tcoeff1 / iFile
  self.traceFr1.sigma_coeff = mean_scoeff1 / iFile
  self.traceFr2.trace_coeff = mean_tcoeff2 / iFile
  self.traceFr2.sigma_coeff = mean_scoeff2 / iFile

  sz1= size(chop_image.int1) & mask1= fltarr(sz1[1],sz1[2])
  sz2= size(chop_image.int2) & mask2= fltarr(sz2[1],sz2[2])
  ; the sizes should better be the same...

  self.traceFr1.xsize= sz1[1] & self.traceFr2.xsize= sz2[1]
  self.traceFr1.ysize= sz1[2] & self.traceFr2.ysize= sz2[2]
  self.traceFr1.trace =  poly(findgen(sz1[1]),self.traceFr1.trace_coeff)
  self.tracefr2.trace =  poly(findgen(sz2[1]),self.traceFr2.trace_coeff)
  sigm2_fit1 = (poly(findgen(sz1[1]),self.traceFr1.sigma_coeff)*width)^2 * 2.
  sigm2_fit2 = (poly(findgen(sz2[1]),self.traceFr2.sigma_coeff)*width)^2 * 2.
  y = findgen(sz1[2])
  FOR i= 0, sz1[1]-1 DO BEGIN
      mask1[i,*]= exp( -(y-self.traceFr1.trace[i])^2. / sigm2_fit1[i])
      mask2[i,*]= exp( -(y-self.traceFr2.trace[i])^2. / sigm2_fit2[i])
  endfor
  self.traceFr1.FWHM = sqrt(sigm2_fit1 * alog(2))
  self.traceFr2.FWHM = sqrt(sigm2_fit2 * alog(2))

  self.traceFr1.nRange= sz1[1] & self.traceFr1.range = findgen(sz1[1])
  self.traceFr2.nRange= sz2[1] & self.traceFr2.range = findgen(sz2[1])

  if keyword_set(show) then begin
      ;rmsdata = Cached_MeanRMS(self.fringefile)
      ;wset,0
      ;tvscl,rebin(rmsdata[1].data1, 604,164,/Sample),4
      ;tvscl,rebin(rmsdata[1].data2, 604,164,/Sample),5

      !P.multi=1
      if (show eq "ps") then begin
	set_plot,"ps"
	device,file="traces_win1.ps",/landscape,/color
	TVLCT, [0,25,255,0,250], [0,25,0,255,250], [0,127,0,0,0]
	!P.thick = 5
      	!x.thick= 5
     	!y.thick= 5
	!P.charthick= 5
      endif else $
        window,3,title="Traces Window 1"
      self->plot_traces, [self.traceFr1, self.trace1[0], self.trace1[1]],$
        title="Window 1", NOCOLOR=0
      rng = self.traceFr1.range[0:self.traceFr1.nrange]
      oplot, rng, self.traceFr1.trace[rng] + self.traceFr1.FWHM[rng], psym=3
      oplot, rng, self.traceFr1.trace[rng] - self.traceFr1.FWHM[rng], psym=3

      if (show eq "ps") then begin
	device,file="traces_win2.ps",/landscape,/color
      endif else $
        window,1,title="Traces Window 2"
      self->plot_traces, [self.traceFr2, self.trace2[0], self.trace2[1]],$
        title="Window 2", NOCOLOR=0
      rng = self.traceFr2.range[0:self.traceFr2.nrange]
      oplot, rng, self.traceFr2.trace[rng] + self.traceFr2.FWHM[rng], psym=3
      oplot, rng, self.traceFr2.trace[rng] - self.traceFr2.FWHM[rng], psym=3

      if (show eq "ps") then begin
	device,/close
      	!P.thick = 0
      	!x.thick= 0
     	!y.thick= 0
	!P.charthick= 0
	set_plot,"X"
      endif
  endif

  if (N_Tags(chop_image) gt 2) then begin
      print,"Four channels in chop-img, must be Sci-Phot"
      return, { data1: mask1, data2: mask1, data3: mask2, data4: mask2 }
      ; we do not create 4 masks, data1 and data4 are just dummies
  endif else begin
      print,"Two channels in chop-img, must be Hi-Sense"
      return, { data1: mask1, data2: mask2 }
  endelse
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; there's currently no way to specify a different sky for photfileB,
;; but we haven't used nodding with shutter A/B open anyway...
;;
PRO xmidispvisi::do_photometry, photfile, maskdata, PhFilt, spec1,spec2,$
               PHOTFILEB=photfileB, MYSKY=mysky, SKY=sky, $
               REVERSE=reverse, BEFORE=before, AFTER=after, DEBUG=debug

  if not keyword_set(before)  then before  = 1
  if not keyword_set(after)   then after   = 1

  im = obj_new('fitsfile',(oirFileList(photfile))[0])
  he = im->prihead()
  PhFilt= strcompress(he->getpar('INS FILT NAME'),/remove_all)
  Grism = strcompress(he->getpar('INS GRIS NAME'),/remove_all)
  DIT   = strcompress(he->getpar('DET DIT'),/remove_all)
  obj_destroy,im

  if Grism eq 'OPEN' then $
    print,"This is undispersed data.  I'm not sure I can handle that..."

  if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'

  if keyword_set(mysky) or keyword_set(sky) then begin
      if not keyword_set(mysky) then mysky=sky
      print, 'Reading nodding data...'
      print, mysky
      im_sky= Cached_MeanRMS(mysky)

      print, photfile
      im_on= Cached_MeanRMS(photfile)
      win1 = im_on[0].data1-im_sky[0].data1
      win2 = im_on[0].data2-im_sky[0].data2

      if keyword_set(photfileB) then begin
          print, photfileB
          im_on= Cached_MeanRMS(photfileB)
          win1 = sqrt(win1 * (im_on[0].data1-im_sky[0].data1) > 0.)
          win2 = sqrt(win2 * (im_on[0].data2-im_sky[0].data2) > 0.)
      endif
  endif else begin
      print, 'Reading Chopping data...'

      if keyword_set(reverse) then begin
          on = 1 & off = 0
      endif else begin
          on = 0 & off = 1
      endelse
      print, photfile
      chop_image= MIDISkySubImage(photfile, before=before, after=after)
      win1 = chop_image.int1
      win2 = chop_image.int2

      if keyword_set(photfileB) then begin
          print, photfileB
          chop_image= MIDISkySubImage(photfileB, before=before, after=after)
          win1 = sqrt(win1 * chop_image.int1 > 0.)
          win2 = sqrt(win2 * chop_image.int2 > 0.)
      endif
  endelse
  ; mask and sum along y-axis
  spec1 = total(win1 * maskdata.data1, 2) / DIT
  spec2 = total(win2 * maskdata.data2, 2) / DIT

  print,"total flux 1:",total(spec1)
  print,"total flux 2:",total(spec2)

  obj_destroy, im
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::skysubtract

    rmsname = "PHOTOMETRY/Mean_" + MIDIname_datime(self.fringefile) + ".fits"
    ; this gets rid of the other filenames in the list, too

    print,"rmsname is -", rmsname,"-"

    if not file_test(rmsname) then begin
        print,"Computing mean&rms..."
        if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'
        spawn, getenv("MIAroot") + '/bin/oirMeanRMS "' + self.fringefile + '" ' + rmsname
    endif
    rmsdata = oirgetdata(rmsname)

    chopimgA= MIDIChopImageC(self.photfiles[0], before=3, after=3)
    chopimgB= MIDIChopImageC(self.photfiles[1], before=3, after=3)

    rmsdata[1].data1 = rmsdata[0].data1 - (chopimgA[1].data1+chopimgB[1].data1)
    rmsdata[1].data2 = rmsdata[0].data2 - (chopimgA[1].data2+chopimgB[1].data2)
    rmsdata[0].data1 = rmsdata[0].data1 - (chopimgA[0].data1+chopimgB[0].data1)
    rmsdata[0].data2 = rmsdata[0].data2 - (chopimgA[0].data2+chopimgB[0].data2)

    return, rmsdata
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::plot_static, NOCOLOR=nocolor

  plot, *self.opd * 1e3, title= "Optical Path Difference",$
    xtitle= "Frame no.", ytitle= 'opd [mm]'

  if not keyword_set(nocolor) then begin
      nFrames= N_elements(*self.opd)
      meanopd= total(*self.opd,1)/self.scanlength
      ;oplot, rebin( meanopd, nFrames), color=self.color[1]
      med = median(meanopd)
      print,nFrames,med
      oplot, [0,nFrames], [med,med]*1e3, linestyle=2
      good = where( abs(meanopd - med) lt self.totopd*1e-6/2.,Ngood)
      if Ngood gt 0 then begin
          for iScan=0,Ngood-1 do begin
              goodi= good[iScan]*self.scanlength + indgen(self.scanlength)
              oplot, goodi, (*self.opd)[goodi]*1e3,color=self.color[1]
          endfor
      endif

      bad = where( abs(meanopd - med) gt self.noiseopd, Nbad)
      if Nbad gt 0 then begin
          for iScan=0,Nbad-1 do begin
              badi= bad[iScan]*self.scanlength + indgen(self.scanlength)
              oplot, badi, (*self.opd)[badi]*1e3,color=self.color[0]
          endfor
      endif
  endif

  maxflux= max( [ max(*self.flux1), max(*self.flux2) ] )
  minflux= min( [ min(*self.flux1), min(*self.flux2) ] )
  plot, *self.flux1, yrange=[minflux,maxflux], xtitle = "Frame no.", title = "Fluxes"
  oplot,*self.flux2, linestyle=1

  ;sz   = size(*self.fringe)
  ;scrsz= get_screen_size()
  ;wsz  = [ scrsz[0]-16, scrsz[1]/2-40 ]
  ;posx = 60
  ;posy = wsz[1]/2 + 30
  ;width= wsz[0]-posx-20
  ;nred = fix(sz[2]*sz[3] / width)+1	;factor to reduce length
  ;width= fix(sz[2]*sz[3] / nred)
  ;height=fix(sz[1]/2)
  ;print,"in: ",sz[2]*sz[3],", out: ",width
  ;fs = reform(*self.fringe,sz[1],sz[2]*sz[3])
  ;fs = rebin(fs[ 0:2*height-1, 0:nred*width-1], height, width)
  ;fs = fs-min(fs)
  ;tv,transpose(fs)*516./max(fs),posx,posy,/device
  ;plot, total(*self.fringe,1), $
  ;  position= [posx,posy,posx+width,posy+height], /device, $
  ;  xtitle = "Frame no.", title= "Raw Amplitude"
  plot, total(*self.fringe,1), xtitle = "Frame no.", title= "Flux Difference"
  self.multi = !P.multi

  plot, *self.amplfou, xtitle = "Scan no.", title = "Fourier Fringe Amplitude (linear)"
  if (self.Ngood gt 1) then begin
      nScans = N_elements(*self.amplfou)
      oplot, [0,nScans], [self.oldgthres,self.oldgthres],linestyle=1,color=0
      oplot, [0,nScans], [self.oldbthres,self.oldbthres],linestyle=1,color=0
  endif
  !P.multi = self.multi
end

PRO xmidispvisi::plot_goodbad

  ;!p.charsize= 1.
  ymax = max(*self.foufri)
  if (self.Ngood gt 0) then begin
      goodfou = (*self.foufri)[*,*self.goodfringes]
      plot, goodfou[*,0], yrange=[0,ymax], $
        title="Good Fringes", xtitle= "Frequency", ytitle="Spectral Power"
      if (self.Ngood gt 1) then begin
          for f=1,self.Ngood-1 do oplot, goodfou[*,f]
          oplot, total(goodfou,2)/self.Ngood, color=self.color[2]
      endif

      lowfreq = floor( self.totopd/self.longwl+0.5)
      highfreq= floor( self.totopd/self.shortwl+0.5)
      if (highfreq gt self.scanlength/2) then highfreq= self.scanlength/2
      oplot, [ lowfreq, lowfreq], [0,ymax], linestyle=2
      oplot, [highfreq,highfreq], [0,ymax], linestyle=2

  endif

  if (self.Nbad gt 0) then begin
      badfou = (*self.foufri)[*,*self.badfringes]
      plot, badfou[*,0], yrange=[0,ymax], $
        title="Bad Fringes", xtitle= "Frequency", ytitle="Spectral Power"
      if (self.Nbad gt 1) then begin
          for f=1,self.Nbad-1 do oplot, badfou[*,f]
          oplot, total(badfou,2)/self.Nbad, color=self.color[2]
      endif
  endif
end

PRO xmidispvisi::plot_fringe_histogram, NOCOLOR=nocolor

  hist = histogram(*self.amplfou,NBINS=40,OMIN=hmin,OMAX=hmax)
  xhist= findgen(40) * (hmax-hmin)/39. + hmin
  mx   = max(hist) * 1.2

  plot, xhist, hist, title="Histogram of Fourier-Amplitudes",ystyle=2, psym=10

  if( self.Ngood gt 0) then begin
      hist = histogram( (*self.amplfou)[*self.goodfringes],$
                        NBINS=40, MIN=hmin, MAX=hmax)
      if keyword_set(nocolor) then begin
          oplot, xhist,hist, linestyle=1, psym=10
      endif else begin
          oplot, xhist,hist, color=self.color[1], psym=10
      endelse
  endif
  if( self.Nbad gt 0) then begin
      hist = histogram( (*self.amplfou)[*self.badfringes],$
                        NBINS=40, MIN=hmin, MAX=hmax)
      if keyword_set(nocolor) then begin
          oplot, xhist,hist, linestyle=2, psym=10
      endif else begin
          oplot, xhist,hist, color=self.color[0], psym=10
      endelse
  endif
  oplot, [self.goodthresh,self.goodthresh], [0,mx]
  oplot, [ self.badthresh, self.badthresh], [0,mx], linestyle=1
end

PRO xmidispvisi::plot_opd_histogram, NOCOLOR=nocolor

  meanopd= total(*self.opd,1)/self.scanlength * 1e3
  hist = histogram(meanopd, NBINS=40,OMIN=hmin,OMAX=hmax)
  xhist= findgen(40) * (hmax-hmin)/39. + hmin
  mx   = max(hist) * 1.2

  plot, xhist, hist, title="Histogram of Mean Optical Path Differences",ystyle=2, psym=10
  if( self.Ngood gt 0) then begin
      hist = histogram( meanopd[*self.goodfringes], NBINS=40, MIN=hmin, MAX=hmax)
      if keyword_set(nocolor) then begin
          oplot, xhist,hist, linestyle=1, psym=10
      endif else begin
          oplot, xhist,hist, color=self.color[1], psym=10
      endelse
  endif
  if( self.Nbad gt 0) then begin
      hist = histogram( meanopd[*self.badfringes], NBINS=40, MIN=hmin, MAX=hmax)
      if keyword_set(nocolor) then begin
          oplot, xhist,hist, linestyle=2, psym=10
      endif else begin
          oplot, xhist,hist, color=self.color[0], psym=10
      endelse
  endif
  thresh= median(meanopd) - self.noiseopd*1e3  &  oplot, [thresh,thresh], [0,mx]
  thresh= median(meanopd) + self.noiseopd*1e3  &  oplot, [thresh,thresh], [0,mx]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Result:
;; [0,*] = lambda at center of wavelength bins
;; [1,*] = width of wavelength bins
;; [2,*] = correlated flux

Function xmidispvisi::plot_disp_goodbad, lolams,hilams, PLOT=plot,VERBOSE=verbose

  ; make sure we round up, the last bin might extend beyond la_end
  nlabins= fix( (self.la_end-self.la_start)/self.la_step + 1)
  corrflx= fltarr(3,nlabins)

  if Keyword_set(plot) then begin
      window,2,ysize=990
      !P.multi= [0,2,nlabins]
  endif

  print,self.Ngood," good and",self.nbad," bad fringes"

  lolams = self.la_start + self.la_step * indgen(nlabins)
  hilams = lolams + self.la_width - 1

  for l=0,nlabins-1 do begin
      lolam = lolams[l] 	; lowest lambda-pix
      hilam = hilams[l] 	; highest lambda-pix

      corrflx[0,l] = (self.lambda[lolam] + self.lambda[hilam]) / 2.
      corrflx[1,l] =  self.lambda[lolam] - self.lambda[hilam]

      lofreq= self.totopd / self.lambda[hilam] * 0.8
      hifreq= self.totopd / self.lambda[lolam] * 1.2
      print,"lambda-bin",l,", lofreq",lofreq,", hifreq",hifreq

      lofreq= floor( lofreq + 0.5)
      hifreq= floor( hifreq + 0.5)
      freqs = [lofreq,lofreq,hifreq,hifreq]

      ; select pixels within lambda-range for this bin
      ; and integrate over lambda (or what's left of it)'
      lamfr= total((*self.fringe)[ lolam:hilam,*,*],1)

      if (self.Ngood gt 0) then begin
                                ; select good fringes and FFT along scans
          goodfou = abs(FFTdim( lamfr[*,*self.goodfringes], Dimension=1))^2
          if Keyword_set(plot) then begin
              ymax = max(goodfou)
              plot, goodfou[*,0], yrange=[0,ymax], $
                Xticks=1,Xtickname=[" "," "],Yticks=1,Ytickname=[" "," "],Ymargin=[0,0]

              for f=1,self.Ngood-1 do oplot, goodfou[*,f]
              if (self.Ngood gt 1) then $
                oplot, total(goodfou,2) / self.Ngood, color=self.color[2]
              oplot, freqs, [0,ymax,ymax,0], color=self.color[3]
              oplot, self.scanlength-freqs, [0,ymax,ymax,0], color=self.color[3]
          endif
          goodamp= 4. * total(goodfou[ lofreq:hifreq,*])/self.Ngood
      endif else begin
          if Keyword_set(plot) then $
            plot, [0,1], xrange=[0,self.scanlength], /nodata, $
                Xticks=1,Xtickname=[" "," "],Yticks=1,Ytickname=[" "," "],Ymargin=[0,0]
          goodamp= 0.
      endelse

      if (self.Nbad gt 0) then begin
                                ; select bad fringes and FFT along scans
          badfou = abs(FFTdim( lamfr[*,*self.badfringes], Dimension=1))^2
          if Keyword_set(plot) then begin
              if (self.Ngood lt 1 ) then ymax = max(badfou)
              plot, badfou[*,0], yrange=[0,ymax], $
                Xticks=1,Xtickname=[" "," "],Yticks=1,Ytickname=[" "," "],Ymargin=[0,0]

              for f=1,self.Nbad-1 do oplot, badfou[*,f]
              if (self.Nbad gt 1) then $
                oplot, total(badfou,2) / self.Nbad, color=self.color[2]
              oplot, freqs, [0,ymax,ymax,0], color=self.color[4]
              oplot, self.scanlength-freqs, [0,ymax,ymax,0], color=self.color[4]
          endif
          badamp= 4. * total( badfou[ lofreq:hifreq,*])/self.Nbad
      endif else begin
          if Keyword_set(plot) then $
            plot, [0,1], xrange=[0,self.scanlength], /nodata, $
                Xticks=1,Xtickname=[" "," "],Yticks=1,Ytickname=[" "," "],Ymargin=[0,0]
          badamp= 0.
      endelse

      corrflx[2,l] = sqrt( (goodamp-badamp) > 0.) / self.DIT
      if Keyword_set(verbose) then $
        print, "lambda = ",lolam,"...",hilam," pix, corrflux =",corrflx[2,l]
  endfor
  if Keyword_set(plot) then !P.multi= 0
  return, corrflx
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Function xmidispvisi::get_photometry,$
                    ATOTAL=atot,A1=a1,A2=a2,BTOTAL=btot,B1=b1,B2=b2,AB=ab

  if keyword_set(a1) then return, self.specA1
  if keyword_set(a2) then return, self.specA2
  if keyword_set(b1) then return, self.specB1
  if keyword_set(b2) then return, self.specB2
  if keyword_set(atot) then return, self.specA1 + self.specA2
  if keyword_set(btot) then return, self.specB1 + self.specB2
  if keyword_set(ab) then return, self.specAB
  return, self.specA1 + self.specA2 + self.specB1 + self.specB2
end


Function xmidispvisi::correlflux, PLOT=plot, CFPLOT=cfplot
  cf = self->plot_disp_goodbad(PLOT=plot)
  if keyword_set(plot) or keyword_set(cfplot) then begin
      window,0
      !P.multi= 0
      plot, cf[0,*], cf[2,*], ystyle=2
  endif
  print,"This correlated flux is sqrt(2)-bug-free"
  print,"REMEMBER THAT THIS IS AN EXPERIMENTAL VERSION."
  print,"DO NOT PUBLISH ANY RESULTS UNLESS YOU KNOW WHAT YOU ARE DOING!"
  return, cf
end

Function xmidispvisi::visibility, PLOT=plot, VISPLOT=visplot, $
                    MASKTHENMULTIPLY=maskmul, MULTIPLYTHENMASK=mulmask

  visibi = self->plot_disp_goodbad(lolams,hilams,PLOT=plot)
  nlabins= (size(visibi))[2]

  if keyword_set(mulmask) then $
    print,"keyword MULTIPLYTHENMASK is obsolete, this is now the default"

  if keyword_set(maskmul) then $
    print,"Using sqrt(total(A*mask)*total(B*mask))" $
  else print,"Using total(sqrt(AB)*mask)"

  for l=0,nlabins-1 do begin
      lolam = lolams[l]
      hilam = hilams[l]
      if keyword_set(maskmul) then begin
          photA1= total( self.specA1[ lolam:hilam ])
          photA2= total( self.specA2[ lolam:hilam ])
          photB1= total( self.specB1[ lolam:hilam ])
          photB2= total( self.specB2[ lolam:hilam ])
          phot = sqrt(photA1*photB1 > 0.) + sqrt(photA2*photB2 > 0.)
      endif else begin
          phot = total( self.specAB[ lolam:hilam ])
      endelse
      visibi[2,l] = visibi[2,l] / 2d / phot
  endfor

  if Keyword_set(visplot) then begin
      if !D.name eq "X" then window,0
      !P.multi= 0
      ;print,"lambda:",visilamb
      ;print,"visi:  ",visibi[2,*]
      ;print,"min: ", min(visispec),!C
      ;print,"max: ", max(visispec),!C
      fname = (oirfilelist(self.fringefile))[0]
      first_char= Strpos(fname,'MIDI.20')
      fname = Strmid(fname,first_char)
      plot, visibi[0,*],visibi[2,*], ystyle=2, xtitle="Wavelength [micron]",ytitle="Visibiltity",$
        title= self.objectname + ", " + fname
  endif
  print,"REMEMBER THAT THIS IS AN EXPERIMENTAL VERSION."
  print,"DO NOT PUBLISH ANY RESULTS UNLESS YOU KNOW WHAT YOU ARE DOING!"
  return, visibi
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; instrumental visibility,
;; i.e. measured visibility divided by expected visibility.
;; currently, only simple disk model is implemented.
;; the parameter is the diameter in mas
;; if diameter is not given, we look it up in the calibrator-database
;; The name under which to look it up can be given with the NAME parameter
;;
Function xmidispvisi::instruvisi, diameter, NAME=name, PLOT=plot, VISPLOT=visplot, $
                    MASKTHENMULTIPLY=maskmul, MULTIPLYTHENMASK=mulmask

  visarr = self->visibility(PLOT=plot,MASKTHENMULTIPLY=maskmul,MULTIPLYTHENMASK=mulmask)
  if N_elements(diameter) le 0 then begin
      print,"Looking up ",self.objectname," in calibrator database"
      if not keyword_set(name) then name= self.objectname
      diameter = (calibrator(name)).diam
      print,"Found: ",diameter," mas"
      if diameter < 0. then return,0
  endif
  visexp = calibrate_disk(self.fringefile, diameter, visarr[0,*])
  visarr[2,*] = visarr[2,*] / visexp
  if keyword_set(visplot) then plot_visibility, visarr
  return, visarr
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; get visibility calibrated by a calibrator observation
;; this uses the instruvisi-method of the calibrator,
;; so we need the diameter of the calibrator
;;
Function xmidispvisi::calibratedvisi, cal,diameter,NAME=name,PLOT=plot,VISPLOT=visplot,$
                    MASKTHENMULTIPLY=maskmul, MULTIPLYTHENMASK=mulmask

  ; tell the calibrator to use our lambda-binning
  cal->set_lambda_bins, self.la_start,self.la_end,self.la_step,self.la_width

  ; get the instrumental visibility
  insvis = cal->instruvisi(diameter,NAME=name,PLOT=plot,$
                           MASKTHENMULTIPLY=maskmul,MULTIPLYTHENMASK=mulmask)

  ; get our own visibility
  visarr = self->visibility(PLOT=plot,MASKTHENMULTIPLY=maskmul)

  visarr[2,*] = visarr[2,*] / insvis[2,*]
  if keyword_set(visplot) then $
    plot_visibility, visarr, ytitle="Calibrated visibility", title=self.objectname
  return, visarr
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::compute_intvisi, good_ampl,bad_ampl,fringe_ampl,correl_flux,$
               visibility, visibility_err

  if (self.Ngood gt 1) then begin
      mom = moment( (*self.amplfou)[*self.goodfringes])
      good_ampl = mom[0]
      good_var  = mom[1]
  endif else if (self.Ngood gt 0) then begin
      good_ampl = (*self.amplfou)[ (*self.goodfringes)[0]]
      good_var  = 0.
  endif else begin
      good_ampl = 0. & good_var = 0.
  endelse

  if (self.Nbad gt 1) then begin
      mom = moment( (*self.amplfou)[*self.badfringes])
      bad_ampl = mom[0]
      bad_var  = mom[1]
  endif else if (self.NBad gt 0) then begin
      bad_ampl = (*self.amplfou)[(*self.badfringes)[0]]
      bad_var  = 0.
  endif else begin
      bad_ampl = 0. & bad_var = 0.
  endelse

  fringe_ampl    = sqrt(good_ampl*good_ampl - bad_ampl*bad_ampl)
  fringe_ampl_err= sqrt(good_ampl*good_ampl*good_var + bad_ampl*bad_ampl*bad_var) / fringe_ampl
  correl_flux    = fringe_ampl     / self.DIT
  correl_flux_err= fringe_ampl_err / self.DIT

  lo = max(where(self.lambda gt self.longwl))  > 0
  hi = min(where(self.lambda lt self.shortwl)) < N_elements(self.lambda)-1

  fluxA1 = total(self.specA1[lo:hi])
  fluxB1 = total(self.specB1[lo:hi])
  fluxA2 = total(self.specA2[lo:hi])
  fluxB2 = total(self.specB2[lo:hi])

  flux_factor	 = 2d * (sqrt(fluxA1 * fluxB1) + sqrt(fluxA2 * fluxB2))
  visibility     = correl_flux     / flux_factor
  visibility_err = correl_flux_err / flux_factor
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::Hardcopy, fname,NOCOLOR=nocolor,TRACES=traces,DONTCLOSE=dontclose

  if N_params() lt 1 then $
    fname= "./Midivisi" + MIDIname_datime(self.fringefile) + ".ps"
  print,"Printing to file ",fname

  xmargin_saved = !X.Margin
  set_plot,"PS"
  device,filename=fname,/Landscape
  if not keyword_set(nocolor) then device,/color

  !P.multi= [ 8, 1, 8, 0, 0 ]
  self->plot_static,NOCOLOR=nocolor

  !P.multi= [ 6, 2, 4, 0, 1 ]
  !X.Margin[1] = 30
  self->plot_goodbad

  !P.multi= [ 2, 2, 4, 0, 1 ]
  !X.Margin[0] = 18 & !X.Margin[1] = xmargin_saved[1]
  self->plot_fringe_histogram,NOCOLOR=nocolor

  !P.multi= [ 1, 2, 4, 0, 1 ]
  self->plot_opd_histogram,NOCOLOR=nocolor

  !X.Margin= xmargin_saved
  !P.multi = [ 1, 1, 1, 0, 1 ]
  device,font_size=10
  nScan = N_elements(*self.amplfou)
  XYOuts, 0.5, 0.45, "Object: "+self.objectname, Alignment=0.5, /NORMAL
  XYOuts, 0.5, 0.42, "Photom.Filter: "+self.PhFilt, Alignment=0.5,/Normal
  XYOuts, 0.5, 0.39, "Fringe Filter: "+self.FrFilt, Alignment=0.5,/Normal
  XYOuts, 0.5, 0.36, "Scanlength "+strtrim(self.scanlength,2)+$
    " => "+strtrim(nScan,2)+" scans", Alignment=0.5,/Normal
  XYOuts, 0.5, 0.33, "Beam is "+self.beam, Alignment=0.5,/Normal
  fluxstr= string( total(self.specA1), total(self.specA2), $
                   Format='("A: ",G0.3,", ",G0.3," ph/sec")')
  XYOuts, 0.5, 0.30, fluxstr, Alignment=0.5,/Normal
  fluxstr= string( total(self.specB1), total(self.specB2), $
                   Format='("B: ",G0.3,", ",G0.3," ph/sec")')
  XYOuts, 0.5, 0.27, fluxstr, Alignment=0.5,/Normal

  XYOuts, 0.5, 0.23, "Good threshold:"+string(self.goodthresh), Alignment=0.5,/Normal
  XYOuts, 0.5, 0.20, " Bad threshold:"+string( self.badthresh), Alignment=0.5,/Normal

  self->compute_intvisi, good_ampl,bad_ampl,fringe_ampl,correl_flux,visibility,visibility_err

  XYOuts, 0.5, 0.16, "Mean signal ampl. "+strtrim(good_ampl,2)  ,Alignment=0.5,/Normal
  XYOuts, 0.5, 0.13, "Mean noise  ampl. "+strtrim( bad_ampl,2)  ,Alignment=0.5,/Normal
  XYOuts, 0.5, 0.10, "Corrected amplit. "+strtrim(fringe_ampl,2),Alignment=0.5,/Normal
  XYOuts, 0.5, 0.07, "Correlated Flux "+$
    strtrim(string(correl_flux,  format='(G10.3)'),2), Alignment=0.5,/Normal
  XYOuts, 0.5, 0.03, "VISIBILITY = "+$
    strtrim(string(visibility,    format='(F6.4)'),2)+"+-"+$
    strtrim(string(visibility_err,format='(F6.4)'),2), Alignment=0.5,/Normal

  if keyword_set(TRACES) then begin
      !X.Margin = xmargin_saved
      !P.multi= [ 0, 2, 1, 0, 1 ]
      self->plot_traces, [self.traceFr1, self.trace1[0], self.trace1[1]],$
        title="Window 1", NOCOLOR=nocolor
      self->plot_traces, [self.traceFr2, self.trace2[0], self.trace2[1]],$
        title="Window 2", NOCOLOR=nocolor
  endif

  if not keyword_set(dontclose) then begin
      device,/close
      set_plot,"X"
  endif
  !X.Margin = xmargin_saved
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Pro xmidispvisi::update_GoodBad
  meanopd= total(*self.opd,1)/self.scanlength

  ;*self.goodfringes= where(abs(meanopd-median(meanopd)) lt self.totopd*1e-6/2. $
  *self.goodfringes= where(*self.amplfou gt self.goodthresh, Ngood)
   self.Ngood = Ngood
  *self.badfringes = where(*self.amplfou lt self.badthresh or $
                           abs(meanopd-median(meanopd)) gt self.noiseopd, Nbad)
   self.Nbad = Nbad
end

Pro xmidispvisi::update_gui

  ;;;;; plot fourier fringe amplitude and thresholds

  wset, self.topdrawID
  !p.multi= self.multi
  save_charsize = !p.charsize
  !p.charsize = !p.charsize * 1.7
  plot, *self.amplfou, xtitle = "Scan no.", title = "Fourier Fringe Amplitude (linear)"
  nScans = N_elements(*self.amplfou)
  if (self.Ngood gt 1) then begin
      oplot,*self.goodfringes, (*self.amplfou)[*self.goodfringes], color=0
      ;oplot,(*self.amplfou) > self.oldgthres, color=0
      oplot,(*self.amplfou) < self.oldbthres, color=0
      oplot, [0,nScans], [self.oldgthres,self.oldgthres],linestyle=1,color=0
      oplot, [0,nScans], [self.oldbthres,self.oldbthres],linestyle=1,color=0
  endif

  self->update_GoodBad

  if (self.Ngood gt 1) then begin
      oplot,*self.goodfringes, (*self.amplfou)[*self.goodfringes], color=self.color[1]
      ;oplot,(*self.amplfou) > self.goodthresh,color=self.color[1]
      oplot,(*self.amplfou) < self.badthresh, color=self.color[0]
      oplot, [0,nScans], [self.goodthresh,self.goodthresh],linestyle=1
      oplot, [0,nScans], [self.badthresh, self.badthresh ],linestyle=1
      self.oldgthres= self.goodthresh
      self.oldbthres= self.badthresh
  endif
  !p.charsize = save_charsize

  self->compute_intvisi, good_ampl,bad_ampl,fringe_ampl,correl_flux,visibility,visibility_err

  Widget_Control, self.goodthl,SET_Value="Good threshold:"+string(self.goodthresh)
  Widget_Control, self.badthl ,SET_Value=" Bad threshold:"+string( self.badthresh)
  Widget_Control, self.sigamp, SET_VALUE="Mean signal ampl. "+strtrim(good_ampl, 2)
  Widget_Control, self.noiamp, SET_VALUE="Mean noise  ampl. "+strtrim( bad_ampl, 2)
  Widget_Control, self.corramp,SET_VALUE="Corrected amplit. "+strtrim(fringe_ampl,2)
  Widget_Control, self.corrflx,SET_VALUE="Correlated flux "+$
    strtrim(string(correl_flux,   format='(G10.3)'),2)
  Widget_Control, self.visibi, SET_VALUE="VISIBILITY = "+$
    strtrim(string(visibility,    format='(F6.4)'),2)+"+-"+$
    strtrim(string(visibility_err,format='(F6.4)'),2)

  ;;;;; plot the good, the bad, and the ugly

  wset, self.lftdrawID
  !P.multi = [0,0,2,0,0]
  self->plot_goodbad

  ;;;;; plot histogram

  wset,self.rgtdrawID
  !P.multi = [0,0,2,0,0]
  self->plot_fringe_histogram
  self->plot_opd_histogram
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO xmidispvisi_event, ev
  WIDGET_CONTROL, ev.TOP, GET_UVALUE=st
  st->event, ev
end

PRO xmidispvisi::event, ev

;print,"event!"

  update = 0
  ;obsolete: draw when our initial event comes
  ;if (TAG_NAMES(ev, /STRUCTURE_NAME) EQ 'WIDGET_TIMER') then update=1
  case (ev.ID) OF
      self.menu: begin
          case (ev.value) of
              "Help.About": begin
                  msg= dialog_message(["MIA Version 1.05 (Halloween version)",$
                                       "by Rainer Koehler, Walter Jaffe,",$
                                       "Bill Cotton, Olivier Chesneau and others"],$
                                      /INFORMATION, DIALOG_PARENT=ev.top)
              end
              "Plot.Dispersed Power": v= self->visibility(/plot)
              "Plot.Visibility": v= self->visibility(/visplot)
              "Print.Visibility": begin
                  fname = Dialog_Pickfile(Title="Please enter name for Postscript-file")
                  if fname ne "" then begin
                      set_plot,"PS"
                      device,file=fname,/landscape
                      v= self->visibility(/visplot)
                      device,/close
                      set_plot,"X"
                  endif
              end
              "File.Lambda GUI": self->lambda_gui
              "File.Quit": begin
                  WIDGET_CONTROL, ev.TOP, /DESTROY
                  return
              end
              ELSE: print,"unknown menu event: ",ev.value
          end
      end
      self.frimov: xmidimovie, *self.opd, total(*self.fringe,1)
      self.foumov: xmidimovie, findgen(self.scanlength), *self.foufri
      self.print:  update = 1	; force update to recompute visibility
      self.wrtlog: update = 1	; force update to recompute visibility
      self.lftdraw: $
          if (ev.type eq 0) then v=self->visibility(/plot,/visplot)
      self.rgtdraw: begin
          if (ev.type eq 0) then self.button= ev.press
          if (ev.type eq 1) then self.button= 0
          if (ev.type eq 0 or (ev.type eq 2 and self.button ne 0)) then begin
              wset,self.rgtdrawID
              ymid = (get_screen_size())[1]/4-40
              if (ev.y gt ymid) then begin
                  !P.multi = [2,0,2,0,0] & self->plot_fringe_histogram
              endif else begin
                  !P.multi = [1,0,2,0,0] & self->plot_opd_histogram
              endelse
              pos = convert_coord( ev.x, ev.y, /DEVICE, /TO_DATA)
                                ;print, "data pos: ",pos
              ;case self.button of
              ;    1:  self.goodthresh = pos[0]
              ;    else: self.badthresh= pos[0]
              ;endcase
              if (ev.y gt ymid) then self.goodthresh = pos[0] $
                else begin
                  med = median(total(*self.opd,1))/self.scanlength * 1e3
                  self.noiseopd = abs(med-pos[0])/1e3
              endelse
              update = 1
          endif
      end
      self.quit: begin
          WIDGET_CONTROL, ev.TOP, /DESTROY
          return
      end
      else: ;relax
  endcase

  if (update eq 0) then return

  self->update_gui

  ;;;;; create hardcopy, if button pressed
  if (ev.ID eq self.print) then self->hardcopy

  ;;;;; write to logfile, if button was pressed

  if (ev.ID eq self.wrtlog) then begin
      self->compute_intvisi, good_ampl,bad_ampl,fringe_ampl,correl_flux,visibility,visibility_err

      if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'
      openw, lun, "PHOTOMETRY/visibilities",/get_lun,/append
      printf,lun, "Object: ",self.objectname
      printf,lun, "Fringefile(s):" & printf,lun,self.fringefile
      printf,lun, "Photfile(s):  " & printf,lun,self.photfiles
      printf,lun, "Maskfile:   ",self.maskfile
      printf,lun, "Noise threshold:",self.badthresh,", Signal threshold:",self.goodthresh
      printf,lun, "Mean signal ampl. "+strtrim(good_ampl,2)
      printf,lun, "Mean noise  ampl. "+strtrim( bad_ampl,2)
      printf,lun, "Corrected amplit. "+strtrim(fringe_ampl,2)
      printf,lun, "Correlated flux "+strtrim(string(correl_flux,format='(G10.3)'),2)
      printf,lun, "VISIBILITY = " +  strtrim(string(visibility,format='(F8.6)'),2)+$
        "+-" + strtrim(string(visibility_err,format='(F8.6)'),2)
      printf,lun, "----------------------------------------------------------------------"
      close, lun & free_lun, lun
  endif
END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

FUNCTION xmidispvisi::init, $
                    fringefile_in, photfileA, photfileB, $
                    NOISEFILE=noisefile, SKY=sky, ASKY=skyA, BSKY=skyB, $
                    MASKFILE=maskfile_in, MASKWIDTH=maskwd, FRINGEMASK=fringemask,$
                    RUNNING_HIPASS=running_hipass, DONT_WRITE=dont_write,$
                    NOCOLOR=nocolor, TRUECOLOR=truecolor, $
                    NOMASKCHECK=nomaskcheck, REUSE=reuse, DWIM=dwim, $
                    DISPERSEDTRACK=dispersedbins, WHITETRACK=whitetrack, $
		    SHOW_TRACES=show_traces, THRESHOLD=threshold

  if (N_params() lt 2) then begin
	print,"Not enough files.  I can't work under these conditions!"
        return,0
  endif
  self.fringefile= fringefile_in[0]
  photfileA = photfileA[0]
  photfileB = photfileB[0]

  if keyword_set(noisefile) then $
    self.fringefile = noisefile + " " + self.fringefile

  !X.style = 1
  !Y.style = 1

  if keyword_set(truecolor) then begin
      print,"Using truecolor colors"
      self.color= [ '0000ff'x, '00ff00'x, 'ff0000'x, 'f0f000'x, 'f000f0'x, '00f0f0'x ]
  endif else begin
      self.color= [ 2, 3, 4, 5, 6, 7 ]
  endelse

  if (N_Params() gt 2) then $
    self.photfiles = [ photfileA, photfileB ] $
  else self.photfiles = photfileA

  ; we have to use the original filenames because oircompress
  ; removes all the interesting things from the header
  filelist = oirFileList(self.fringefile)

  im = obj_new('imagedata', filelist[0])
  he = im->prihead()
  self.objectname= StrCompress(he->getpar('OBS TARG NAME'),/remove_all)
  self.beam   = he->getpar('INS OPT1')
  self.DIT    = StrCompress(he->getpar('DET DIT'),	/remove_all)
  self.FrFilt = StrCompress(he->getpar('INS FILT NAME'),/remove_all)
  Grism       = strcompress(he->getpar('INS GRIS NAME'),/remove_all)
  detwin1strx = he->getpar('DET WIN1 STRX')
  ;detwin1stry= he->getpar('DET WIN1 STRY')
  ;detwin2strx= he->getpar('DET WIN2 STRX')
  ;detwin2stry= he->getpar('DET WIN2 STRY')

  self.scanlength = (im->head())->getpar('maxstep')
  obj_destroy,im
  while (self.scanlength le 0) do begin
      print,""
      print,"Impossible number of frames per scans!"
      line= "" & read, Prompt="Please enter the correct number: ",line
      self.scanlength = line
  endwhile

  short_fringefile= MIDIname_datime(self.fringefile)

  if keyword_set(maskfile_in) then begin
      self.maskfile = maskfile_in
      maskdata = oirgetdata(maskfile_in)
  endif else if keyword_set(fringemask) then begin
      self.maskfile = 'PHOTOMETRY/Mask_'+short_fringefile+'.fits'
      print,"Maskfile is ",self.maskfile
      maskdata = (self->FTframe(MASKWD=maskwd,/show,NOCOLOR=nocolor))[2]
      if not keyword_set(dwim) then begin
          testOk= "" & read, '> Is the mask satisfactory? (y/n) ', testOK
          if (strupcase(testOK) eq "N") then begin
              print, '<!> Sorry about that.  Better luck next time'
              return,0
          endif
          nomaskcheck = 1
      endif
      if not keyword_set(dont_write) then begin
          print,"Writing mask ",self.maskfile
          if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'
          oirNewData, filelist[0], self.maskfile, maskdata
      endif
  endif else begin
      self.maskfile = 'PHOTOMETRY/Mask'
      maskdata = self->meanmask(WIDTH=maskwd,show=show_traces)
      self.maskfile = self.maskfile + ".fits"
      print,"Maskname is ",self.maskfile
      if not keyword_set(dont_write) then begin
          print,"Writing mask ",self.maskfile
          if not file_test("PHOTOMETRY") then spawn,'mkdir "PHOTOMETRY"'
          oirNewData, (oirFilelist(photfileA))[0], self.maskfile, maskdata
      endif
  endelse
  if not keyword_set(nomaskcheck) and not keyword_set(dwim) then begin
      if (check_mask_position(self.fringefile,self.maskfile) eq 0) then $
        return,0
      wdelete,0
  endif

  if N_Params() gt 2  then begin
      ; we have photometry A and B

      print,"Photometry A..."
      self->do_photometry, photfileA, maskdata, $
        PhFiltA, spec1, spec2, sky=sky, mysky=skyA
      self.PhFilt = PhFiltA
      self.specA1 = spec1
      self.specA2 = spec2

      print,"Photometry B..."
      self->do_photometry, photfileB, maskdata,$
        PhFiltB, spec1,spec2, sky=sky, mysky=skyB
      self.specB1 = spec1
      self.specB2 = spec2

      if PhFiltA ne PhFiltB then begin
          eniki= "any key"
          print, "Photometric filter A is ",PhFiltA, ", but"
          print, "Photometric filter B is ",PhFiltB,"."
          read,  "Do you know what you are doing? ",eniki
          if eniki ne "y" then return,0
      endif

      print,"sqrt(A*B)..."
      self->do_photometry, photfileA, maskdata, PHOTFILEB=photfileB,$
        PhFiltB, spec1,spec2, sky=sky, mysky=skyB
      self.specAB = spec1 + spec2

  endif else begin
      print,"Photometry A&B..."
      self->do_photometry, photfileA, maskdata,$
        PhFilt, spec2,spec1, mysky=sky
      self.specA1 = spec1/2. & self.specA2 = self.specA1
      self.specB1 = spec2/2. & self.specB2 = self.specB1
  endelse

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  data = getCompressSpec(self.fringefile, self.maskfile, $
                         'PHOTOMETRY/CmprSp_' + short_fringefile + '_' + STRMID(self.maskfile,11),$
                         REUSE=reuse, DWIM=dwim)

  junk= where(strtrim(data.tartyp1,2) ne 'T', count)
  junk= 0
  if (count gt 0) then begin
      print,"-------------------------------------------------------------------------"
      print,"WARNING: tartyp is not always 'T'!  Was chopping on during fringe track??"
      print,"-------------------------------------------------------------------------"
  endif

  nData = N_ELEMENTS(data)
  nScan = nData/self.scanlength
  print, nData," frames =>", nScan, " scans"
  if (self.beam EQ 'HIGH_SENS') then begin
      flux1spec = data.data1
      flux2spec = data.data2
  endif else if (self.beam EQ 'SCI_PHOT') then begin
      flux1spec = data.data2
      flux2spec = data.data3
  endif else begin
      print,"INS OPT1 is",self.beam
      print,"Neither HIGH_SENS nor SCI_PHOT, don't know where the fringes are!"
      return,0
  endelse
  fringe = flux1spec - flux2spec
  ;
  ; fringe is now 2D: 1.dim is wavelength, 2.dim is time
  ;
  sz = size(fringe)
  nLambda = sz[1]
  ;old: self.lambda = -0.0519 * (dindgen(nLambda)+detwin1strx) + 21.1844
  P = dindgen(nLambda) + detwin1strx
  if Grism eq "PRISM" then begin
      print,"using wavelength calibration for PRISM"
      self.lambda = -0.0001539 * P^2 + 0.009491 * P + 15.451905
  endif else begin
      print,"+------------------------------------------------------+"
      print,"| WARNING! WARNING! WARNING! WARNING! WARNING! WARNING |"
      print,"| Using preliminary wavelength calibration for grism.  |"
      print,"| This is valid for the first interferometric channel, |"
      print,"| but not for the second.  This means that all the     |"
      print,"| results you will get are preliminary and useless!    |"
      print,"+------------------------------------------------------+"
      ok="" & read,"Do you want to continue? ",ok
      self.lambda = -1.21122e-6 * P^2 + 0.0232192 * P + 6.61739
  endelse
  la_range = where(self.lambda ge 8. and self.lambda le 13.)
  self->set_lambda_bins, min(la_range), (max(la_range) < nLambda-4), 4, 4

  opd = data.localOpd[1]-data.localOpd[0] + data.opd[1]-data.opd[0]
  ; last two opd points are garbage
  ;opd[nData-2:nData-1] = median(opd[nData-self.scanlength:nData-2])

  if keyword_set(running_hipass) then begin
      print, "Applying running Hi-pass filter..."

      ; the following was (partly) stolen from midiRawFringe
      dtime = 86400. * (data[1].time[0] - data[0].time[0])
      tZero = 1.
      nZero = (tZero/dtime) > 10
      ;print,"dtime ",dtime,", nzero ",nzero

      for i=0,nLambda-1 do begin
          ; 1st two points are garbage
          fringe[i, 0:1] = MEDIAN(fringe[i, 2:200])
                                ; remove low freq trend
          fringe[i,*] = fringe[i,*] - SMOOTH(fringe[i,*], nZero,/EDGE_TRUNCATE)
      end
  endif

  ; reform into scans, droping last fractional scan if any
  nFringe= LONG(self.scanlength) * LONG(nScan)
  fringe = REFORM(fringe[*,0:nFringe-1],nLambda,self.scanlength,nScan,/overwrite)
  opd    = REFORM(   opd[  0:nFringe-1],        self.scanlength,nScan,/overwrite)
  self.fringe= ptr_new(fringe, /NO_COPY)
  self.opd   = ptr_new(  opd , /NO_COPY)
  self.goodfringes= ptr_new(fltarr(nScan,/NOzero),/NO_COPY)
  self.badfringes = ptr_new(fltarr(nScan,/NOzero),/NO_COPY)

  if not keyword_set(running_hipass) then begin
      print, "Subtracting mean amplitude of scans..."
      for la=0,nLambda-1 do begin
	  ; 1st two points are garbage
          (*self.fringe)[la,0:1,0]= MEDIAN( (*self.fringe)[la,2:self.scanlength-1,0])
          for sc=0,nScan-1 do begin
              (*self.fringe)[la,*,sc] = $
                (*self.fringe)[la,*,sc] - mean( (*self.fringe)[la,*,sc])
          endfor
      endfor
  endif

  case self.FrFilt of
	; "OPEN" added by FP, wavelength range just estimated
	; limited to 13.4, because that is the edge of the window, RK 01-aug03
  'OPEN':  begin print,"filter: open"  & self.longwl=13.4 & self.shortwl= 7.  & end
  'Nband': begin print,"N-band filter" & self.longwl=13.  & self.shortwl= 7.7 & end
  'N8.7':  begin print,"N 8.7 filter"  & self.longwl=10.3 & self.shortwl= 7.2 & end
  'SiC':   begin print,"SiC filter"    & self.longwl=13.0 & self.shortwl=10.5 & end
  else: begin print,"Unknown filter: ", self.FrFilt & return,0 & end
  endcase

  ; create spectrally integrated arrays
  wherelam = where(self.lambda ge self.shortwl and self.lambda le self.longwl)
  lo = min(wherelam) > 0
  hi = max(wherelam) < nLambda-1

  if keyword_set(dispersedbins) then begin
      ;; the following useless "if" is preparation for the day when
      ;; dispersedtrack will be the default
    if keyword_set(dispersedbins) then begin
	if (dispersedbins le 1  $
            or dispersedbins gt hi-lo) then dispersedbins= 10
        if (dispersedbins MOD 2 ne 0) then dispersedbins= dispersedbins-1
    endif else dispersedbins= 10
    print,"Dispersed tracking, using",dispersedbins," bins in wavelength"
    mid = fix((lo+hi)/2.)
    dpx = fix((hi-lo)/dispersedbins)	; No. of pix per lambda-bin
    lo = mid - dispersedbins/2 * dpx
    hi = mid + dispersedbins/2 * dpx - 1
  endif else begin
    print,"Using code for Undispersed tracking"
    dispersedbins = 0
  endelse
  print, self.shortwl, self.longwl, lo, hi, $
    Format='("integrating over lambda = ",F4.1,"...",F4.1,", x =",I3,"...",I3)'

  self.flux1 = ptr_new( total( flux1spec[lo:hi,*], 1), /NO_COPY)
  self.flux2 = ptr_new( total( flux2spec[lo:hi,*], 1), /NO_COPY)

  ; sum over lambda, then do fft along opd-scan
  if (dispersedbins gt 0) then begin
      foufriSpec = FFTdim( rebin((*self.fringe)[lo:hi,*,*],$
                                 dispersedbins,self.scanlength,nScan),Dimension=2)
  endif else begin
      foufriSpec = FFTdim( total((*self.fringe)[lo:hi,*,*], 1), Dimension=1)
  endelse
  self.foufri= ptr_new( abs(foufriSpec)^2, /NO_COPY)
  foufriSpec = 0                ; free memory

  ;NOTE: we should figure out some way to remove bad scans
  ;     (but the total opd jitters somewhat because of rounding errors!)
  opdlen = dblarr(nScan)
  for f=0,nScan-1 do opdlen[f]= max((*self.opd)[*,f]) - min((*self.opd)[*,f])
  self.totopd = median(opdlen) * 1e6
  print," total opd length per scan:",self.totopd," micron"

  self.amplfou= ptr_new(fltarr(nScan,/nozero),/NO_COPY)

  lowfreq = floor( self.totopd / self.longwl  +0.5)
  highfreq= floor( self.totopd / self.shortwl +0.5)
  if (highfreq gt self.scanlength/2) then highfreq= self.scanlength/2

  for f=0,nScan-1 do begin
      ;print, "scan",f,": ",lowfreq,"...",highfreq
      if (dispersedbins gt 0) then begin
          (*self.amplfou)[f] = 0.
          lastep = (self.longwl-self.shortwl)/dispersedbins
          for la=0,dispersedbins-1 do begin
              lowfreq = floor( self.totopd / (self.shortwl+lastep*(la+1)) +0.5)
              highfreq= floor( self.totopd / (self.shortwl+lastep* la   ) +0.5)
              if (highfreq gt self.scanlength/2) then highfreq= self.scanlength/2
              (*self.amplfou)[f] = (*self.amplfou)[f] $
                + 4. * sqrt(total( (*self.foufri)[la,lowfreq:highfreq,f]))
          endfor
      endif else begin
          (*self.amplfou)[f] = 4. * sqrt(total( (*self.foufri)[lowfreq:highfreq,f]))
      endelse
  endfor
  if (dispersedbins gt 1) then *self.foufri = total(*self.foufri,1)
  ; we need the *integrated* power-sepctra later

  if NOT keyword_set(threshold) then threshold= max(*self.amplfou)/3.

  self.goodthresh= threshold
  self.badthresh = 0.
  self.noiseopd  = 4e-3	; this is in meters
  self->update_GoodBad

  return,1
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

pro xmidispvisi::gui, BLOCK=block, TRUECOLOR=truecolor

  if keyword_set(truecolor) then begin
      print,"Using truecolor colors"
      self.color= [ '0000ff'x, '00ff00'x, 'ff0000'x, 'f0f000'x, 'f000f0'x, '00f0f0'x ]
  endif else begin
      self.color= [ 2, 3, 4, 5, 6, 7 ]
  endelse

  screensz = get_screen_size()

  ; Create a base widget containing a draw widget and a sub-base
  base  = Widget_Base(Title="Midivisi "+self.objectname, $
                      Space=0, MBAR=mbar, /COLUMN, /BASE_ALIGN_LEFT)

  self.menu = cw_pdmenu(mbar, ['1\File', '0\Lambda GUI', '2\Quit', $
                               '1\Plot', '0\Dispersed Power', '2\Visibility', $
                               '1\Print', '0\Dispersed Power', '2\Visibility', $
                               '1\Help', '2\About' ],$
                        /COLUMN,/MBAR,/HELP,/RETURN_FULL_NAME)

  filename = oirFileList(self.fringefile)
  fringelbl = filename[0]
  for i=1,N_elements(filename)-1 do $
    fringelbl = fringelbl + " + " + MIDIname_time(filename[i]) + ".fits"

  fringe_l= Widget_Label(base, Value= "Fringefile: "+fringelbl)
  if N_elements(self.photfiles) le 1 then begin
      phot_l= Widget_Label(base,Value="  Photfile: "+self.photfiles[0])
  endif else begin
      phot_l= Widget_Label(base,Value="  Photfile: "+self.photfiles[0] + $
                           " + " + MIDIname_time(self.photfiles[1]) + ".fits")
  endelse
  mask_l  = Widget_Label(base, Value= "  Maskfile: "+self.maskfile)

  self.topdraw  = Widget_Draw(base, XSIZE=screensz[0]-16, YSIZE=screensz[1]/2-60)

  base2 = Widget_Base(base, Space=0, /ROW)
  self.lftdraw= Widget_Draw(base2, XSIZE=(screensz[0]-200)/2, YSIZE=screensz[1]/2-80,$
                       /BUTTON_EVENT)
  bbase  = Widget_Base(base2, Space=0, /Column, /Align_Top, /BASE_align_center)
  self.rgtdraw= Widget_Draw(base2, XSIZE=(screensz[0]-200)/2, YSIZE=screensz[1]/2-80,$
                       /BUTTON_EVENT, /MOTION_EVENT)

  objectl= Widget_Label(bbase, Value="Object name: "+self.objectname)
  phfiltl= Widget_Label(bbase, Value="Photometric Filter: "+self.PhFilt)
  frfiltl= Widget_Label(bbase, Value="Fringe Filter: "+self.FrFilt)
  scanlnl= Widget_Label(bbase, Value="Scanlength: "+strtrim(self.scanlength,2)+$
                        " => "+strtrim(N_elements(*self.amplfou),2)+" scans")
  beaml  = Widget_Label(bbase, Value="Beam is "+self.beam)
  fluxstr= string( total(self.specA1), total(self.specA2), $
                   Format='("A: ",G0.3,", ",G0.3," ph/sec")')
  fluxAl = Widget_Label(bbase, Value=fluxstr)
  fluxstr= string( total(self.specB1), total(self.specB2), $
                   Format='("B: ",G0.3,", ",G0.3," ph/sec")')
  fluxBl = Widget_Label(bbase, Value=fluxstr)

  self.goodthl= Widget_Label(bbase, Value="Good threshold:"+string(self.goodthresh))
  self.badthl = Widget_Label(bbase, Value=" Bad threshold:"+string( self.badthresh))

  self.sigamp = Widget_Label(bbase, Value="Mean signal ampl. ????????")
  self.noiamp = Widget_Label(bbase, Value="Mean noise  ampl. ????????")
  self.corramp= Widget_Label(bbase, Value="Corrected amplit. ????????")
  self.corrflx= Widget_Label(bbase, Value="Correlated Flux ??????????")
  self.visibi = Widget_Label(bbase, Value="VISIBILITY = 0.1234+-0.1234",/Align_Center)

  self.frimov = Widget_Button(bbase, Value="Fringe Movie", XSize=200-24, Ysize=36)
  self.foumov = Widget_Button(bbase, Value="Fourier Movie",XSize=200-24, Ysize=36)
  self.print  = Widget_Button(bbase, Value="Hardcopy", XSize=200-24, Ysize=36)
  self.wrtlog = Widget_Button(bbase, Value="Write to log", XSize=200-24, Ysize=36)
  self.quit   = Widget_Button(bbase, Value="Quit", XSize=200-24, Ysize=36)

  WIDGET_CONTROL, base, /REALIZE
  WIDGET_CONTROL, self.topdraw, GET_VALUE=ID & self.topdrawID= ID
  WIDGET_CONTROL, self.lftdraw, GET_VALUE=ID & self.lftdrawID= ID
  WIDGET_CONTROL, self.rgtdraw, GET_VALUE=ID & self.rgtdrawID= ID

  ;; now draw the static stuff

  wset, self.topdrawID
  !p.multi= [0, 0, 4, 0, 0]
  save_charsize = !p.charsize
  !p.charsize = !P.charsize * 1.7
  self->plot_static
  multi2 = !P.multi
  !p.charsize= save_charsize

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  WIDGET_CONTROL, base, SET_UVALUE=self

  ;obsolete: trigger some event to draw dynamic stuff
  ;WIDGET_CONTROL, self.topdraw, TIMER=0.0
  ;print,"starting xmanager"
  ;device,decomposed=0
  if (self.color[0] < 100) then tek_color

  self->update_gui

  if keyword_set(block) then $
    XMANAGER, 'xmidispvisi', base $
  else $
    XMANAGER, 'xmidispvisi', base, /NO_BLOCK

  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
end


PRO xmidispvisi::cleanup
  print,"xmidispvisi cleaning up...Aaaarghhhh..."

  ComprFile = 'PHOTOMETRY/CmprSp_' + $
    MIDIname_datime(self.fringefile) + '_' + STRMID(self.maskfile,11)
  if file_test(ComprFile) then begin
      print,"Removing ",ComprFile
      file_delete, ComprFile
  endif
  PTR_FREE, self.opd
  PTR_FREE, self.flux1
  PTR_FREE, self.flux2
  PTR_FREE, self.fringe
  PTR_FREE, self.foufri
  PTR_FREE, self.amplfou
  PTR_FREE, self.goodfringes
  PTR_FREE, self.badfringes
end


PRO xmidispvisi::stop
  help,*self,/struct
  stop
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; wrapper for lazy programmers
;;
Function xmdv, files_in, NOISEFILE=noisefile, SKY=sky, ASKY=skyA, BSKY=skyB,$
               MASKFILE=maskfile_in, MASKWIDTH=maskwd, FRINGEMASK=fringemask,$
               RUNNING_HIPASS=running_hipass, DONT_WRITE=dont_write,$
               NOCOLOR=nocolor, TRUECOLOR=truecolor, $
               NOMASKCHECK=nomaskcheck, REUSE=reuse, DWIM=dwim,$
               DISPERSEDTRACK=dispersedbins, WHITETRACK=whitetrack,$
               SHOW_TRACES=show_traces, THRESHOLD=threshold

  ; _EXTRA=extra would be much nicer, but then the check if keywords are
  ; allowed is turned off...
  ; I'm not going to comment on IDL's ingenuity now.

  if File_Test(files_in[0],/Directory) then files_in= midiguis(dir=files_in)
  ; user gave a dir, so ask for files in it

  if N_Elements(files_in) eq 1 then begin
    ftrfile= files_in
    file0  = (oirFileList(ftrfile))[0]
    im = obj_new('imagedata', file0)
    he = im->prihead()
    objectname= STRCOMPRESS(he->getpar('OBS TARG NAME'),/remove_all)
    nrts_mode = strcompress(he->getpar('NRTS MODE'),/remove_all)
    obj_destroy,im
    if (nrts_mode ne 'OBS_FRINGE_TRACK_FOURIER' and $
        nrts_mode ne 'OBS_FRINGE_SEARCH_FOURIER') then begin
		print,"you must specify a fringe-track file!" & return,0
    endif
    ;print,ftrfile," has object ",objectname," in NRTS_mode ",nrts_mode

    last_char = strpos(file0,'MIDI.20')
    dirname = strmid(file0,0,last_char)
    print,"dir is '",dirname,"'"
    restore, dirname+"midiGui.SAV"
    arg = 1
    for f=0,N_Elements(FBTFilelistFilelist)-1 do begin
      ff0 = (oirFileList(FBTFilelistFilelist[f]))[0]
      if (ff0 gt file0) then begin
        im = obj_new('imagedata', ff0)
        he = im->prihead()
        objname= strcompress(he->getpar('OBS TARG NAME'),/remove_all)
        nrts_m = strcompress(he->getpar('NRTS MODE'),/remove_all)
	shutter= strcompress(he->getpar('INS SHUT NAME'),/remove_all)
        obj_destroy,im
	if (objectname ne objname) then break
        if (nrts_m eq "DEFAULT_CHOP") then begin
          ;print,ff0," -> ",objname,", ",nrts_m,", ",shutter
	  if (shutter eq 'AOPEN') then phAfile = FBTFilelistFilelist[f]
	  if (shutter eq 'BOPEN') then phBfile = FBTFilelistFilelist[f]
        endif
      endif
    endfor
    if (phAfile eq "" or phBfile eq "") then begin
	print,"no photometry with same object after fringe" & return,0
    endif
  endif else begin
    ; more than one file given
    for i=0,N_elements(files_in)-1 do begin
      print, "checking ", files_in[i]
      file0  = (oirFileList(files_in[i]))[0]
      if not file_test(file0,/READ) then begin
	print,"file ",file0," does not exist or is not readable"
        return,0
      endif
      im = obj_new('imagedata', file0)
      he = im->prihead()
      nrts_mode= strcompress(he->getpar('NRTS MODE'),/remove_all)
      shutter  = strcompress(he->getpar('INS SHUT NAME'),/remove_all)
      obj_destroy,im
      if (nrts_mode eq 'OBS_FRINGE_TRACK_FOURIER' or $
	  nrts_mode eq 'OBS_FRINGE_SEARCH_FOURIER') then begin
		ftrfile = files_in[i]
      endif else begin
	if (nrts_mode eq "DEFAULT_CHOP") then begin
          ;print,ff0," -> ",objname,", ",nrts_m,", ",shutter
	  if (shutter eq 'AOPEN') then phAfile = files_in[i]
	  if (shutter eq 'BOPEN') then phBfile = files_in[i]
        endif
      endelse
    endfor
  endelse

  print,"Fringes:      ",ftrfile
  print,"Photometry A: ",phAfile
  print,"Photometry B: ",phBfile

  return, obj_new('xmidispvisi', ftrfile, phAfile, phBfile, $
                    NOISEFILE=noisefile, SKY=sky, ASKY=skyA, BSKY=skyB, $
                    MASKFILE=maskfile_in,MASKWIDTH=maskwd, FRINGEMASK=fringemask,$
                    RUNNING_HIPASS=running_hipass, DONT_WRITE=dont_write,$
                    NOCOLOR=nocolor, TRUECOLOR=truecolor, $
                    NOMASKCHECK=nomaskcheck, REUSE=reuse, DWIM=dwim,$
                    DISPERSEDTRACK=dispersedbins, WHITETRACK=whitetrack,$
                    SHOW_TRACES=show_traces, THRESHOLD=threshold	)
end
