;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
; $Id: xmidispvisi_lambda.pro,v 1.4 2003/11/08 06:44:19 koehler Exp $
;
; xmidispvisi_lambda.pro
; Created:     Sat Jul 19 14:49:22 2003 by Koehler@rommie
; Last change: Thu Feb  5 03:45:44 2004
;
; PURPOSE:
;	save the world
;
; CALLING SEQUENCE:
;
;
; INPUT:
;
;
; OUTPUT:
;
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi_lambda_event, ev
  WIDGET_CONTROL, ev.TOP, GET_UVALUE=st
  st.me->Lambda_event, ev, st
end

PRO xmidispvisi::lambda_event, ev, st
  WIDGET_CONTROL, ev.ID, GET_VALUE=v
  case (ev.ID) OF
      st.start: self.la_start= v
      st.ende:  self.la_end  = v
      st.step:  self.la_step = v
      st.width: self.la_width= v
      st.quit:  WIDGET_CONTROL, ev.TOP, /DESTROY
  endcase
  if (ev.ID eq st.start or ev.ID eq st.ende or ev.ID eq st.step or ev.ID eq st.width) then $
    if (ev.type eq 0) then begin
      if (ev.ch eq 10B) then begin
                                ; someone hit return
          wset, st.draw
          self->draw_lambda_window, st.mdata1, st.sz, 40+st.sz[1], 30,/LABINS
          self->draw_lambda_window, st.mdata2, st.sz, 40         , 30,/LABINS
      endif
  endif
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::draw_lambda_window, data, sz, x0,y0, LABINS=labins

  dsz = size(data)
  tvscl, rebin(sqrt(data),sz[1],sz[2],/sample), x0,y0
  plot, [0,dsz[1]], [0,dsz[2]], /nodata,$
    position=[ x0,y0, x0+sz[1],y0+sz[2]], yticklen=1e-9,/device,/noerase

  if keyword_set(labins) then $
    for x= self.la_start, self.la_end, self.la_step do $
      oplot, [x,x,x+self.la_width,x+self.la_width], [0,dsz[2],dsz[2],0], color=255

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::lambda_gui

  rmsdata = Cached_MeanRMS(self.fringefile)
  maskdata= oirgetdata(self.maskfile)

  factor= 4
  sz= size(rmsdata.data1) * factor

  base = Widget_base(Title="Midivisi - Lambda bins "+self.objectname,$
                     Space=0, /Column, /Base_Align_Left)

  draw  = Widget_Draw(base, XSIZE=60+sz[1], YSIZE=80+sz[2]*4)
  bbase = Widget_base(base, /Row)
  startl= Widget_Label(bbase, Value="Start:")
  starte= Widget_Text( bbase, Value= strtrim(string(self.la_start),2),XSize=4,/Editable,/All_Events)
  endl  = Widget_Label(bbase, Value="End:")
  ende  = Widget_Text( bbase, Value= strtrim(string(self.la_end),2),  XSize=4,/Editable,/All_Events)
  stepl = Widget_Label(bbase, Value="Step:")
  stepe = Widget_Text (bbase, Value= Strtrim(string(self.la_step),2), XSize=4,/Editable,/All_Events)
  widthl= Widget_Label(bbase, Value="Width:")
  widthe= Widget_Text (bbase, Value= Strtrim(string(self.la_width),2),XSize=4,/Editable,/All_Events)

  quit = Widget_Button(bbase, Value="Dismiss", /Align_Center, Xsize=100,Ysize=32)

  WIDGET_CONTROL, base, /REALIZE
  WIDGET_CONTROL, draw, GET_VALUE=ID

  wset, ID
  xx = [ 0, sz[1]/factor ]
  yy = [ 0, sz[2]/factor ]
  loadct,0

  self->draw_lambda_window, rmsdata[1].data1, sz, 40, 31+sz[2]
  self->draw_lambda_window, rmsdata[1].data2, sz, 40, 61+sz[2]*3.

  mdata1= rmsdata[1].data1*maskdata[0].data1
  mdata2= rmsdata[1].data2*maskdata[0].data2
  self->draw_lambda_window, mdata1, sz, 40, 30         ,/LABINS
  self->draw_lambda_window, mdata2, sz, 40, 60+sz[2]*2.,/LABINS

  st={ mdata1: mdata1, mdata2: mdata2, sz: sz, $
       me: self, draw: ID, start: starte, ende: ende, step: stepe, width: widthe, quit: quit }

  WIDGET_CONTROL, base, SET_UVALUE=st
  XMANAGER, 'xmidispvisi_lambda', base, /NO_BLOCK
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO xmidispvisi::set_lambda_bins, la_start, la_end, la_step, la_width
  self.la_start= la_start
  self.la_end  = la_end
  self.la_step = la_step
  self.la_width= la_width
end

Function xmidispvisi::get_lambda_bins
  return, [ self.la_start, self.la_end, self.la_step, self.la_width ]
end

PRO xmidispvisi::set_lambda, lambda
  self.lambda = lambda
end

Function xmidispvisi::lambda
  return, self.lambda
end
