;
; This file is part of MIA.
;
; MIA is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; MIA is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with MIA; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
PRO Getuvwa, ha, dec, xyz, uvw, ALT=alt, POSITIONANGLE=pa, para=para
   IF N_ELEMENTS(xyz) EQ 2 THEN xyz = [xyz, 0.] ; <<< In case z not given
   uvw = FLTARR(3);

  lat_vlti  = -0.429838786

  ihar = ha / 57.29577951
  decr = dec / 57.29577951

; compute u,v,w
  cha = cos(ihar)
  sha = sin(ihar)
  cdec = cos (decr)
  sdec = sin (decr)
  clat = cos(lat_vlti)
  slat = sin(lat_vlti)
  uvw[0] = xyz[0]*cha - xyz[1]*slat*sha + xyz[2]*clat*sha
  uvw[1] = xyz[0]*sha*sdec + xyz[1]*(slat*cha*sdec + clat*cdec) - $
           xyz[2]*(clat*cha*sdec - slat*cdec)
  uvw[2] = -xyz[0]*sha*cdec - xyz[1]*(slat*cha*cdec - clat*sdec) + $
           xyz[2]*(clat*cha*cdec + slat*sdec)

  SinAlt =   (clat*cha*cdec + slat*sdec)
  alt = asin(SinAlt) ; <<<  in Radians!
  pa = atan(uvw[1], uvw[0]) ; also in radians
  para = atan(sha, (slat/clat)*cdec-sdec*cha)
RETURN
END

FUNCTION midifitsuvw, fitsfile,para=para,doprint=doprint,alt=alt
   fif = strsplit(fitsfile,' ',/extract)
   f = OBJ_NEW('fitsfile', fif[0])
   fh = f->prihead()
   ra = fh->getpar('RA')
   dec = fh->getpar('DEC')
   lst = fh->getpar('LST')*360./86400. ; degrees
   if (KEYWORD_SET(doprint)) then PRINT,'recorded lst ',fh->getpar('LST')
   ha = lst - ra
   if (ha gt 180.) then ha = ha - 360.
   x = fh->getpar('CONF T1X') - fh->getpar('CONF T2X')
   y = fh->getpar('CONF T1Y') - fh->getpar('CONF T2Y')
   z = fh->getpar('CONF T1Z') - fh->getpar('CONF T2Z')
   xyz = [x,y,z]
   uvw = xyz
   a = 1.
   getuvwa,ha, dec, xyz, uvw,para=para,alt=a
   alt = a*!radeg ; to degrees
   if (KEYWORD_SET(doprint)) then begin
      PRINT,'lst ',lst,' ',lst/15.
      PRINT,'ha ',ha,' ',ha/15.
      PRINT,'xyz ',xyz,sqrt(total(xyz^2)),format='(a,4f10.5)'
      PRINT,'uvw ',uvw,sqrt(total(uvw^2)),sqrt(total(uvw[0:1]^2)),format='(a, 5f10.5)'
      PRINT,'para ',para*!radeg
      PRINT,'alt ',alt
      PRINT,'airmass ',1./sin(alt/!radeg)
   endif
   OBJ_DESTROY,fh
   OBJ_DESTROY,f
   RETURN, uvw
END

PRO midiFieldAngle, file, para, phiN, phiE, rot
;  calculate various rotation angles from header data

   f = strsplit(file,' ',/extract)
   fits = OBJ_NEW('fitsfile', f[0])
   head = fits->head()
   fits->close
   alt = head->getpar('ISS ALT')
   az  = -head->getpar('ISS AZ')
   para = head->getpar('ISS PARANG START')
   phiN = para - alt + az +167.02
   phiE = phiN - 90.
   rot = phiN + 180.

   if (phiN gt 180.) then phiN = phiN - 360.
   if (phiN lt (-180.)) then phiN = phiN + 360.
   if (phiE gt 180.) then phiE = phiE - 360.
   if (phiE lt (-180.)) then phiE = phiE + 360.
   if (rot gt 180.) then rot = rot - 360.
   if (rot lt (-180.)) then rot = rot + 360.
   OBJ_DESTROY,fits
   OBJ_DESTROY,head
RETURN 
END
