/*
 * This file is part of MIA.
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: delay.h,v 1.1 2003/07/24 08:32:00 jaffe Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jaffe 2003-07-24  written
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */
/* $Id: delay.h,v 1.1 2003/07/24 08:32:00 jaffe Exp $ */

#ifndef _DELAY_TABLE_H
#define _DELAY_TABLE_H
/*----------------------------------------------------------------------- */
/* Delay table FITS binary tables routines                                */
/*----------------------------------------------------------------------- */
#include "VLTItable.h"

typedef struct      /* table definition for delay table */
{
  table_def *def;       /* basic table information */
  long   row;           /* last row (0-rel) read or written, -1=> none. */
  char   *date_obs;     /* observing date */
  int    time_col;      /* Time column number (0-rel) */
  int    telescope_col; /* Telescope number column number (0-rel) */
  int    delay_col;     /* delay column number */
} delay_table_def;

typedef struct      /* structure used for jaffe row */
{
  double     time;     /* time (days) */
  int        telescope; /* which telescope */
  double     delay;    /* delay (seconds) */
  table_row *raw;    /* generic table row */
} delay_table_row;

/* public functions */

/* delay_table_row functions */
delay_table_row* make_delay_table_row(delay_table_def* table);
/*----------------------------------------------------------------------- */
/*   Make an delay_table_row structure                           */
/*   Inputs:                                                              */
/*      table     pointer to delay_table table structure         */
/*   Returns:                                                             */
/*      An delay_table_row structure pointer ; NULL => failure   */
/*      Data pointers set to generic structure values                     */
/*----------------------------------------------------------------------- */

void kill_delay_table_row (delay_table_row *row);
/*----------------------------------------------------------------------- */
/*   Delete an delay_table_row structure                         */
/*   Inputs:                                                              */
/*      row    pointer to the delay_table_row structure.         */
/*----------------------------------------------------------------------- */

/* delay_table functions */

delay_table_def* 
make_delay_table_def(fitsfile *fptr, char *date_obs, int *ierr);
/*----------------------------------------------------------------------- */
/*   Make a new delay_table in an open FITS file                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      date_obs  Date (yyyy-mm-dd) observations began                    */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      A delay_table_def pointer; NULL indicates failure        */
/*----------------------------------------------------------------------- */

delay_table_def* 
open_delay_table(fitsfile *fptr, int *ierr);
/*----------------------------------------------------------------------- */
/*   Open an existing delay_table in the active FITS file        */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      An delay_table_def pointer; NULL indicates failure       */
/*----------------------------------------------------------------------- */

int read_delay_table(fitsfile *fptr,  delay_table_def* table, 
			    delay_table_row *row, long row_number,
			    int *ierr);
/*----------------------------------------------------------------------- */
/*   Read the next row from an delay_table                       */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row_number which row (1-rel) in the table                         */   
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      row       pointer to an extant row structure                      */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int write_delay_table(fitsfile *fptr, delay_table_def *table, 
			     delay_table_row *row,  long row_number,
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Write  the next row to an delay_table                               */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      table     table definitions structure from open or create         */
/*      row       pointer to an extant row structure                      */
/*      row_number which row (1-rel) in the table                         */
/*                 -1 => next row                                         */  
/*   Input/output:                                                        */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

int close_delay_table(fitsfile *fptr, delay_table_def* table, 
			     int *ierr);
/*----------------------------------------------------------------------- */
/*   Close an open delay_table, associated structures are        */
/*   destroyed                                                            */
/*   Note: this closes the full FITS file                                 */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Returns:                                                             */
/*      0 if successful, else failed                                      */
/*----------------------------------------------------------------------- */

void kill_delay_table_def (delay_table_def* table);
/*----------------------------------------------------------------------- */
/*   Delete an delay_table structure                             */
/*   Inputs:                                                              */
/*      table     table definitions structure from open or create         */
/*----------------------------------------------------------------------- */

#endif /* end _DELAY_TABLE_H */
/* */
