/*
 * This file is part of MIA.
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
$Id: OPD.c,v 1.3 2001/05/16 16:13:56 jdejong Exp $
#define _POSIX_SOURCE_1
#include "vltPort.h"

/*-----------------------------------------------------------------------*/
/*  Create model OPD series.                                             */
/*  Command line parameters:                                             */
/*   -OPDfile Name of output OPDfile, def OPD.file                       */
/*   -n       Number of samples, def 8192                                */
/*   -dt      Interval in msec  def 10                                   */
/*   -tau0    Atm. decoher. in sec.  def 0.10                            */
/*-----------------------------------------------------------------------*/
#include <math.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "fourg.h"


/* global variables */
char     *OPDfile="OPD.file"; /* output OPD file name */
long     number=8192;         /* number of samples */
int      dt=10;               /* sample interval in msec */
double   tau0 = 0.10;         /* Desired atm. coherence time (sec) */

/* internal prototypes */
void OPDin (int argc, char **argv, int *ierr);
void Usage(void);

int main ( int argc, char **argv )
/*----------------------------------------------------------------------- */
/*   Program to generate a model OPD series                               */
/*----------------------------------------------------------------------- */
{
  int  ierr;

  /*  Startup - parse command line arguments*/
  ierr = 0;
  OPDin (argc, argv, &ierr);
  if (ierr!=0) return ierr;

    
  return ierr;
} /* end of main */

void OPDin (int argc, char **argv, int *ierr)
/*----------------------------------------------------------------------- */
/*  Parse control info from command line */
/*   Input: */
/*      argc   Number of arguments from command line */
/*      argv   Array of strings from command line */
/*   Output: */
/*      ierr       Error code: 0 => ok */
/*----------------------------------------------------------------------- */
{
  int ax;
  char *arg;

  if (argc<=1) Usage(); /* must have arguments */
/* parse command line */
  for (ax=1; ax<argc; ax++) {
    arg = argv[ax];
    if (strcmp(arg, "-OPDfile") == 0) { /* OPD file name */
      OPDfile = argv[++ax];
    } else if (strcmp(arg, "-n") == 0){ /* number of samples */
      sscanf (argv[++ax],"%ld",&number);
    } else if (strcmp(arg, "-dt") == 0){ /* sample interval */
      sscanf (argv[++ax],"%d",&dt);
    } else if (strcmp(arg, "-tau0") == 0){ /* Tau0 */
      sscanf (argv[++ax],"%f",&tau0);
    }     else { /* unknown argument */
      Usage();
    }
  }
  *ierr = 0;
} /* end OPDin */

void Usage(void)
/*----------------------------------------------------------------------- */
/*   Tells about usage of program and bails out */
/*----------------------------------------------------------------------- */
{
    fprintf(stderr, "Usage: OPD <arguments>\n");
    fprintf(stderr, "Generates model atmospheric OPD series\n");
    fprintf(stderr, "Argument:\n");
    fprintf(stderr, "  -OPDfile Name of output OPDfile, def OPD.file\n");
    fprintf(stderr, "  -n       Number of samples, def 8192\n");
    fprintf(stderr, "  -dt      Interval in msec  def 10\n");
    fprintf(stderr, "  -tau0    Atm. decoher. in sec.  def 0.10\n");
    
    /* exit(1);  bail out */
  }/* end Usage */

/* */
