/*
 * This file is part of MIA.
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#define _POSIX_SOURCE_1
#include "vltPort.h"

/* program to filter a time OPD series */
#include <math.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#define NPOINT 300  /* number of points to sum */
double startit(char *file, int nskip, int navg, double last[NPOINT]);
int main ( int argc, char **argv ) {
  int i, indx;
  double sum, inpoi, last[NPOINT], t1, t2;
  char inLine[121], outLine[121], *jerr;
  FILE *inFile, *outFile;
  char *infile="series2", *outfile = "filter.out"; /* files */
  int npoints=NPOINT; /* number of points to average */
  int nskip=3;        /* number of header lines */

  /* initial running sum */
  sum = startit(infile, nskip, npoints, last);
  indx = 0;
  inpoi = 1.0 / npoints;

  /* open files */
  inFile = fopen (infile, "rt");
  if (!inFile) {
    fprintf (stderr,"ERROR opening file %s\n", infile);
  }
  outFile = fopen (outfile, "wt");
  if (!inFile) {
    fprintf (stderr,"ERROR opening file %s\n", outfile);
  }

  /* copy first lines */
  for (i=0; i<nskip; i++) {
    jerr = fgets (inLine, 120, inFile);
    fputs (inLine, outFile);

  }
  /* read and sum */
  i = 0;
  while(++i) { /* loop til eof */
    jerr = fgets (inLine, 120, inFile);
    if (!jerr) break; /* EOF or error? */
    sscanf (inLine, "%lf %lf", &t1, &t2);
    /* correct running sum once past initial part */
    if (i>npoints) {
      sum += t2;
      sum -= last[indx];
      last[indx] = t2;
      indx++;
      if (indx >=npoints) indx = 0; /* circular buffer */
    }
    t2 = t2 - sum * inpoi; /* subtract running mean */
    /* format and write output */
    sprintf (outLine, "%10.5lf %10.5lf  ! %10.5lf\n", 
	     t1, t2, sum * inpoi);
    fputs (outLine, outFile);
  }
  fclose (inFile);
  fclose (outFile);

} /* end main */

/* get sum of first nsum points */
double startit(char *infile, int nskip, int nsum, 
	       double last[NPOINT]) {
  FILE *inFile;
  char inLine[121], *jerr;
  double out=0.0;
  double t1, t2;
  int i;

  inFile = fopen (infile, "rt");
  if (!inFile) {
    fprintf (stderr,"ERROR opening file %s\n", infile);
  }
  /* skip first lines */
  for (i=0; i<nskip; i++) {
    jerr = fgets (inLine, 120, inFile);
  }

  /* read and sum */
  for (i=0; i<nsum; i++) {
    jerr = fgets (inLine, 120, inFile);
    sscanf (inLine, "%lf %lf", &t1, &t2);
    out += t2;
    last[i] = t2;
  }

  fclose (inFile);

  return out;
} /* end startit */
/* */
