/*
 * This file is part of MIA.
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: makeID.c,v 1.4 2002/06/04 09:29:29 jdejong Exp $ */
#define _POSIX_SOURCE_1
#include "vltPort.h"

/*-----------------------------------------------------------------------*/
/*  This program generates a FITS file containing an Imaging Detector    */
/* table as defined for Optical/IR interferometer data sets.             */ 
/* Command line arguments:                                               */
/* -input  the name of the input parameter file, def. makeID.in          */
/* -output the name of the output FITS tablefile, def. makeID.fit        */
/*-----------------------------------------------------------------------*/
#define SIMULATEMAIN /* this contains the main for a data makeIDator */
#include "imaging_detector.h"
#include "parser.h"

/* global variables, possibly read from command line */
char     *input_file="makeID.in";      /* input parameter file name */
char     *output_file="!makeID.fit";    /* output FITS file name */
/* global variables possibly read from the input_file */
int nregion = 1;   /* number of windows on the detector */
int XLo[20];       /* start X pixel numbers */
int XHi[20];       /* end X pixel numbers */
int YLo[20];       /* start Y pixel numbers */
int YHi[20];       /* end Y pixel numbers */
char rname[20][17];/* window names */
int rcorr[20];     /* data products */

/* internal prototypes */
void makeIDin (int argc, char **argv, int *ierr);
void Usage(void);
imaging_detector_table_def* make_ID_table (fitsfile *fptr, int *ierr);
/* create parser list */
parse_list* make_parse_list();
/* copy parse_list to global variables */
void parse_list_to_global (parse_list* list);

int main ( int argc, char **argv )
/*----------------------------------------------------------------------- */
/*   Program to write a FITS Imaging Detector binary table                */
/*----------------------------------------------------------------------- */
{
  int  ierr, nx, ny, nreg;
  imaging_detector_table_def* tabledef;
  fitsfile *outfptr=NULL;

  /* Startup - parse command line */
  ierr = 0;
  makeIDin (argc, argv, &ierr);
  if (ierr!=0) return ierr;

  /* Open/create basic FITS output file*/
  fits_create_file(&outfptr, output_file, &ierr);
  if (ierr>0) { /* error check */
    fprintf(stderr,"ERROR %d opening output FITS file %s \n", 
	    ierr, output_file);
    table_print_error_stack();  /* Show any cfitsio error messages */
    return ierr;
  }/* end error test */

  /* Create basic table structures and write IMAGING_DETECTOR table */
  tabledef = make_ID_table(outfptr, &ierr);
  if ((tabledef==NULL) || (ierr>0)) { /* error test */
    fprintf (stderr,"error %d after table create\n",ierr);
    table_print_error_stack();  /* Show any cfitsio error messages */
    return ierr;
  } /* end error test */
  
  /* delete table_def */
  kill_imaging_detector_table_def(tabledef);
  
  /* Close FITS file - this will flush open table buffer */
  fits_close_file (outfptr, &ierr);

   /* Show any cfitsio error messages */
  table_print_error_stack(); 

  return ierr;
} /* end of main */

void makeIDin (int argc, char **argv, int *ierr)
/*----------------------------------------------------------------------- */
/*  Parse control info from command line                                  */
/*   Input:                                                               */
/*      argc   Number of arguments from command line                      */
/*      argv   Array of strings from command line                         */
/*   Output:                                                              */
/*      ierr       Error code: 0 => ok                                    */
/*----------------------------------------------------------------------- */
{
  int ax;
  char *arg;
  parse_list* list;

  /* command line arguments */
  /*debug    fprintf(stderr, "number of arguments = %d\n",argc);*/
  if (argc<=1) Usage(); /* must have arguments */
  /* parse command line */
  for (ax=1; ax<argc; ax++) {
    arg = argv[ax];
    /*debug    fprintf(stderr, "arg = %s\n",arg);*/
    if (strcmp(arg, "-input") == 0){ /* input parameters */
      input_file = argv[++ax];
    } else if (strcmp(arg, "-output") == 0){ /* output parameters */
      output_file = argv[++ax];
    } else { /* unknown argument */
      Usage();
    }
    }
  
  /* must specify file */
  if (!input_file) {Usage(); exit(1);}

  /* parse input file */
  /* create/init parse_list */
  list = make_parse_list();

  /* parse input file, bail out on error */
  if (parse_input_file(input_file, list)) exit(1);

  /* copy to input_info */
  parse_list_to_global (list);

  /* debug 
     return;*/

  /* delete parser list */
  kill_parse_list(list);

  *ierr = 0;
} /* end makeIDin */

void Usage(void)
/*----------------------------------------------------------------------- */
/*   Tells about usage of program and bails out                           */
/*----------------------------------------------------------------------- */
{
    fprintf(stderr, "Usage: makeID -input file\n");
    fprintf(stderr, "Create Imaging Detector binary FITS table\n");
    fprintf(stderr, "Arguments:\n");
    fprintf(stderr, "  -input input parameter file, def makeID.in\n");
    fprintf(stderr, "  -output output FITS file, def makeID.fit\n");
    
    /*/exit(1);  bail out */
  }/* end Usage */

  parse_list* make_parse_list() {
/*----------------------------------------------------------------------- */
/*  create parser list for makeID                                         */
/*----------------------------------------------------------------------- */
  parse_list *out = create_parse_list();
  int i;
  char name[10], value[50];

  /* add parser items */
  /* number of windows */
  out->list[(out->NumItem)++] = 
    make_parse_item ("nregion", PINTEGER, 0.0, 1, NULL);

  /* for each possible window */
  for (i=1; i<=20; i++) {
    /* XLo? */
    sprintf (name, "XLo%d", i);  
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PINTEGER, 0.0, 1, NULL);
    /* XHi? */
    sprintf (name, "XHi%d", i);  
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PINTEGER, 0.0, 1, NULL);
    /* YLo? */
    sprintf (name, "YLo%d", i);  
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PINTEGER, 0.0, 1, NULL);
    /* YHi? */
    sprintf (name, "YHi%d", i);  
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PINTEGER, 0.0, 1, NULL);
    /* name? */
    sprintf (name, "name%d", i); 
    sprintf (value, "Window%d", i);
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PSTRING, 0.0, 0, value);
    /* data product */
    sprintf (name, "corr%d", i);  
    out->list[(out->NumItem)++] = 
      make_parse_item (name, PINTEGER, 0.0, 2, NULL);
  } /* end loop over windows */

  return out;
} /* end make_parse_list */

/* copy parse_list to global variables */
void parse_list_to_global (parse_list* list) {
/*----------------------------------------------------------------------- */
/* copy parse_list to global variables                                    */
/*----------------------------------------------------------------------- */
  long indx;
  int i;
  char name[10];

  /* number of windows */
  indx = find_parse_entry("nregion", list);
  if (indx>=0) nregion = list->list[indx]->ivalue;

  /* for each possible window */
  for (i=1; i<=20; i++) {
    /* XLo? */
    sprintf (name, "XLo%d", i); 
    indx = find_parse_entry(name, list);
    if (indx>=0) XLo[i-1] = list->list[indx]->ivalue;
    /* XHi? */
    sprintf (name, "XHi%d", i); 
    indx = find_parse_entry(name, list);
    if (indx>=0) XHi[i-1] = list->list[indx]->ivalue;
    /* YLo? */
    sprintf (name, "YLo%d", i); 
    indx = find_parse_entry(name, list);
    if (indx>=0) YLo[i-1] = list->list[indx]->ivalue;
    /* YHi? */
    sprintf (name, "YHi%d", i); 
    indx = find_parse_entry(name, list);
    if (indx>=0) YHi[i-1] = list->list[indx]->ivalue;
    /* name? */
    sprintf (name, "name%d", i); 
    indx = find_parse_entry(name, list);
    if (indx>=0) strcpy (rname[i-1], list->list[indx]->cvalue);
    /* data product */
    sprintf (name, "corr%d", i);  
    indx = find_parse_entry(name, list);
    if (indx>=0) rcorr[i-1] = list->list[indx]->ivalue;
  } /* end loop over windows */
} /* end  parse_list_to_global */

imaging_detector_table_def* make_ID_table (fitsfile *fptr, int *ierr)
/*----------------------------------------------------------------------- */
/*   Create FITS file and initialize table                                */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*       imaging_data_table_def structure                                 */
/*       NULL on failure                                                  */
/*----------------------------------------------------------------------- */
{
  imaging_detector_table_def *detector;
  int ireg;

  if (*ierr>0) return NULL; /* existing error condition? */

  /* make simple imaging_detector_table_def */
  detector = make_testing_detector_table_def(1, 1, nregion);
  if (detector==NULL) { /* error check */
    *ierr = 10000;
    return NULL;
  } /* end error test */

  /* modify imaging_detector_table_def */
  detector->nregion = nregion;

  for (ireg=0; ireg<nregion; ireg++) { /* per region */
     detector->gain[ireg] = 1.0;
     detector->corner[ireg][0] = XLo[ireg];
     detector->corner[ireg][1] = YLo[ireg];
     detector->naxis[ireg][0] = XHi[ireg] - XLo[ireg] + 1;
     detector->naxis[ireg][1] = YHi[ireg] - YLo[ireg] + 1;
     detector->correlation[ireg] = rcorr[ireg];
     detector->regname[ireg] = 
       (char*)malloc(strlen(rname[ireg])+1);;  
     strcpy (detector->regname[ireg], rname[ireg]);
  } /* end loop over regions */

  /* write imaging_detector table  */
  write_imaging_detector_table(fptr, detector, ierr);

  return detector;
} /* end make_ID_table */

/* */
