/*
 * This file is part of MIA.
 *
 * MIA is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include "oirSupport.h"
#define TINY 1.e-20

void minrtsLudCmp(float *a, int n, int *indx, float *d) {

   int i, imax,j,k;
   float big,dum,sum,temp;
   float *vv;

   vv = (float *)malloc(n*sizeof(float));

   *d=1.;

   for (i=0;i<n;i++) {
      big = 0.0;
      for (j=0;j<n;j++) {
         temp = fabs(a[n*i+j]);
         if (temp > big) big = temp;
      }
      vv[i] = 1./big;
   }

   for (j=0; j<n;j++){
      for (i=0;i<j;i++) {
         sum = a[n*i+j];
         for (k=0;k<i;k++) sum -= a[i*n+k]*a[k*n+j];
         a[i*n+j] = sum;
      }
      big = 0.0;
      for (i=j;i<n;i++) {
         sum = a[n*i+j];
         for (k=0;k<j;k++) sum -= a[n*i+k] * a[n*k+j];
         a[n*i+j] = sum;
         dum = vv[i]*fabs(sum);
         if (dum >=big) {
            big = dum;
            imax = i;
         }
      }
      if (j != imax) {
         for (k=0;k<n;k++) {
            dum=a[n*imax+k];
            a[n*imax+k] = a[n*j+k];
            a[n*j+k] = dum;
         }
         *d = -(*d);
         vv[imax] = vv[j];
      }indx[j] = imax;
      if (a[n*j+j] == 0.0) a[n*j+j] = TINY;

      if (j != (n-1)) {
         dum = 1./(a[j*n+j]);
         for (i=j+1;i<n;i++) a[n*i+j] *= dum;
      }
   }
   free(vv);
   return;
}

void minrtsLubKsb(float *a, int n, int *indx, float *b) {
   int i,ii,ip,j;
   float sum;

   ii = -1;
   for (i=0;i<n;i++) {
      ip = indx[i];
      sum = b[ip];
      b[ip] = b[i];
      if (ii >= 0)
         for (j=ii;j<=i-1;j++) sum -= a[n*i+j]*b[j];
      else if (sum) ii=i;
      b[i] = sum;
   }
   for (i=n-1;i>=0;i--) {
      sum = b[i];
      for (j=i+1;j<n;j++) sum -=a[n*i+j]*b[j];
      b[i] = sum/a[n*i+i];
   }
   return;
}


void minrtsPolyFit(int order, int nData, float *xCoord, float *data, 
   float *weight, float *coeff) {

   float *matrix, *vector;
   float d, pj, pk; 
   int *indx;
   int i,j,k;

   matrix = (float *)malloc((1+order)*(1+order)*sizeof(float));
   vector = (float *)malloc((1+order)*sizeof(float));
   indx = (int *)malloc((1+order)*sizeof(float));
   memset(matrix, 0, (1+order)*(1+order)*sizeof(float));
   memset(vector, 0, (1+order)*sizeof(float));

   for (i=0; i<nData; i++) {
      for (j=0;j<=order; j++) {
         if (j == 0) pj = 1; else pj = pow(xCoord[i], j);
         vector[j] += data[i]*weight[i]*pj;
         for (k=0;k<=order; k++) {
            if (k == 0) pk = 1; else pk = pow(xCoord[i], k);
            matrix[k+(1+order)*j] += weight[i] * pj * pk;
         }
      }
   }
   minrtsLudCmp(matrix, order+1, indx, &d);
   minrtsLubKsb(matrix, order+1, indx, vector);
   for (i=0;i<=order;i++) coeff[i] = vector[i];

   free(matrix);
   free(vector);
   free(indx);
   return;
   }


/***************************************************************
 * Split a blank-separated set of file names into separate character strings
 * INPUTS:
 *    char *string     blank-separated list
 * OUTPUTS:
 *    int    nString      number of strings found
 * RETURNS
 *    char **   array of output file names; these have been malloced and
 *              should be freed by calling program killFileList
 **************************************************************** */
char** oirStrSplit (char *string, int *nString) {
   char *blank = " ";
   char *oldString;
   int   nS, iS ;
   char *tok;
   char *stringCopy;
   char **output;

   *nString = 0;
/*            copy input string                */
   stringCopy = (char *)malloc(sizeof(char) * (1+strlen(string)));
   strcpy (stringCopy, string);
   oldString = stringCopy;
/*          find pointer to first blank delimited string   */
   tok = strtok(oldString, blank);
   if (tok == NULL) return NULL;
   oldString = NULL;
/*          get pointers to the rest of the sub strings */
   for (nS = 0; tok != NULL; nS++) {
      tok = strtok(oldString, blank);
   }
   *nString = nS;
/*          malloc output arrays and copy strings */
   output = (char **)malloc(nS * sizeof(char *));
   strcpy (stringCopy, string);
   oldString = stringCopy;
   blank = " ";
   for (iS=0; iS<nS; iS++) {
      tok = strtok(oldString, blank);
      oldString = NULL;
      output[iS] = (char *)malloc(sizeof(char) * (1+strlen(tok)));
      strcpy(output[iS], tok);
   }
   free(oldString);
   return output;
} /* end of oirStrSplit */ 

/***************************************************************
 * kill space associated with array of file names
 * INPUTS:
 *    char **string     array of file names
 *    int    nFiles     number of file names
 **************************************************************** */
void oirKillFileList(char **fileList, int nFiles) {
   int iFile;
   for (iFile=0; iFile<nFiles;iFile++) {free(fileList[iFile]);}
   free(fileList);
   return;
} /* end of oirKillFileList */

void oirGetMaskImages(char *maskFileName, float *maskImages[], 
   int nRegion, int nxList[], int nyList[], int *ierr){

   fitsfile *maskFile;
   imaging_detector_table_def *DetectorDef;
   imaging_fdata_table_def *idata;
   imaging_fdata_row *inData, *inData1;

   char maskType[72];
   int image;
   int i, ix, iy, npix, nx;
   int iRegion;
   float x, sigma;

   fits_open_file (&maskFile, maskFileName, READONLY, ierr);
   if (*ierr != 0) {
      printf("fits openMaskfile of %s gave error %i",maskFileName, *ierr);
      printf("%s \n",strerror(errno));
      return;
   }
   i = fits_read_key(maskFile, TSTRING, "MASKTYPE", maskType, 0, ierr);
   if (*ierr != 0) image = 1; else {
      if(strstr(maskType, "IMAGE") != 0) image = 1;
         else image = 0;
      }
   *ierr = 0;
   DetectorDef = read_imaging_detector_table(maskFile, ierr);
   if (*ierr !=0) {
      printf("error %i opening detector table in %s\n",
         *ierr, maskFileName);
      return ;
   }

   idata = open_imaging_fdata_table(maskFile, ierr);
   if (*ierr !=0) {
      printf("error %i opening imaging data in file %s\n",
         *ierr, maskFileName);
      return;
   }
   inData = make_imaging_fdata_row(idata);
   if (image ==1 ) {   /* mask is an image */
      i = read_imaging_fdata_table(maskFile, idata, inData, 1, ierr);
/*
           check if dimensions agree
*/
      for (iRegion=0; iRegion<nRegion;iRegion++) {
         if(DetectorDef->naxis[iRegion][0] != nxList[iRegion] ||
            (DetectorDef->naxis[iRegion][1] != nyList[iRegion]) ) {
            printf("mask dimensions of region %i dont agree with data dimensions\n",
               iRegion);
            *ierr = 1;
            return;
         }
/*
           allocate array and copy data
*/
         npix = nxList[iRegion]*nyList[iRegion];
         maskImages[iRegion] = (float *)malloc(npix*sizeof(float));
         for (i=0;i<npix;i++) maskImages[iRegion][i] = inData->data[iRegion][i];
      }
   } else {   /* mask is a bunch of gaussian components */
      inData1 = make_imaging_fdata_row(idata);
      i = read_imaging_fdata_table(maskFile, idata, inData,  1, ierr); /* AData */
      i = read_imaging_fdata_table(maskFile, idata, inData1, 2, ierr); /* BData */
/*
           check if dimensions agree
*/
      for (iRegion=0; iRegion<nRegion;iRegion++) {
         if(DetectorDef->naxis[iRegion][0] != nxList[iRegion]) {
            printf("mask dimensions of region %i dont agree with data dimensions\n",
               iRegion);
            *ierr = 1;
            return;
         }
/*
           allocate array 
*/
         nx = nxList[iRegion];
         npix = nx*nyList[iRegion];
         maskImages[iRegion] = (float *)malloc(npix*sizeof(float));
/*
           average A & B and compute gaussian components
*/
         for (ix=0;ix<nx;ix++) {
            for (iy=0;iy<3;iy++) inData->data[iRegion][ix+nx*iy] =
               0.5*(inData->data[iRegion][ix+nx*iy]+inData1->data[iRegion][ix+nx*iy]);

            for (iy=0;iy<nyList[iRegion];iy++) {
               x = inData->data[iRegion][ix] - (iy+1);
               sigma = inData->data[iRegion][ix+2*nx];
               x = 0.5*x*x/(sigma*sigma);
               if (x>10) x=10.;
               maskImages[iRegion][ix + nx*iy] = inData->data[iRegion][ix+nx]*exp(-x);
            } /* end of iy loop */
         }
      }
      kill_imaging_fdata_row(inData1);
   } /* end of image/gauss if else */
    i= close_imaging_fdata_table(maskFile, idata, ierr);
    kill_imaging_fdata_row(inData);
   return;
} /* end of oirGetMaskImages */

